#pragma warning(disable: 4100 4201 4514)

#include <windows.h>

#include "BT848.h"
#include "tvichw32.h"


const unsigned short BROOKTREE = 0x109E;

const unsigned short BT848 = 0x0350;
const unsigned short BT849 = 0x0351;
const unsigned short BT878 = 0x036E;
const unsigned short BT879 = 0x036F;


CBT848::CBT848()
{
   nAdresseBT = FindeAdresse(BROOKTREE, BT848);
   if(nAdresseBT == 0)
   {
      nAdresseBT = FindeAdresse(BROOKTREE, BT849);
      if(nAdresseBT == 0)
      {
         nAdresseBT = FindeAdresse(BROOKTREE, BT878);
         if(nAdresseBT == 0)
         {
            nAdresseBT = FindeAdresse(BROOKTREE, BT879);
         }
      }
   }

   if(BTGefunden() == true)
   {
      if(OpenTVicHW32() == FALSE)
      {
         nAdresseBT = 0;
         return;
      }
   }
}


CBT848::~CBT848()
{
   if(BTGefunden() == true)
   {
      CloseTVicHW32();
      nAdresseBT = 0;
   }
}


bool CBT848::BTGefunden()
{
   if(nAdresseBT != 0)
      return true;

   return false;
}


unsigned int CBT848::FindeAdresse
(
   unsigned short nVendorID,
   unsigned short nDeviceID
)
{
   // Wenn WINDOWS NT luft, darf der Assemblercode nicht ausgefhrt 
   // werden sonst gibt es eins auf die Finger
   OSVERSIONINFO  sOSVersion;

   sOSVersion.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
   GetVersionEx(&sOSVersion);

   // Es hat nicht sollen sein
   if(sOSVersion.dwPlatformId == VER_PLATFORM_WIN32_NT)
     return 0;

   // Adresse durchs PCI-BIOS ermitteln lassen
   unsigned int nAdresse = 0;

   _asm
   {
      push ax;          // Register sichern. In der Onlinehilfe steht man braucht
      push bx;          // das nicht. Wenn man es aber nicht tut lufts nicht.
      push cx;
      push dx;
      push si;

      mov cx,nDeviceID; // DeviceID setzen
      mov dx,nVendorID; // VendorID setzen
      mov si,0;         // Index
      mov ah,0xb1;      // PCI_FUNCTION_ID
      mov al,0x02;      // Finde PCI Device
      int 0x1a;         // BIOS Int 1Ah aufrufen 
                        // Ist der Aufruf erfolgreich steht in BH die Busnummer
                        // und in BL die Devicenummer
      jc  ErrorBT;      // Wenn das CARRY FLAG gesetzt ist, ist ein Fehler aufgetreten
      cmp ah,00;
      jnz ErrorBT;      // Wenn AH null ist, ist ein Fehler aufgetreten

      mov di,0x10;      // Register PCI_BASE_ADDRESS_0
      mov ah,0xb1;      // PCI_FUNCTION_ID
      mov al,0x0a;      // Lese PCI Konfiguration DWORD
      int 0x1a;         // BIOS Int 1Ah aufrufen

      jc  ErrorBT;      // Wenn das CARRY FLAG gesetzt ist, ist ein Fehler aufgetreten
      cmp ah,00;
      jnz ErrorBT;      // Wenn AH null ist, ist ein Fehler aufgetreten

      mov nAdresse,ecx; // Ist der Aufruf erfolgreich steht in ECX die Adresse

      ErrorBT:          // Hierher, wenn ein Fehler auftritt
      pop si;           // Register wieder holen
      pop dx;
      pop cx;
      pop bx;
      pop ax;
   }

   // Adresse zurckgeben. Wenn es nicht geklappt hat ist die Adresse 0
	 return nAdresse & 0xfffff000;
}


void CBT848::InitBT()
{
   // Register optimieren
   WriteBT(E_VSCALE_HI, 0x00);
   WriteBT(O_VSCALE_HI, 0x00);
   //const byte SCLOOP =(1<<7||1<<6||1<<5||0<<4||0<<3||0<<2||0<<1||0);
   //const byte SCLOOP=0x00;
   WriteBT(E_SCLOOP,    0x00);
   WriteBT(O_SCLOOP,    0x00);
   WriteBT(SAT_U_LO,    0xb4);//  Chrominance U
   WriteBT(SAT_V_LO,    0xb4);//  Chrominance V
   
   // Test by Xetra
   //WriteBT(CONTRAST_LO, 0x00);
   //WriteBT(BRIGHT, 0xff);
   //const byte CROP=3;//0,1,2,3
   //WriteBT(E_CROP, 0xff);//CROP<<6||CROP<<4||CROP<<2||CROP);
   //WriteBT(O_CROP, 0xff);//CROP<<6||CROP<<4||CROP<<2||CROP);
   //WriteBT(TEST, 0x0);
   /*const bool LNOTCH=0;//0=enable
   const bool COMP=0;//0=composite 1=Y/C
   const bool LDEC=0;//0=enable 		          		  	      Y     U     V
   const byte CONTROL =(LNOTCH<<7||COMP<<6||LDEC<<5||0<<4||0<<3||0<<2||0<<1||0);
   WriteBT(E_CONTROL,    CONTROL);
   WriteBT(O_CONTROL,    0xff);//CONTROL);*/
}


void CBT848::ResetBT()
{
   // Ausgangseinstellung wieder herstellen
   //WriteBT(SRESET,      0xff); // Xetra
   WriteBT(E_VSCALE_HI, 0x60);
   WriteBT(O_VSCALE_HI, 0x60);
   WriteBT(E_SCLOOP,    0x40);
   WriteBT(O_SCLOOP,    0x40);
   WriteBT(SAT_U_LO,    0xfe);
   WriteBT(SAT_V_LO,    0xb4);
}


bool CBT848::ReadBT
(
   int            nRegister,
   unsigned int  *nValue
)
{
   if(BTGefunden() == false)
      return false;

   if(IsDriverOpened() == FALSE)
      return false;

   unsigned int *pPhysical = (unsigned int*)MapPhysToLinear(nAdresseBT+nRegister, 0x04);
   *nValue = *pPhysical;

   return true;
}


bool CBT848::WriteBT
(
   int            nRegister,
   unsigned int   nValue
)
{
   if(BTGefunden() == false)
      return false;

   if(IsDriverOpened() == FALSE)
      return false;

   unsigned int *pPhysical = (unsigned int*)MapPhysToLinear(nAdresseBT+nRegister, 0x04);
   *pPhysical = nValue;

   return true;
}
