#pragma warning(disable: 4100 4201 4514)

#include <windows.h>
#include <stdio.h>
#include <vfw.h>
#include <pbt.h>

#ifdef DIRECTX
#include <ddraw.h>
#endif

#include "Decode.h"
#include "StartDialog.h"
#include "BT848.h"
#include "resource.h"


#ifdef DIRECTX
LPDIRECTDRAW         pDirectDrawOld          = NULL;
LPDIRECTDRAW4        pDirectDraw             = NULL;
LPDIRECTDRAWSURFACE4 pDirectDrawSurface      = NULL;
LPDIRECTDRAWSURFACE4 pDirectDrawOverlay      = NULL;
#else
DWORD                dwDestination           = 0;
BYTE *               pDestination            = NULL;
BYTE *               pDestinationFinal       = NULL;

BITMAPINFO          *pDIB                    = NULL;
#endif

DWORD                dwFirstLines            = 0;
LPVOID               pFirstLines             = NULL;

CSyster				  *pSyster                 = NULL;
//CBT848              *pBT848                  = NULL;
CStartDialog        *pStartDialog            = NULL;

const char           pAppTitle[]             = "MoreTV";
const char           pBndDateiGerade[]       = "MoreTVEven.bnd";
const char           pBndDateiUngerade[]     = "MoreTVOdd.bnd";
const char           pStatuszeile[]          = "%02dfps  'F1'=Start 'F2'=Decodeur 'ESC'=Pause  'F10'=Fin";

HINSTANCE            hInstance;
HWND                 hWndMain                = NULL;
HWND                 hWndCap                 = NULL;

const int            VIDEO_HEIGHT            = 576;
//const int            VIDEO_BITCOUNT          =  16;
const int            VIDEO_BITCOUNT          =  12;
//const DWORD          VIDEO_FORMAT            = MAKEFOURCC('U','Y','V','Y');
//const DWORD          VIDEO_FORMAT            = MAKEFOURCC('Y','U','Y','2');
const DWORD          VIDEO_FORMATY2            = MAKEFOURCC('Y','U','Y','2');
const DWORD          VIDEO_FORMAT            = MAKEFOURCC('Y','V','1','2');
//const DWORD          VIDEO_FORMAT            = MAKEFOURCC5('R','G','B','2','4');

//const int            SCREEN_BITCOUNT         =  16;
const DWORD          OVERLAY_FORMAT          = MAKEFOURCC('Y','V','1','2');

bool                 bDecode                 = true;
bool                 bCapture                = false;
bool                 bReady                  = false;

// Reihenfolge der Zeilen
int                  nLineOrder[ZEILENZAHL];

// Anzahl der Videoquellen
const int            MAXVIDDRIVERS           =  10;

extern short YU12toRGB[256*4*4];

bool ReadBND(const char*);
int  MessageBoxID(UINT, UINT);
int  VidcapEnumerateDrivers(void);
bool VidcapInitHardware(HWND, HWND, UINT);

#ifdef DIRECTX
bool OpenDirectX(HWND);
void CloseDirectX(void);
bool EraseSurface();
#else
void Layout(HWND);
#endif


LRESULT CALLBACK MainWnd(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK VideoStreamCallback(HWND, LPVIDEOHDR);


int WINAPI WinMain
(
   HINSTANCE   hCurrInstance,
   HINSTANCE   hPrevInstance,
   LPSTR       lpszCmdLine, 
   int         nCmdShow
)
{
   hInstance = hCurrInstance;

   /*pBT848 = new CBT848();
	if(pBT848 == NULL)
		return false;*/

   pStartDialog = new CStartDialog();
	if(pStartDialog == NULL)
		return false;

   if(pStartDialog->DoModal(hInstance, 0) != IDOK)
		return false;

   if(pStartDialog->PalPlus())
   {
      if(ReadBND(pBndDateiGerade) == false)
         return false;
   }
   else
   {
      if(ReadBND(pBndDateiUngerade) == false)
         return false;
   }

   if(pStartDialog->Secam())
   {
      // Hier wird entschieden welches Bildschirmformat benutzt wird
      switch(pStartDialog->PixelMode())
      {
         case PIXELMODE_RGB15:
   	      pSyster = new CRGBSysterSecam(pStartDialog->TestZeilen(), GRAFIKMODE_RGB15);
         break;

         case PIXELMODE_RGB16:
   	      pSyster = new CRGBSysterSecam(pStartDialog->TestZeilen(), GRAFIKMODE_RGB16);
         break;

         case PIXELMODE_RGB24:
   	     // pSyster = new CRGBSysterSecam(pStartDialog->TestZeilen(), GRAFIKMODE_RGB24);
         break;

         case PIXELMODE_YUY2:
            pSyster = new CYVUSysterSecam(pStartDialog->TestZeilen());
         break;
      }
   }
   else
   {
      //pour le scam donner un peu de temps pour le calcul du dcodage son
      SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
      SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);

      // Hier wird entschieden welches Bildschirmformat benutzt wird
      switch(pStartDialog->PixelMode())
      {
         case PIXELMODE_RGB15:
   	      pSyster = new CRGBSyster(pStartDialog->TestZeilen(), GRAFIKMODE_RGB15);
         break;

         case PIXELMODE_RGB16:
   	      //pSyster = new CRGBSyster(pStartDialog->TestZeilen(), GRAFIKMODE_RGB16);
   	      pSyster = new CRGBSysterSecam(pStartDialog->TestZeilen(), GRAFIKMODE_RGB16);
         break;
/*
         case PIXELMODE_RGB24:
   	      pSyster = new CRGBSysterSecam(pStartDialog->TestZeilen(), GRAFIKMODE_RGB24);
         break;*/

         case PIXELMODE_YUY2:
            pSyster = new CYVUSyster(pStartDialog->TestZeilen());
         break;
      }
   }

	if(pSyster == NULL)
		return false;

   if(pSyster->FehlerStatus() == true)
		return false;

   // Windowsklasse Registrieren
   WNDCLASS sWinClass;

   sWinClass.lpszClassName = pAppTitle;
   sWinClass.hInstance     = hInstance;
   sWinClass.lpfnWndProc   = MainWnd;
   sWinClass.hCursor       = NULL;
   sWinClass.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_FRED));
   sWinClass.lpszMenuName  = NULL;
   sWinClass.hbrBackground = GetStockObject(BLACK_BRUSH);
   sWinClass.style         = CS_HREDRAW | CS_VREDRAW;
   sWinClass.cbClsExtra    = 0;
   sWinClass.cbWndExtra    = 0;

   if(!RegisterClass(&sWinClass)) 
   {
      MessageBoxID(IDS_ERR_REGISTER_CLASS, MB_ICONEXCLAMATION);
      return false;
   }

   HACCEL hAccel = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDA_MORETV));
   if(hAccel == NULL)
      return false;

   hWndMain = 
   CreateWindow( pAppTitle,
                 pAppTitle,
                 WS_VISIBLE | WS_OVERLAPPED | WS_SYSMENU | WS_CLIPCHILDREN,
                 CW_USEDEFAULT,
                 CW_USEDEFAULT,
                 pStartDialog->VideoWidth(),
                 pStartDialog->ZeilenAusgabe(),
                 HWND_DESKTOP,
                 NULL,
                 hInstance,
                 NULL
               );

   if(hWndMain == NULL)
   {
      MessageBoxID(IDS_ERR_CREATE_WINDOW, MB_ICONEXCLAMATION | MB_OK);
      return false;
   }

#ifdef DIRECTX
	if(OpenDirectX(hWndMain) == false)
      return false;
#endif

	CAPTUREPARMS sCapParms;

	if(VidcapEnumerateDrivers() == 0)
	{
		MessageBoxID(IDS_ERR_FIND_HARDWARE, MB_ICONEXCLAMATION);
      return false;
	}

	hWndCap = capCreateCaptureWindow(pAppTitle, WS_CHILD, 0, 0, 0, 0, hWndMain, 1);

  if(hWndCap == NULL)
	{
		MessageBoxID(IDS_ERR_CREATE_WINDOW, MB_ICONEXCLAMATION | MB_OK);
      return false;
	}

	// Reserve la carte Graphique pour l'application
	if(VidcapInitHardware(hWndMain, hWndCap, 0) == false)
	{
		MessageBoxID(IDS_ERR_FIND_HARDWARE, MB_ICONEXCLAMATION);
      return false;
	}

	// VideoStreamCallBack serra appel lorsque une image sera capture.
	capSetCallbackOnVideoStream(hWndCap, VideoStreamCallback);

	// Recupere les reglages courant de la capture
	capCaptureGetSetup(hWndCap, &sCapParms, sizeof(CAPTUREPARMS));

	// Modifie quelque reglage pour une meilleur capture (25 images/secondes)
	sCapParms.dwRequestMicroSecPerFrame = 4000;
	sCapParms.fMakeUserHitOKToCapture   = false;
	sCapParms.fAbortLeftMouse           = false;
	sCapParms.fAbortRightMouse          = false;
	sCapParms.fYield                    = true;
	sCapParms.fCaptureAudio             = false;
	//sCapParms.wPercentDropForError      = 10;
	//sCapParms.wNumVideoRequested        = 1;
	sCapParms.wNumVideoRequested        = 3;
	sCapParms.wNumAudioRequested        = 0;
	//sCapParms.fUsingDOSMemory           = false;
	//sCapParms.fStepCaptureAt2x          = false;
	//sCapParms.wStepCaptureAverageFrames = 5;

	// On rergle ces mmes paramtres
	capCaptureSetSetup(hWndCap, &sCapParms, sizeof(CAPTUREPARMS));

#ifndef DIRECTX
	// Video Puffer
	dwDestination = pStartDialog->VideoWidth() * pStartDialog->ZeilenAusgabe() * 3;
	//dwDestination = 1024 * 768 * 3;
	//dwDestination = pStartDialog->VideoWidth() * pStartDialog->ZeilenAusgabe() * VIDEO_BITCOUNT / 8;
  //dwDestination = 400 * 300 * 2;

	//pDestination = new char[dwDestination];
	pDestination =  (BYTE *) malloc (dwDestination);
  pDestinationFinal =  (BYTE *) malloc (dwDestination);
	if(pDestination == NULL)
      return false;

	if(pDestinationFinal == NULL)
      return false;
#endif

   // Buffer fr die letzten 32 Zeilen festlegen
   if(pStartDialog->PalPlus())
   {
      dwFirstLines = pStartDialog->VideoWidth() * 32 * VIDEO_BITCOUNT / 8;

	   pFirstLines = new char[dwFirstLines];
	   if(pFirstLines == NULL)
         return false;
   }

	//dfinir le format de saisie (input)
	BITMAPINFO *pBitmapInfo = new BITMAPINFO;
   if(pBitmapInfo == NULL)
      return false;

	//capGetVideoFormat(hWndCap, pBitmapInfo, sizeof(BITMAPINFOHEADER)); 
	pBitmapInfo->bmiHeader.biSize          = sizeof(BITMAPINFOHEADER);
	pBitmapInfo->bmiHeader.biWidth         = pStartDialog->VideoWidth();
	//pBitmapInfo->bmiHeader.biHeight        = VIDEO_HEIGHT;
	pBitmapInfo->bmiHeader.biHeight        = pStartDialog->ZeilenAusgabe();
	//pBitmapInfo->bmiHeader.biHeight        = 287;
	pBitmapInfo->bmiHeader.biBitCount      = VIDEO_BITCOUNT;
	//pBitmapInfo->bmiHeader.biBitCount      = 12;
	pBitmapInfo->bmiHeader.biCompression   = VIDEO_FORMAT;
	//pBitmapInfo->bmiHeader.biCompression   = BI_RGB;
	pBitmapInfo->bmiHeader.biPlanes        = 1;
	pBitmapInfo->bmiHeader.biSizeImage     = pBitmapInfo->bmiHeader.biWidth    *
														  pBitmapInfo->bmiHeader.biHeight   *
														  pBitmapInfo->bmiHeader.biBitCount / 8;
	pBitmapInfo->bmiHeader.biClrUsed       = 0;
	pBitmapInfo->bmiHeader.biXPelsPerMeter = 0;
	pBitmapInfo->bmiHeader.biYPelsPerMeter = 0;
	pBitmapInfo->bmiHeader.biClrImportant  = 0;

	// Regle le format d'entree (input) par rapport aux infos de pBitmapInfo
	BOOL e=capSetVideoFormat(hWndCap, pBitmapInfo, sizeof(BITMAPINFO));
	delete pBitmapInfo; 

#ifndef DIRECTX
	Layout(hWndMain);

	pDIB = new BITMAPINFO;
	if(pDIB == NULL)
      return false;

	pDIB->bmiHeader.biSize          = sizeof(pDIB->bmiHeader);
	pDIB->bmiHeader.biWidth         = pStartDialog->VideoWidth();
	pDIB->bmiHeader.biHeight        = pStartDialog->ZeilenAusgabe();
	//pDIB->bmiHeader.biHeight        = 287;
	pDIB->bmiHeader.biPlanes        = 1;
	//pDIB->bmiHeader.biBitCount      = VIDEO_BITCOUNT;
	pDIB->bmiHeader.biBitCount      = 16;
	pDIB->bmiHeader.biCompression   = BI_RGB;
	//pDIB->bmiHeader.biCompression   = VIDEO_FORMAT;
	/*pDIB->bmiHeader.biSizeImage     = pDIB->bmiHeader.biWidth    *
												 pDIB->bmiHeader.biHeight   *
												 pDIB->bmiHeader.biBitCount / 8;*/
	/*IB->bmiHeader.biSizeImage     = pDIB->bmiHeader.biWidth    *
												 pDIB->bmiHeader.biHeight   * 3;*/
	pDIB->bmiHeader.biSizeImage     = 0;
	pDIB->bmiHeader.biXPelsPerMeter = 0;
	pDIB->bmiHeader.biYPelsPerMeter = 0;
	pDIB->bmiHeader.biClrUsed       = 0;
	pDIB->bmiHeader.biClrImportant  = 0;
	/*IB->bmiColors[0].rgbBlue = 200; 
  pDIB->bmiColors[0].rgbGreen = 200; 
  pDIB->bmiColors[0].rgbRed = 200; 
	pDIB->bmiColors[1].rgbBlue = 200; 
  pDIB->bmiColors[1].rgbGreen = 200; 
  pDIB->bmiColors[1].rgbRed = 200; */
#endif
   
   // Flag setzen, da alle Zeiger initialisiert sind
   bReady = true;

   // Videostream Starten
   SendMessage(hWndMain, WM_COMMAND, MAKEWPARAM(IDM_START, 0), 0);

   MSG sMsg;
   while(GetMessage(&sMsg, NULL, 0, 0)) 
   {
      if(! TranslateAccelerator(hWndMain, hAccel, &sMsg)) 
      {     
         TranslateMessage(&sMsg);
         DispatchMessage(&sMsg);
      }
   }

	if(pSyster != NULL)
		delete pSyster;

   /*if(pBT848 != NULL)
      delete pBT848;*/

   if(pStartDialog != NULL)
		delete pStartDialog;

   return sMsg.wParam;
}


int VidcapEnumerateDrivers()
{
   char  strDeviceName[80];
   char  strDeviceVersion[80];
   short nIndex;
   short nDriverCount = 0;

   for(nIndex=0; nIndex<MAXVIDDRIVERS; nIndex++) 
   {
      if(capGetDriverDescription( nIndex,
                                  (LPSTR)strDeviceName,
                                  sizeof(strDeviceName),
                                  (LPSTR)strDeviceVersion, 
                                  sizeof(strDeviceVersion)
                                )
        ) 
      {
         nDriverCount++;
      }
   }		 

   return nDriverCount;
}


bool VidcapInitHardware
(
   HWND  hWnd,
   HWND  hWndCap,
   UINT  uIndex
)
{
   if(!capDriverConnect(hWndCap, uIndex)) 
      return false;

   capOverlay(hWndCap, true);
   capPreview(hWndCap, false);

   return true;
}


#ifndef DIRECTX
void Layout
(
   HWND  hWnd
)
{
   int   nXBorder;
   int   nYBorder;
   int   nYMenuAndCaption;
   RECT  sClientRect;
   RECT  sWindowRect;

   nYBorder = GetSystemMetrics(SM_CYBORDER);
   nXBorder = GetSystemMetrics(SM_CXBORDER);

   GetWindowRect(hWnd, &sWindowRect);
   GetClientRect(hWnd, &sClientRect);

   ClientToScreen(hWnd, (LPPOINT)&sClientRect);
   nYMenuAndCaption = sClientRect.top - sWindowRect.top + 1;

   MoveWindow( hWnd,
               sWindowRect.left,
               sWindowRect.top,
               pStartDialog->VideoWidth() + nXBorder*2,
               pStartDialog->ZeilenAusgabe() + nYBorder*2 + nYMenuAndCaption,
               TRUE
             );
}
#endif


int MessageBoxID
(
	UINT	uString, 
	UINT	uStyle
)
{
    char cMessage[400];

    LoadString(hInstance, uString, cMessage, sizeof(cMessage));
    return MessageBox(NULL, cMessage, pAppTitle, uStyle);
}


#ifdef DIRECTX
LRESULT CALLBACK VideoStreamCallback
(
   HWND        hWnd,
   LPVIDEOHDR  pVideoHeader
)
{
   int            nIndex;
	DDSURFACEDESC2 sDirectDrawSurfaceDesc;

   if(bReady == false)
      return false;

   bCapture = true;

   // Array mit Sortierindex
	int pSortLines[ZEILENZAHL];

   // Array mit den Anfangsadressen der Zeilen. Es wird nur ein Halbbild verarbeitet
	int *pLinesAddress[ZEILENZAHL];

   if(bDecode == true)
   {
      if(pStartDialog->PalPlus())
      {
         // Adressen der Zeilen im 1. Halbbild bestimmen
	      for(nIndex=0; nIndex<32; nIndex++)
	         pLinesAddress[nIndex] = (int*)pFirstLines+nIndex*pStartDialog->VideoWidth()/2;

	      for(nIndex=32; nIndex<ZEILENZAHL; nIndex++)
	         pLinesAddress[nIndex] = (int*)pVideoHeader->lpData+nLineOrder[nIndex]*pStartDialog->VideoWidth()/2;
      }
      else
      {
         // Adressen der Zeilen im 2. Halbbild bestimmen
	      for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
	         pLinesAddress[nIndex] = (int*)pVideoHeader->lpData+nLineOrder[nIndex]*pStartDialog->VideoWidth()/2;
      }

	   // Sortierung finden
	   pSyster->FindLineOrder(pLinesAddress, pSortLines, pStartDialog->VideoWidth(), pStartDialog->PalPlus());

	   // Die fehlende Zeile durch Zeile davor ersetzen
      for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
      {
         if(pSortLines[nIndex] == 32)
         {
            pSortLines[nIndex] = pSortLines[nIndex-1];
            break;
         }
      }
   }
   else
   {
      // Adressen der Zeilen bestimmen
	   for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
		   pLinesAddress[nIndex] = ((int*)pVideoHeader->lpData+nIndex*pStartDialog->VideoWidth());

	   for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
         pSortLines[nIndex] = nIndex;
   }

   if(pStartDialog->PixelMode() == PIXELMODE_YUY2)
   {
      // Overlay Surface Locken
	   ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
      sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
	   if(pDirectDrawOverlay->Lock(NULL, &sDirectDrawSurfaceDesc, 0, 0) != DD_OK)
      {
         bCapture = false;
		   return false;
      }
   }
   else
   {
      // Surface Locken
	   ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
      sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
	   if(pDirectDrawSurface->Lock(NULL, &sDirectDrawSurfaceDesc, 0, 0) != DD_OK)
      {
         bCapture = false;
		   return false;
      }
   }

   // Start der nchsten Zeile abfragen
   int nStartOfNextLine;

   if(sDirectDrawSurfaceDesc.dwFlags & DDSD_PITCH)
      nStartOfNextLine = sDirectDrawSurfaceDesc.lPitch;
   else
      nStartOfNextLine = sDirectDrawSurfaceDesc.dwWidth;

   pSyster->DecodeColor( pLinesAddress,
                         pSortLines,
                         (int*)sDirectDrawSurfaceDesc.lpSurface,
								 pStartDialog->VideoWidth(),
                         nStartOfNextLine,
                         pStartDialog->ZeilenAusgabe(),
                         pStartDialog->GetErsteZeile(),
                         pStartDialog->GetLetzteZeile(),
                         pStartDialog->PalPlus()
                       );

   if(pStartDialog->PixelMode() == PIXELMODE_YUY2)
	   pDirectDrawOverlay->Unlock(NULL);
   else
      pDirectDrawSurface->Unlock(NULL);

   if(pStartDialog->PalPlus())
   {
      // Die ersten 32 Zeilen fr das nchste Bild retten
      for(nIndex=0; nIndex<32; nIndex++)
         CopyMemory((short*)pFirstLines+nIndex*pStartDialog->VideoWidth(), (int*)pVideoHeader->lpData+nLineOrder[nIndex]*pStartDialog->VideoWidth()/2, pStartDialog->VideoWidth()*2);
   }

		// Auto Format by Xetra
	    if (pStartDialog->GetDetectFormat())
		{	int nLigne=pSyster->FindBlackLine( pLinesAddress,pStartDialog->VideoWidth());
			pStartDialog->SetDetectFormat(false);
			EraseSurface();
			pStartDialog->SetErsteZeile(nLigne);
		    pStartDialog->SetLetzteZeile(ZEILENZAHL-nLigne);
		}

   if(pStartDialog->Statistic() == true)
   {
      static DWORD   dwLastCount = 0;
      static int     nFrame = 0;

      // FPS berechnen und Statuszeile und ausgeben
      nFrame++;

      if(dwLastCount+1000 <= GetTickCount())
      {
		 HDC  hDC;
         char pText[80];
         RECT sRect;
	     		 		 
		 dwLastCount = GetTickCount();
         wsprintf(pText, pStatuszeile, nFrame);
         nFrame = 0;

         if(pDirectDrawSurface->GetDC(&hDC) != DD_OK)
         {
            bCapture = false;
		      return false;
         }
         
         // Hintergrund mit schwarz fllen
         SetRect(&sRect, 0, pStartDialog->ScreenHeight()-16, pStartDialog->ScreenWidth(), pStartDialog->ScreenHeight()-1);
         FillRect(hDC, &sRect, GetStockObject(BLACK_BRUSH));

         // Status ausgeben
         SetBkMode(hDC, TRANSPARENT);
         SetTextColor(hDC, RGB(255, 255, 255));
         TextOut(hDC, 0, pStartDialog->ScreenHeight()-16, pText, strlen(pText));

         if(pDirectDrawSurface->ReleaseDC(hDC) != DD_OK)
         {
            bCapture = false;
		      return false;
         }
      }
   }

   bCapture = false;
   return true;
}

#else

BYTE CouperValeur(BYTE Valeur, BYTE Decalage)
{
	return ((Valeur & (0x3 << Decalage)) >> Decalage); 
}

LRESULT CALLBACK VideoStreamCallback
(
   HWND        hWnd,
   LPVIDEOHDR  pVideoHeader
)
{
   HDC hDC;
   int nIndex;

   if(bReady == false)
      return false;

   bCapture = true;

   hDC = GetDC(hWndMain);

   // Array mit Sortierindex
	int pSortLines[ZEILENZAHL];

   // Array mit den Anfangsadressen der Zeilen. Es wird nur ein Halbbild verarbeitet
	int *pLinesAddress[ZEILENZAHL];

   if(bDecode == true)
   {
		 //MessageBox (NULL, "ER", NULL, MB_OK );
/*      if(pStartDialog->PalPlus())
      {
         // Adressen der Zeilen im 1. Halbbild bestimmen
	      for(nIndex=0; nIndex<32; nIndex++)
	         pLinesAddress[nIndex] = (int*)pFirstLines+nIndex*pStartDialog->VideoWidth()/2;

	      for(nIndex=32; nIndex<ZEILENZAHL; nIndex++)
	         pLinesAddress[nIndex] = (int*)pVideoHeader->lpData+nLineOrder[nIndex]*pStartDialog->VideoWidth()/2;
      }
      else
      {
         // Adressen der Zeilen im 2. Halbbild bestimmen
	      for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
	         pLinesAddress[nIndex] = (int*)pVideoHeader->lpData+nLineOrder[nIndex]*pStartDialog->VideoWidth()/2;
      }

	   // Sortierung finden
	   pSyster->FindLineOrder(pLinesAddress, pSortLines, pStartDialog->VideoWidth(), pStartDialog->PalPlus());

	   // Die fehlende Zeile durch Zeile davor ersetzen
      for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
      {
         if(pSortLines[nIndex] == 32)
         {
            pSortLines[nIndex] = pSortLines[nIndex-1];
            break;
         }
      }*/
   }
   else
   {
      // Adressen der Zeilen bestimmen

		 //MessageBox (NULL, "ER", NULL, MB_OK );
	   /*for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
		   pLinesAddress[nIndex] = ((int*)pVideoHeader->lpData+nIndex*pStartDialog->VideoWidth());
	//	   pLinesAddress[nIndex] = (((char*)pVideoHeader->lpData+nIndex*pStartDialog->VideoWidth()));

	   for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
         pSortLines[nIndex] = nIndex;*/

		 DWORD PE = 0; //+ (pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight) ;
	   DWORD E = pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight * 2;
		 //DWORD E = pDIB->bmiHeader.biWidth * 100;

     for (DWORD P=0; P<=E; P += 2)
		 { 
		   pDestination [P] = (char) pVideoHeader->lpData[PE++];
			 //pDestination [P] = 0;
			 pDestination [P+1] = 0;
		 }

     int Ux, Uy;
		 P = 1;

		 DWORD AZERTY = (pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight);

	   for (Uy = pDIB->bmiHeader.biHeight+(pDIB->bmiHeader.biHeight/2); Uy > pDIB->bmiHeader.biHeight; Uy --)
		 {
       for (Ux = (pDIB->bmiHeader.biWidth/2); Ux > 0; Ux --)
			 {

				 BYTE COUCOU = pVideoHeader->lpData[AZERTY++] >> 4;
         pDestination [P] = COUCOU;
         pDestination [P+2] = COUCOU;
				 P+=(pDIB->bmiHeader.biWidth*2);

         pDestination [P] = COUCOU;
         pDestination [P+2] = COUCOU;
         P-=(pDIB->bmiHeader.biWidth*2); 
				 P+=4;
			 }
			 P += (pDIB->bmiHeader.biWidth*2);
		 }


     P=1;
		 AZERTY = (pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight) + (pDIB->bmiHeader.biWidth * (pDIB->bmiHeader.biHeight/4));

	   for (Uy = pDIB->bmiHeader.biHeight+(pDIB->bmiHeader.biHeight/2); Uy > pDIB->bmiHeader.biHeight; Uy --)
		 {
       for (Ux = (pDIB->bmiHeader.biWidth/2); Ux > 0; Ux --)
			 {

				 BYTE COUCOU = (pVideoHeader->lpData[AZERTY++] >> 4) << 4;
         pDestination [P] |= COUCOU;
         pDestination [P+2] |= COUCOU;
				 P+=(pDIB->bmiHeader.biWidth*2);

         pDestination [P] |= COUCOU;
         pDestination [P+2] |= COUCOU;
         P-=(pDIB->bmiHeader.biWidth*2); 
				 P+=4;
			 }
			 P += (pDIB->bmiHeader.biWidth*2);
		 }

		 PE = 0; //+ (pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight) ;
	   E = pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight * 2;
		 //DWORD E = pDIB->bmiHeader.biWidth * 100;

		 E = pDIB->bmiHeader.biHeight * pDIB->bmiHeader.biWidth * 2;
     for (P=0; P<=E; P += 2)
		 { 
       unsigned short B = pDestination [P] + (pDestination [P+1] << 8);
			 //unsigned short B = A & 0xFF;
			 pDestination [P] = (BYTE) (YU12toRGB [B] & 0xFF);
			 pDestination [P+1] = (BYTE) ((YU12toRGB [B] & 0xFF00) >> 8);
		 }

		 for (DWORD A=0; A<=pDIB->bmiHeader.biHeight; A++)
		 {
			 memcpy (&pDestinationFinal[A*pDIB->bmiHeader.biWidth*2], 
				 &pDestination[(pDIB->bmiHeader.biHeight-A)*pDIB->bmiHeader.biWidth*2],
				 pDIB->bmiHeader.biWidth*2);
		 }
		 
		 /*     for (DWORD P=0; P<=E;)
		 { 
		   ert = (char) pVideoHeader->lpData[PE];
       pDestination [P++] = (char) YU12toRGB[ert] & 0x00FF;
       pDestination [P++] = (char) ((YU12toRGB[ert] & 0xFF00)>>8);
		   PE --;
		 }

     DWORD Ux = 0;
		 DWORD Uy = pDIB->bmiHeader.biHeight;
	   unsigned char ert;

		 //pSyster->m_pYUV2RGBTabelle;*/
   }
/*
   pSyster->DecodeColor( pLinesAddress,
                         pSortLines,
                         (int*)pDestination,
								 pStartDialog->VideoWidth(),
                         pDIB->bmiHeader.biWidth*2,
                         pStartDialog->ZeilenAusgabe(),
                         pStartDialog->GetErsteZeile(),
                         pStartDialog->GetLetzteZeile(),
                         pStartDialog->PalPlus()
                       );*/
/*
   if(pStartDialog->PalPlus())
   {
      // Die ersten 32 Zeilen fr das nchste Bild retten
      for(nIndex=0; nIndex<32; nIndex++)
         CopyMemory((short*)pFirstLines+nIndex*pStartDialog->VideoWidth(), (int*)pVideoHeader->lpData+nLineOrder[nIndex]*pStartDialog->VideoWidth()/2, pStartDialog->VideoWidth()*2);
   }*/

	/*DWORD PE = (pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight);// + (pDIB->bmiHeader.biWidth * (pDIB->bmiHeader.biHeight/2) );
	DWORD E = (pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight) * 2;
	char ert;

	for (DWORD P=0; P<=E;)
	{
		ert = (char) pVideoHeader->lpData[PE];
    pDestination [P++] = ert;
    pDestination [P++] = ert;
    //pDestination [P++] = ert;
		PE --;
	}*/
	/*
	DWORD PE = 0; //(pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight);// + (pDIB->bmiHeader.biWidth * (pDIB->bmiHeader.biHeight/2) );
	DWORD E = (pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight) * 3;
	char ert;

	for (DWORD P=0; P<=E;)
	{
		ert = (char) pVideoHeader->lpData[PE];
    pDestination [P++] = ert;
    pDestination [P++] = ert;
    pDestination [P++] = ert;
		PE ++;
	}*/

   //CopyMemory ((char *)pDestination, pVideoHeader->lpData, (pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight));
   //CopyMemory ((char *)&pDestination[pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight], pVideoHeader->lpData, pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight);
   //CopyMemory ((char *)&pDestination[pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight * 2], pVideoHeader->lpData, pDIB->bmiHeader.biWidth * pDIB->bmiHeader.biHeight);

   SetDIBitsToDevice( hDC,
                      0,
                      0,
                      pDIB->bmiHeader.biWidth,
//320,
                      pDIB->bmiHeader.biHeight,
											//240,
                      0,
                      0,
                      0,
                      pDIB->bmiHeader.biHeight,
											//240,
                      pDestinationFinal,
											//pVideoHeader->lpData,
                      pDIB,
											//DIB_PAL_COLORS
                      DIB_RGB_COLORS
                    );
/*
		// Auto Format by Xetra
		if (pStartDialog->GetDetectFormat())
		{	int nLigne=pSyster->FindBlackLine( pLinesAddress,pStartDialog->VideoWidth());
			pStartDialog->SetDetectFormat(false);
			if(pDestination != NULL)
					ZeroMemory(pDestination, dwDestination);
			pStartDialog->SetErsteZeile(nLigne);
		    pStartDialog->SetLetzteZeile(ZEILENZAHL-nLigne);
		}*/
   
   
	 // Affichage Nombre de Frame Seconde ...
   /*if(pStartDialog->Statistic())
   {
      static DWORD   dwLastCount = 0;
      static int     nFrame = 0;

      // FPS berechnen und ausgeben
      nFrame++;

      if(dwLastCount+1000 <= GetTickCount())
      {
         char pText[80];
         dwLastCount = GetTickCount();

         wsprintf(pText, pStatuszeile, nFrame);
			SetWindowText(hWndMain, pText);

         nFrame = 0;
      }
   } */

   ReleaseDC(hWndMain, hDC);

   bCapture = false;

   return true;
}
#endif


// Reihenfolge der Zeilen aus Datei einlesen
bool ReadBND
(
   const char *pBndDatei
)
{
   int   nZlr;
   int   nInputValue;
   FILE *sFile = fopen(pBndDatei, "r");

   if(sFile == NULL)
   {
      MessageBoxID(IDS_ERR_READ_BND, MB_ICONEXCLAMATION);
      return false;
   }

   for(nZlr=0; nZlr<ZEILENZAHL; nZlr++)
   {
      if(fscanf(sFile, "%d\n", &nInputValue) == EOF)
      {
         MessageBoxID(IDS_ERR_READ_BND, MB_ICONEXCLAMATION);
         return false;
      }

      if(nInputValue < 0 || nInputValue >= VIDEO_HEIGHT)
      {
         MessageBoxID(IDS_ERR_READ_BND, MB_ICONEXCLAMATION);
         return false;
      }

      nLineOrder[nZlr] = nInputValue;
   }

   fclose(sFile);
   return true;
}


LRESULT CALLBACK MainWnd
(
   HWND     hWnd,
   UINT     uMessage,
   WPARAM   wParam,
   LPARAM   lParam
)
{
   switch(uMessage) 
   {
      case WM_COMMAND:
         switch(LOWORD(wParam)) 
         {
            case IDM_EXIT:
               DestroyWindow(hWnd);
            break;

            case IDM_BILD_KLEINER:
            {
               int nErsteZeile = pStartDialog->SetErsteZeile(pStartDialog->GetErsteZeile()+1);
               int nLetzteZeile = pStartDialog->SetLetzteZeile(pStartDialog->GetLetzteZeile()-1);

#ifdef DIRECTX
               EraseSurface();
#else
               if(pDestination != NULL)
                  ZeroMemory(pDestination, dwDestination);
#endif
            }
            break;

            case IDM_BILD_GROESSER:
               pStartDialog->SetErsteZeile(pStartDialog->GetErsteZeile()-1);
               pStartDialog->SetLetzteZeile(pStartDialog->GetLetzteZeile()+1);
            break;

            case IDM_FORMAT1:
            {
#ifdef DIRECTX
               EraseSurface();
#else
               if(pDestination != NULL)
                  ZeroMemory(pDestination, dwDestination);
#endif

               pStartDialog->SetErsteZeile(63);
               pStartDialog->SetLetzteZeile(ZEILENZAHL-63);
            }
            break;

            case IDM_FORMAT2:
            {
#ifdef DIRECTX
               EraseSurface();
#else
               if(pDestination != NULL)
                  ZeroMemory(pDestination, dwDestination);
#endif

               pStartDialog->SetErsteZeile(27);
               pStartDialog->SetLetzteZeile(ZEILENZAHL-27);
			}
            break;

            case IDM_FORMAT3:
            {
#ifdef DIRECTX
               EraseSurface();
#else
               if(pDestination != NULL)
                  ZeroMemory(pDestination, dwDestination);
#endif
			   {
			   pStartDialog->SetErsteZeile(0);
               pStartDialog->SetLetzteZeile(ZEILENZAHL);
			   }
			}
            break;

			case IDM_DETECTFORMAT:
            {
				pStartDialog->SetDetectFormat(true);
			}
            break;



            case IDM_BILD_SPEICHERN:
               pStartDialog->FormatSpeichern();
            break;

            case IDM_START:
               // VideoStreaming ohne zu speichern
               capCaptureSequenceNoFile(hWndCap);

               // Register optimieren
             /*  if(pStartDialog->OptimierungBT())
                  pBT848->InitBT();*/

#ifdef DIRECTX
               EraseSurface();
#endif
            break;

            case IDM_DEKODER:
               bDecode = !bDecode;
            break;
         }
      break;

		// Verhindern das das System in den Suspendmode geht
		case WM_POWERBROADCAST:
			if(wParam == PBT_APMQUERYSUSPEND)
				return BROADCAST_QUERY_DENY;
		break;

      case WM_PALETTECHANGED:
      case WM_QUERYNEWPALETTE:
         return SendMessage(hWndCap, uMessage, wParam, lParam);
      break;

      case WM_DESTROY:
      {
         MSG sMsg;

         // Flag setzen, da die Zeiger ungltig sind
         bReady = false;

         // Warten bis letztes Bild bearbeitet wurde
         while(bCapture == true)
         {
            while(PeekMessage(&sMsg, NULL, NULL, NULL, PM_REMOVE))
            {
               TranslateMessage(&sMsg);
               DispatchMessage(&sMsg);
            }
         }

         // Capture beenden falls es noch luft
         capCaptureStop(hWndCap);
         capSetCallbackOnVideoStream(hWndCap, NULL);

         // Disconnect the current capture driver
         capDriverDisconnect(hWndCap);

         // Alles wieder zurcksetzen
        /* if(pStartDialog->OptimierungBT())
            pBT848->ResetBT();*/

#ifdef DIRECTX
         CloseDirectX();
#else
         if(pDestination != NULL)
         {
            delete[] pDestination;
            pDestination = NULL;
         }

         if(pDIB != NULL)
         {
            delete pDIB;
            pDIB = NULL;
         }
#endif
         if( pStartDialog->PalPlus() &&
             pFirstLines != NULL
           )
         {
            delete[] pFirstLines;
            pFirstLines = NULL;
         }

         PostQuitMessage(0);
      }
      break;

      default:
         return DefWindowProc(hWnd, uMessage, wParam, lParam);
   }

   return 0;
}


#ifdef DIRECTX
bool OpenDirectX
(
   HWND hWnd
)
{
	DDSURFACEDESC2 sDirectDrawSurfaceDesc;

	// DirectDraw Object ffnen
	if(DirectDrawCreate(NULL, &pDirectDrawOld, NULL) != DD_OK)
	{
		MessageBoxID(IDS_ERR_INITDIRECTDRAW, MB_ICONEXCLAMATION);
		return false;
	}

   // DirectDraw4 interface ffnen
   if(pDirectDrawOld->QueryInterface(IID_IDirectDraw4, (LPVOID*)&pDirectDraw) != DD_OK)
	{
		MessageBoxID(IDS_ERR_INITDIRECTDRAW4, MB_ICONEXCLAMATION);
		return false;
	}

	// Den Ganzen Bildschirm exklusiv besorgen
	if(pDirectDraw->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN) != DD_OK)
	{
		MessageBoxID(IDS_ERR_SETCOOPERATIVELEVEL, MB_ICONEXCLAMATION);
		return false;
	}

	// Auflsung whlen
	if(pDirectDraw->SetDisplayMode(pStartDialog->ScreenWidth(), pStartDialog->ScreenHeight(), SCREEN_BITCOUNT, 0, 0) != DD_OK)
	{
		MessageBoxID(IDS_ERR_SETDISPLAYMODE, MB_ICONEXCLAMATION);
		return false;
	}

   // Surface erzeugen
   ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
	sDirectDrawSurfaceDesc.dwSize         = sizeof(sDirectDrawSurfaceDesc);
	sDirectDrawSurfaceDesc.dwFlags        = DDSD_CAPS;
	sDirectDrawSurfaceDesc.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
	if(pDirectDraw->CreateSurface(&sDirectDrawSurfaceDesc, &pDirectDrawSurface, NULL) != DD_OK)
	{
		MessageBoxID(IDS_ERR_CREATEPRIMARYSURFACE, MB_ICONEXCLAMATION);
		return false;
	}

   if(pStartDialog->PixelMode() == PIXELMODE_YUY2)
   {
      // Pixelformat fr Overlay
      DDPIXELFORMAT sDirectDrawPixelFormat;

      ZeroMemory(&sDirectDrawPixelFormat, sizeof(sDirectDrawPixelFormat));
      sDirectDrawPixelFormat.dwSize        = sizeof(DDPIXELFORMAT);
      sDirectDrawPixelFormat.dwFlags       = DDPF_FOURCC;
      sDirectDrawPixelFormat.dwFourCC      = OVERLAY_FORMAT;
      sDirectDrawPixelFormat.dwYUVBitCount = SCREEN_BITCOUNT;

      // Overlay erzeugen
      ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
	   sDirectDrawSurfaceDesc.dwSize          = sizeof(sDirectDrawSurfaceDesc);
	   sDirectDrawSurfaceDesc.dwFlags         = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_PIXELFORMAT;
	   sDirectDrawSurfaceDesc.ddsCaps.dwCaps  = DDSCAPS_OVERLAY | DDSCAPS_VIDEOMEMORY | DDSCAPS_LIVEVIDEO;
	   sDirectDrawSurfaceDesc.dwWidth         = pStartDialog->VideoWidth();
	   sDirectDrawSurfaceDesc.dwHeight        = ZEILENZAHL;
	   sDirectDrawSurfaceDesc.ddpfPixelFormat = sDirectDrawPixelFormat;
      if(pDirectDraw->CreateSurface(&sDirectDrawSurfaceDesc, &pDirectDrawOverlay, NULL) != DD_OK)
	   {
		   MessageBoxID(IDS_ERR_CREATEOVERLAYSURFACE, MB_ICONEXCLAMATION);
		   return false;
	   }

      // Overlay aktivieren und positionieren
      RECT  sSourceRect;
      RECT  sDestinationRect;

      sSourceRect.left   = 0;
      sSourceRect.top    = 0;
      sSourceRect.right  = pStartDialog->VideoWidth();
      sSourceRect.bottom = ZEILENZAHL;

      sDestinationRect.left   = 0;
      sDestinationRect.top    = 0;
      sDestinationRect.right  = pStartDialog->ScreenWidth()-1;

	   // Etwas Platz fr die Statistik lassen
	   if(pStartDialog->Statistic() == true)
		   sDestinationRect.bottom = pStartDialog->ScreenHeight()-16;
	   else
		   sDestinationRect.bottom = pStartDialog->ScreenHeight()-1;

      if(pDirectDrawOverlay->UpdateOverlay(&sSourceRect, pDirectDrawSurface, &sDestinationRect, DDOVER_SHOW, NULL) != DD_OK)
	   {
		   MessageBoxID(IDS_ERR_UPDATEOVERLAY, MB_ICONEXCLAMATION);
		   return false;
	   }
   }

   // Surface lschen
   EraseSurface();

   ShowCursor(false);

   return true;
}


void CloseDirectX()
{
   if(pDirectDraw != NULL)
	{
      if( pStartDialog->PixelMode() == PIXELMODE_YUY2 &&
          pDirectDrawOverlay != NULL
        )
      {
         pDirectDrawOverlay->UpdateOverlay(NULL, pDirectDrawSurface, NULL, DDOVER_HIDE, NULL);
			pDirectDrawOverlay->Release();
			pDirectDrawOverlay = NULL;
      }

		if(pDirectDrawSurface != NULL)
		{
			pDirectDrawSurface->Release();
			pDirectDrawSurface = NULL;
		}

		pDirectDraw->Release();
		pDirectDraw = NULL;
	}

   ShowCursor(true);
}


bool EraseSurface()
{
	int            nStartOfNextLine;
	DDSURFACEDESC2 sDirectDrawSurfaceDesc;

   if(pStartDialog->PixelMode() == PIXELMODE_YUY2)
   {
      if(pDirectDrawOverlay == NULL)
		   return false;

	   // Overlay lschen
      ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
	   sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
      if(pDirectDrawOverlay->Lock(NULL, &sDirectDrawSurfaceDesc, 0, 0) != DD_OK)
      {
         MessageBoxID(IDS_ERR_LOCKOVERLAY, MB_ICONEXCLAMATION);
		   return false;
      }

      // Start der nchsten Zeile abfragen
      if(sDirectDrawSurfaceDesc.dwFlags & DDSD_PITCH)
         nStartOfNextLine = sDirectDrawSurfaceDesc.lPitch;
      else
         nStartOfNextLine = sDirectDrawSurfaceDesc.dwWidth;

      for(int nZeile=0; nZeile<pStartDialog->ScreenHeight(); nZeile++)
	      for(int nPixel=0; nPixel<pStartDialog->ScreenWidth()*SCREEN_BITCOUNT/8; nPixel++)
		      *((int*)sDirectDrawSurfaceDesc.lpSurface+(nZeile*nStartOfNextLine+nPixel)/4) = 0x80008000;

      pDirectDrawOverlay->Unlock(NULL);
   }
   else
   {
      if(pDirectDrawSurface == NULL)
		   return false;

	   // Surface lschen
      ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
	   sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
      if(pDirectDrawSurface->Lock(NULL, &sDirectDrawSurfaceDesc, 0, 0) != DD_OK)
      {
         MessageBoxID(IDS_ERR_LOCKSURFACE, MB_ICONEXCLAMATION);
		   return false;
      }

      // Start der nchsten Zeile abfragen
      if(sDirectDrawSurfaceDesc.dwFlags & DDSD_PITCH)
         nStartOfNextLine = sDirectDrawSurfaceDesc.lPitch;
      else
         nStartOfNextLine = sDirectDrawSurfaceDesc.dwWidth;

      for(int nZeile=0; nZeile<pStartDialog->ScreenHeight(); nZeile++)
	      for(int nPixel=0; nPixel<pStartDialog->ScreenWidth()*SCREEN_BITCOUNT/8; nPixel++)
		      *((int*)sDirectDrawSurfaceDesc.lpSurface+(nZeile*nStartOfNextLine+nPixel)/4) = 0;

      pDirectDrawSurface->Unlock(NULL);
   }
   
   return true;
}

#endif