//----------------------------//
// BTChip.c   by:   KennyWood //
//----------------------------//

#include <windows.h>
#include "Docs/Tvichw32.h"	// This *ONLY* provides generic hardware access (don'r worry about its source code)
#include "NagraView.h"
#include "BTChip.h"

//-----------------------------------------------------------------------------
//
// SOME BT CHIP REGISTERS
//
#define E_VSCALE_HI 0x4C
#define O_VSCALE_HI 0xCC
#define E_SCLOOP    0x40
#define O_SCLOOP    0xC0
#define SAT_U_LO    0x34
#define SAT_V_LO    0x38
#define E_VDELAY_LO 0x10
#define O_VDELAY_LO 0x90
#define BRIGHT      0x28
#define CONTRAST    0x30

//-----------------------------------------------------------------------------
//
// SUPPORTED BT CHIP SIGNATURES
//
#define BROOKTREE 0x109E
#define BT848     0x0350
#define BT849     0x0351
#define BT878     0x036E
#define BT879     0x036F

#define SUPPORTEDCHIPS 4
USHORT supportedBT [ SUPPORTEDCHIPS ] = { BT848 , BT849 , BT878 , BT879 } ;

//-----------------------------------------------------------------------------
//
// INTERNAL STATE VARIABLES
//
int  BTAddress = 0x00000000 ;
BOOL haveBT848 = FALSE ;

//-----------------------------------------------------------------------------
//
// BT REGISTERS ACCESS MACROS
//
#define BTChip_Write(reg,val) *(PINT)MapPhysToLinear( BTAddress + (reg) , 0x04 ) = (val)

//-----------------------------------------------------------------------------
//
// BTChip_GetAddress:
//
//  Here we get the BT Chip Linear mem port address.
//
int BTChip_GetAddress ( USHORT man , USHORT dev )
{
	_asm
	{
		pushad ;

		xor		SI,SI ;
		mov		AX,0xB102 ;
		mov		CX,dev ;
		mov		DX,man
		int		0x1A ;
		jc		BTChip_Error ;
		or		AH,AH ;
		jnz		BTChip_Error ;

		mov		DI,0x10 ;
		mov		AX,0xB10A ;
		int		0x1A ;
		jc		BTChip_Error ;
		or		AH,AH ;
		jnz		BTChip_Error ;

		and		ECX,0xFFFFF000 ;
		mov		BTAddress,ECX ;

	BTChip_Error:

		popad ;
	}

	return BTAddress ;
}

//-----------------------------------------------------------------------------
//
// BTChip_Init:
//
//  Find BT Chip address, and init HW access crap.
//
BOOL BTChip_Init ()
{
	int i ;

	for ( i = 0 ; i < SUPPORTEDCHIPS ; i ++ )
		if ( BTChip_GetAddress( BROOKTREE , supportedBT[i] ) )
			break ;

	if ( !BTAddress )			// Have we got any BT Chip ?
		return FALSE ;

	if ( !OpenTVicHW32() )		// Can we access to hardware safely ?
		return FALSE ;

	return IsDriverOpened() ;	// Is the TVicHW32 driver opened ?
}

//-----------------------------------------------------------------------------
//
// BTChip_SetPhaser:
//
//  Sets the proper register for vertical color phasing
//
void BTChip_SetPhaser ( int s , int l )
{
	if ( haveBT848 )
	{
//		Sleep( 500 ) ;   // For sanity
		BTChip_Write( SAT_U_LO    , 0x99 ) ;
		BTChip_Write( SAT_V_LO    , 0x99 ) ;
		BTChip_Write( BRIGHT      , 0x0A ) ;
		BTChip_Write( CONTRAST    , 0xF7 ) ;
//		Sleep( 500 ) ;   // For sanity
		BTChip_Write( E_VSCALE_HI , s ) ;
		BTChip_Write( O_VSCALE_HI , s ) ;
		BTChip_Write( E_SCLOOP    , l ) ;
		BTChip_Write( O_SCLOOP    , l ) ;
		BTChip_Write( E_VDELAY_LO , 0x20 ) ;
		BTChip_Write( O_VDELAY_LO , 0x20 ) ;
//		Sleep( 500 ) ;   // For sanity
	}
}

//-----------------------------------------------------------------------------
//
// BTChip_BugFix_LowQuality:
//
//  Overrides some VFW wrong values in the BT registers.
//
void BTChip_BugFix_LowQuality ()
{
	BTChip_SetPhaser( 0x00 , 0x00 ) ;
}

//-----------------------------------------------------------------------------
//
// BTChip_BugFix_HighQuality:
//
//  Overrides some VFW wrong values in the BT registers.
//
void BTChip_BugFix_HighQuality ()
{
	BTChip_SetPhaser( 0x20 , 0xCC ) ;
}

//-----------------------------------------------------------------------------
//
// BTChip_Restore:
//
//  Resets the old 'wrong' VFW values in the BT registers.
//
void BTChip_Restore ()
{
	if ( haveBT848 )
	{
		BTChip_Write( E_VSCALE_HI , 0x60 ) ;
		BTChip_Write( O_VSCALE_HI , 0x60 ) ;
		BTChip_Write( E_SCLOOP    , 0x40 ) ;
		BTChip_Write( O_SCLOOP    , 0x40 ) ;
		BTChip_Write( SAT_U_LO    , 0xFE ) ;
		BTChip_Write( SAT_V_LO    , 0xB4 ) ;
		BTChip_Write( E_VDELAY_LO , 0x22 ) ;
		BTChip_Write( O_VDELAY_LO , 0x22 ) ;
	}
}

//-----------------------------------------------------------------------------
//
// BTChip_Close:
//
//  Closes everything related with the BT Chip
//
void BTChip_Close ()
{
	if ( haveBT848 )
	{
		BTChip_Restore() ;
		CloseTVicHW32() ;
	}
}
