#pragma warning(disable: 4201 4514)

#include <windows.h>
#include <stdio.h>

#include "BT848.h"
#include "Konfig.h"
#include "Debug.h"
#include "resource.h"


// BT848 Register
const int IFORM         = 0x04;
const int TDEC          = 0x08;
const int E_CROP        = 0x0C;
const int E_VDELAY_LO   = 0x10;
const int E_VACTIVE_LO  = 0x14;
const int E_DELAY_LO    = 0x18;
const int E_HACTIVE_LO  = 0x1C;
const int E_HSCALE_HI   = 0x20;
const int E_HSCALE_LO   = 0x24;
const int BRIGHT        = 0x28;
const int CONTRAST_LO   = 0x30;
const int SAT_U_LO      = 0x34;
const int SAT_V_LO      = 0x38;
const int E_SCLOOP      = 0x40;
const int E_VSCALE_HI   = 0x4C;
const int E_VSCALE_LO   = 0x50;
const int E_VTC         = 0x6C;
const int O_CROP        = 0x8C;
const int O_VDELAY_LO   = 0x90;
const int O_VACTIVE_LO  = 0x94;
const int O_DELAY_LO    = 0x98;
const int O_HACTIVE_LO  = 0x9C;
const int O_HSCALE_HI   = 0xA0;
const int O_HSCALE_LO   = 0xA4;

const int E_CONTROL     = 0x2C;
const int O_CONTROL     = 0xAC;
const int SAT_V_MSB     = 0x01;
const int SAT_U_MSB     = 0x02;
const int CON_MSB       = 0x04;

const int O_VSCALE_HI   = 0xCC;
const int O_SCLOOP      = 0xC0;
const int O_VSCALE_LO   = 0xD0;
const int COLOR_FMT     = 0xD4;
const int CAP_CTL       = 0xDC;
const int O_VTC         = 0xEC;
const int INT_STAT      = 0x100;
const int INT_MASK      = 0x104;
const int GPIO_DMA_CTL  = 0x10C;
const int I2C           = 0x110;
const int RISC_STRT_ADD = 0x114;
const int GPIO_DATA     = 0x200;

// BT848 Bits
const DWORD INT_STAT_I2CDONE = 0x00000100;
const DWORD INT_STAT_RACK    = 0x02000000;

const DWORD I2C_SDA          = 0x00000001;
const DWORD I2C_SCL          = 0x00000002;
const DWORD I2C_W3B          = 0x00000004;
const DWORD I2C_MODE         = 0x00000080;


/*
const int YUY2P         = 0x88;


// RISC Befehle
const int RESYNCEVENTSTART = 0x8C00800C;
const int RESYNCPACKEDDATA = 0x8C008006;
const int RESYNCPLANARDATA = 0x8C00800E;
const int SYNCODDSTART     = 0x8C000004;   // SET RISC STATUS TO X AFTER EVENT END
const int SYNCEVENTSTART   = 0x8C00000C;   // SET RISC STATUS TO Y AFTER ODD END
const int JUMP             = 0x7C000000;
const int WRITE768e        = 0x1CC30300;   // PACKED
const int WRITE768o        = 0x1C3C0300;   // Y8

const int WRITE123Y        = 0x9C000300;   // PLANAR
const int WRITE123UVC      = 0x01800180;   // 4:2:2

const int SKIPY1           = 0xACE10300;
const int SKIPY2           = 0xACD20300;
const int SKIPY3           = 0xACC30300;
const int SKIPY4           = 0xACB40300;
const int SKIPUV           = 0x01800180;


DWORD nProgramm[8192];

char pEvenYBuffer1[287][768];
char pEvenCBBuffer1[287][384];
char pEvenCRBuffer1[287][384];
char pOddYBuffer1[287][768];
char pOddCBBuffer1[287][384];
char pOddCRBuffer1[287][384];
char pEvenYBuffer2[287][768];
char pEvenCBBuffer2[287][384];
char pEvenCRBuffer2[287][384];
char pOddYBuffer2[287][768];
char pOddCBBuffer2[287][384];
char pOddCRBuffer2[287][384];
*/


const char pDllName[] = "hwiodrv.dll";


CBT848::CBT848()
{
   m_bError						 = false;
   m_nKanal						 = 0;
   m_nTuner						 = 0;
   m_nTVCard                = 0;
   m_dwPhysicalAddress		 = 0;
   m_dwMemoryLength			 = 0;
   m_dwMemoryBase				 = 0;
   m_dwIrqNumber				 = 0;
   m_hLibrary					 = NULL;
   m_fnGetHardwareResources = NULL;
   m_fnIsDriverOpened       = NULL;
   m_fnMemoryUnmap          = NULL;
   m_fnMemoryMap            = NULL;
   m_fnMemoryRead           = NULL;
   m_fnMemoryWrite          = NULL;

   //
   // Load our low level hardware access helper library
   //
   UINT nErrorMode = SetErrorMode(SEM_NOOPENFILEERRORBOX);
   m_hLibrary = LoadLibrary(pDllName);
   SetErrorMode(nErrorMode);

   if(m_hLibrary == 0)
   {
      MessageBox(IDS_ERR_LOADDLL);
      m_bError = true;
      return;
   }

   //
   // Get some entry point from our DLL
   //
   m_fnGetHardwareResources = (PPCIGetHardwareResources)GetProcAddress(m_hLibrary, "pciGetHardwareResources");
   m_fnIsDriverOpened       = (PIsDriverOpened)         GetProcAddress(m_hLibrary, "isDriverOpened");
   m_fnMemoryUnmap          = (PMemoryUnmap)            GetProcAddress(m_hLibrary, "memoryUnmap");
   m_fnMemoryRead           = (PMemoryRead)             GetProcAddress(m_hLibrary, "memoryRead");
   m_fnMemoryWrite          = (PMemoryWrite)            GetProcAddress(m_hLibrary, "memoryWrite");
   m_fnMemoryMap            = (PMemoryMap)              GetProcAddress(m_hLibrary, "memoryMap");

   if( m_fnGetHardwareResources == NULL ||
       m_fnIsDriverOpened       == NULL ||
       m_fnMemoryMap            == NULL ||
       m_fnMemoryUnmap          == NULL ||
       m_fnMemoryRead           == NULL ||
       m_fnMemoryWrite          == NULL
     )
   {
      m_bError = true;
      return;
   }

   //
   // OK, the interface DLL is valid, check for our hardware
   //
   if(FindeAdresse() == false)
   {
      m_bError = true;
      return;
   }

   //
   // Hardware found, map the physical hardware address from PCI
   // configuration space to linear memory space in RING 0
   //
   m_dwMemoryBase = m_fnMemoryMap(m_dwPhysicalAddress, m_dwMemoryLength);
   if(m_dwMemoryBase == 0)
   {
      m_bError = true;
      return;
   }
}


CBT848::~CBT848()
{
   if(m_fnMemoryUnmap != NULL && m_dwPhysicalAddress != NULL)
      m_fnMemoryUnmap(m_dwPhysicalAddress, m_dwMemoryLength);

   if(m_hLibrary)
      FreeLibrary(m_hLibrary);
}


bool CBT848::BTGefunden()
{
   if(m_bError == true)
      return false;

   return true;
}


int CBT848::HoleFarbe()
{
   DWORD nCroma;
   DWORD nCromaMSB;

   ReadBT(SAT_U_LO, &nCroma);
   ReadBT(E_CONTROL, &nCromaMSB);

   if(nCromaMSB & SAT_U_MSB)
      nCroma += 256;

   return nCroma;
}


int CBT848::SetzeFarbe
(
   int nCroma
)
{
   DWORD nCromaMSB;

   ReadBT(E_CONTROL, &nCromaMSB);

   if(nCroma < 0)
      nCroma = 0;

   if(nCroma > 511)
      nCroma = 511;

   if(nCroma > 255)
   {
      WriteBT(E_CONTROL, nCromaMSB | SAT_V_MSB | SAT_U_MSB);
      WriteBT(O_CONTROL, nCromaMSB | SAT_V_MSB | SAT_U_MSB);
      WriteBT(SAT_U_LO, nCroma-256);
      WriteBT(SAT_V_LO, nCroma-256);
   }
   else
   {
      WriteBT(E_CONTROL, nCromaMSB & ~(SAT_V_MSB | SAT_U_MSB));
      WriteBT(O_CONTROL, nCromaMSB & ~(SAT_V_MSB | SAT_U_MSB));
      WriteBT(SAT_U_LO, nCroma);
      WriteBT(SAT_V_LO, nCroma);
   }

   return nCroma;
}


int CBT848::HoleHelligkeit()
{
   DWORD nBright;

   ReadBT(BRIGHT, &nBright);

   return ((int)(char)nBright)+128;
}


int CBT848::SetzeHelligkeit
(
   int nBright
)
{
   if(nBright < 0)
      nBright = 0;

   if(nBright > 255)
      nBright = 255;

   WriteBT(BRIGHT, (char)(nBright-128));

   return nBright;
}


int CBT848::HoleKontrast()
{
   DWORD nLuma;
   DWORD nLumaMSB;

   ReadBT(CONTRAST_LO, &nLuma);
   ReadBT(E_CONTROL, &nLumaMSB);

   if(nLumaMSB & CON_MSB)
      nLuma += 256;

   return nLuma;
}


int CBT848::SetzeKontrast
(
   int nLuma
)
{
   DWORD nLumaMSB;

   ReadBT(E_CONTROL, &nLumaMSB);

   if(nLuma < 0)
      nLuma = 0;

   if(nLuma > 511)
      nLuma = 511;

   if(nLuma > 255)
   {
      WriteBT(E_CONTROL, nLumaMSB | CON_MSB);
      WriteBT(O_CONTROL, nLumaMSB | CON_MSB);
      WriteBT(CONTRAST_LO, nLuma-256);
   }
   else
   {
      WriteBT(E_CONTROL, nLumaMSB & ~CON_MSB);
      WriteBT(O_CONTROL, nLumaMSB & ~CON_MSB);
      WriteBT(CONTRAST_LO, nLuma);
   }

   return nLuma;   
}


bool CBT848::FindeAdresse()
{
   const DWORD BT848     = 0x0350;
   const DWORD BT849     = 0x0351;
   const DWORD BT878     = 0x036E;
   const DWORD BT879     = 0x036F;
   const DWORD BROOKTREE = 0x109E;

   if(ERROR_SUCCESS == m_fnGetHardwareResources( BROOKTREE,
                                                 BT848,
                                                 &m_dwPhysicalAddress,
                                                 &m_dwMemoryLength,
                                                 &m_dwIrqNumber
                                               )
     )
      return true;

   if(ERROR_SUCCESS == m_fnGetHardwareResources( BROOKTREE,
                                                 BT849,
                                                 &m_dwPhysicalAddress,
                                                 &m_dwMemoryLength,
                                                 &m_dwIrqNumber
                                               )
     )
      return true;

   if(ERROR_SUCCESS == m_fnGetHardwareResources( BROOKTREE,
                                                 BT878,
                                                 &m_dwPhysicalAddress,
                                                 &m_dwMemoryLength,
                                                 &m_dwIrqNumber
                                               )
     )
      return true;

   if(ERROR_SUCCESS == m_fnGetHardwareResources( BROOKTREE,
                                                 BT879,
                                                 &m_dwPhysicalAddress,
                                                 &m_dwMemoryLength,
                                                 &m_dwIrqNumber
                                               )
     )
      return true;

   return false;
}


void CBT848::InitBTxx()
{
   // Register optimieren
   WriteBT(E_VSCALE_HI, 0x00);
   WriteBT(O_VSCALE_HI, 0x00);
   WriteBT(E_SCLOOP,    0x00);
   WriteBT(O_SCLOOP,    0x00);
   WriteBT(E_VDELAY_LO, 0x20);
   WriteBT(O_VDELAY_LO, 0x20);
}


void CBT848::InitBTxxHQ()
{
   // Register optimieren
   WriteBT(E_VSCALE_HI, 0x20);
   WriteBT(O_VSCALE_HI, 0x20);
   WriteBT(E_SCLOOP,    0xCC);
   WriteBT(O_SCLOOP,    0xCC);
   WriteBT(E_VDELAY_LO, 0x20);
   WriteBT(O_VDELAY_LO, 0x20);

/*
   WriteBT(TDEC,         0x00);
   WriteBT(E_CROP,       0x23);
   WriteBT(O_CROP,       0x23);
   WriteBT(E_VACTIVE_LO, 0x40);
   WriteBT(O_VACTIVE_LO, 0x40);
   WriteBT(E_DELAY_LO,   0xA0);
   WriteBT(O_DELAY_LO,   0xA0);
   WriteBT(E_HACTIVE_LO, 0x00);
   WriteBT(O_HACTIVE_LO, 0x00);
   WriteBT(E_HSCALE_HI,  0x03);
   WriteBT(O_HSCALE_HI,  0x03);
   WriteBT(E_HSCALE_LO,  0x0A);
   WriteBT(O_HSCALE_LO,  0x0A);
   WriteBT(E_VSCALE_LO,  0x00);
   WriteBT(O_VSCALE_LO,  0x00);
   WriteBT(E_VTC,        0x00);
   WriteBT(O_VTC,        0x00);

   // Capture starten
   WriteBT(INT_MASK,      0xF0000000);          // INT MASK
   WriteBT(RISC_STRT_ADD, (DWORD)nProgramm);    // RISC ADRESS
   WriteBT(COLOR_FMT,     YUY2P);               // FIFO ENABLE
   WriteBT(GPIO_DMA_CTL,  0xB1);                // FIFO ENABLE        ! TRIGGER POINTS
   WriteBT(GPIO_DMA_CTL,  0xB3);                // RISC/FIFO ENABLE   ! SET TO 32 DWORDS
   WriteBT(CAP_CTL,       0x03);                // CAPTURE CONTROL
*/
}


void CBT848::ResetBT()
{
/*
   // Capture stopen
   WriteBT(GPIO_DMA_CTL,0x00);        // RISC/FIFO DISABLE
   WriteBT(CAP_CTL,     0x00);        // STOP CAPTURE
   WriteBT(INT_MASK,    0x00);        // INT MASK
*/
   // Ausgangseinstellung wieder herstellen
   WriteBT(E_VSCALE_HI, 0x60);
   WriteBT(O_VSCALE_HI, 0x60);
   WriteBT(E_SCLOOP,    0x40);
   WriteBT(O_SCLOOP,    0x40);
   WriteBT(E_VDELAY_LO, 0x22);
   WriteBT(O_VDELAY_LO, 0x22);
}


void CBT848::BaueProgramm()
{
/*
   int nZlr;
   int nProgrammZlr = 0;

   nProgramm[nProgrammZlr++] = JUMP;
   nProgramm[nProgrammZlr++] = (DWORD)nProgramm;

   nProgramm[nProgrammZlr++] = RESYNCEVENTSTART;
   nProgramm[nProgrammZlr++] = 0;
   nProgramm[nProgrammZlr++] = RESYNCPLANARDATA;
   nProgramm[nProgrammZlr++] = 0;

   // Gerade Zeilen
   for(nZlr=32; nZlr<287; nZlr++)
   {
      nProgramm[nProgrammZlr++] = WRITE123Y;
      nProgramm[nProgrammZlr++] = WRITE123UVC;
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenYBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenCBBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenCRBuffer1[nZlr][0];
   }

   for(nZlr=0; nZlr<32; nZlr++)
   {
      nProgramm[nProgrammZlr++] = WRITE123Y;
      nProgramm[nProgrammZlr++] = WRITE123UVC;
      nProgramm[nProgrammZlr++] = (DWORD)&pOddYBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pOddCBBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pOddCRBuffer1[nZlr][0];
   }

   nProgramm[nProgrammZlr++] = SKIPY1;
   nProgramm[nProgrammZlr++] = SKIPUV;

   nProgramm[nProgrammZlr++] = SYNCODDSTART;
   nProgramm[nProgrammZlr++] = 0;
   nProgramm[nProgrammZlr++] = RESYNCPLANARDATA;
   nProgramm[nProgrammZlr++] = 0;

   // Ungerade Zeilen
   for(nZlr=32; nZlr<287; nZlr++)
   {
      nProgramm[nProgrammZlr++] = WRITE123Y;
      nProgramm[nProgrammZlr++] = WRITE123UVC;
      nProgramm[nProgrammZlr++] = (DWORD)&pOddYBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pOddCBBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pOddCRBuffer1[nZlr][0];
   }

   for(nZlr=0; nZlr<32; nZlr++)
   {
      nProgramm[nProgrammZlr++] = WRITE123Y;
      nProgramm[nProgrammZlr++] = WRITE123UVC;
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenYBuffer2[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenCBBuffer2[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenCRBuffer2[nZlr][0];
   }

   nProgramm[nProgrammZlr++] = SKIPY2;
   nProgramm[nProgrammZlr++] = SKIPUV;
   nProgramm[nProgrammZlr++] = SYNCEVENTSTART;
   nProgramm[nProgrammZlr++] = 0;
   nProgramm[nProgrammZlr++] = RESYNCPLANARDATA;
   nProgramm[nProgrammZlr++] = 0;

   for(nZlr=32; nZlr<287; nZlr++)
   {
      nProgramm[nProgrammZlr++] = WRITE123Y;
      nProgramm[nProgrammZlr++] = WRITE123UVC;
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenYBuffer2[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenCBBuffer2[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenCRBuffer2[nZlr][0];
   }

   for(nZlr=0; nZlr<32; nZlr++)
   {
      nProgramm[nProgrammZlr++] = WRITE123Y;
      nProgramm[nProgrammZlr++] = WRITE123UVC;
      nProgramm[nProgrammZlr++] = (DWORD)&pOddYBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pOddCBBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pOddCRBuffer1[nZlr][0];
   }

   nProgramm[nProgrammZlr++] = SKIPY3;
   nProgramm[nProgrammZlr++] = SKIPUV;
   nProgramm[nProgrammZlr++] = SYNCODDSTART;
   nProgramm[nProgrammZlr++] = 0;
   nProgramm[nProgrammZlr++] = RESYNCPLANARDATA;
   nProgramm[nProgrammZlr++] = 0;

   for(nZlr=32; nZlr<287; nZlr++)
   {
      nProgramm[nProgrammZlr++] = WRITE123Y;
      nProgramm[nProgrammZlr++] = WRITE123UVC;
      nProgramm[nProgrammZlr++] = (DWORD)&pOddYBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pOddCBBuffer1[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pOddCRBuffer1[nZlr][0];
   }

   for(nZlr=0; nZlr<32; nZlr++)
   {
      nProgramm[nProgrammZlr++] = WRITE123Y;
      nProgramm[nProgrammZlr++] = WRITE123UVC;
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenYBuffer2[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenCBBuffer2[nZlr][0];
      nProgramm[nProgrammZlr++] = (DWORD)&pEvenCRBuffer2[nZlr][0];
   }

   nProgramm[nProgrammZlr++] = SKIPY4;
   nProgramm[nProgrammZlr++] = SKIPUV;
   nProgramm[nProgrammZlr++] = SYNCEVENTSTART;
   nProgramm[nProgrammZlr++] = 0;
   nProgramm[nProgrammZlr++] = JUMP;
   nProgramm[nProgrammZlr++] = (DWORD)nProgramm;
*/
}


int CBT848::HoleKanal()
{
   return m_nKanal;
}


//
// Tunersteuerung von TL
//
void CBT848::SetzeKanal
(
   int   nKanal,
   char *pName
)
{
   // Kanalliste
   const struct FREQUENZTAB
   {
      int  nFrequenz;
      char pName[6];
   }
   sFrequenzTab[] =
   {
      {0,     "Video"},

      {4825,  "K02"  }, {5525,  "K03"  }, {6225,  "K04"  },

      {10525, "S01"  }, {11225, "S02"  }, {11925, "S03"  }, {12625, "S04"  }, {13325, "S05"  },
      {14025, "S06"  }, {14725, "S07"  }, {15425, "S08"  }, {16125, "S09"  }, {16825, "S10"  },

      {17525, "K05"  }, {18225, "K06"  }, {18925, "K07"  }, {19625, "K08"  }, {20325, "K09"  },
      {21025, "K10"  }, {21725, "K11"  }, {22425, "K12"  },

      {23125, "S11"  }, {23825, "S12"  }, {24525, "S13"  }, {25225, "S14"  }, {25925, "S15"  },
      {26625, "S16"  }, {27325, "S17"  }, {28025, "S18"  }, {28725, "S19"  }, {29425, "S20"  },
      {30325, "S21"  }, {31125, "S22"  }, {31925, "S23"  }, {32725, "S24"  }, {33525, "S25"  },
      {34325, "S26"  }, {35125, "S27"  }, {35925, "S28"  }, {36725, "S29"  }, {37525, "S30"  },
      {38325, "S31"  }, {39125, "S32"  }, {39925, "S33"  }, {40725, "S34"  }, {41525, "S35"  },
      {42325, "S36"  }, {43125, "S37"  }, {43925, "S38"  }, {44725, "S39"  }, {45525, "S40"  },
      {46325, "S41"  },

      {47125, "K21"  }, {47925, "K22"  }, {48725, "K23"  }, {49525, "K24"  }, {50325, "K25"  },
      {51125, "K26"  }, {51925, "K27"  }, {52725, "K28"  }, {53525, "K29"  }, {54325, "K30"  },
      {55125, "K31"  }, {55925, "K32"  }, {56725, "K33"  }, {57525, "K34"  }, {58325, "K35"  },
      {59125, "K36"  }, {59925, "K37"  }, {60725, "K38"  }, {61525, "K39"  }, {62325, "K40"  },
      {63125, "K41"  }, {63925, "K42"  }, {64725, "K43"  }, {65525, "K44"  }, {66325, "K45"  },
      {67125, "K46"  }, {67925, "K47"  }, {68725, "K48"  }, {69525, "K49"  }, {70325, "K50"  },
      {71125, "K51"  }, {71925, "K52"  }, {72725, "K53"  }, {73525, "K54"  }, {74325, "K55"  },
      {75125, "K56"  }, {75925, "K57"  }, {76725, "K58"  }, {77525, "K59"  }, {78325, "K60"  },
      {79125, "K61"  }, {79925, "K62"  }, {80725, "K63"  }, {81525, "K64"  }, {82325, "K65"  },
      {83125, "K66"  }, {83925, "K67"  }, {84725, "K68"  }, {85525, "K69"  }
   };

   const int KANAL_MAX = (sizeof(sFrequenzTab) / sizeof(FREQUENZTAB));

   // Kartendaten
   const struct KARTE
   {
      char nID;
      int  nVideoMux0; // Tuner
      int  nVideoMux1; // Video 1
      int  nVideoMux2; // Video 2
      int  nVideoMask;
      int  nTonMux0;   // Tuner
      int  nTonMux1;   // Video 1
      int  nTonMux2;   // Video 2
      int  nTonMute;   // Stumm
      int  nTonMask;
   }
   sKarte[] =
   {
      { HAUPPAUGE_848, 0x40, 0x60, 0x20, 0x60, 0x08, 0x0A, 0x09, 0x0C, 0x0F },
      { HAUPPAUGE_878, 0x40, 0x00, 0x20, 0x60, 0x08, 0x0A, 0x09, 0x0C, 0x0F },
      { MIRO_PCTV,     0x40, 0x60, 0x20, 0x60, 0x02, 0x00, 0x00, 0x08, 0x0F }
   };

   const int KARTE_MAX = (sizeof(sKarte) / sizeof(KARTE));

   // Tunerdaten
   const struct TUNER
   {
      char nID;
      int  nAdresse;
      int  nVHFLow;
      int  nVHFHigh;
      int  nUHF;
      int  nThreshold1;
      int  nThreshold2;
   }
   sTuner[] =
   {
      { TEMIC_PAL_B_G,   0xC2, 0x8E02, 0x8E04, 0x8E01, 14025, 46325},
      { TEMIC_PAL_I,     0xC2, 0x8E02, 0x8E04, 0x8E01, 16825, 46325},
      { PHILIPS_PAL_B_G, 0xC2, 0x8EA0, 0x8E90, 0x8E30, 16825, 44725},
      { PHILIPS_PAL_I,   0xC2, 0x8EA0, 0x8E90, 0x8E30, 14025, 46325},
      { PHILIPS_SECAM,   0xC2, 0x8EA7, 0x8E97, 0x8E37, 16825, 44725},
   };

   const int TUNER_MAX = (sizeof(sTuner) / sizeof(TUNER));

   // Name initialisieren
   pName[0] = 0;

   // Karte suchen
   int nKartenType;

   for(nKartenType=0; nKartenType<KARTE_MAX; nKartenType++)
   {
      if(sKarte[nKartenType].nID == m_nTVCard)
         break;
   }

   if(nKartenType >= KARTE_MAX)
      return;

   // Tuner suchen
   int nTunerType;

   for(nTunerType=0; nTunerType<TUNER_MAX; nTunerType++)
   {
      if(sTuner[nTunerType].nID == m_nTuner)
         break;
   }

   if(nTunerType >= TUNER_MAX)
      return;

   // Kanal auswhlen
   m_nKanal = nKanal;

   if(m_nKanal < 0)
      m_nKanal = KANAL_MAX-1;

   if(m_nKanal >= KANAL_MAX)
      m_nKanal = 0;

   // Name bergeben
   lstrcpy(pName, sFrequenzTab[m_nKanal].pName);

   // Kanal holen
   int   nFrequenz;
   DWORD nValue;

   nFrequenz = sFrequenzTab[m_nKanal].nFrequenz;

   // Eingang whlen
   if(nFrequenz == 0)
   {
      // Video
      ReadBT(IFORM, &nValue);
      WriteBT(IFORM, (nValue&~sKarte[nKartenType].nVideoMask)|sKarte[nKartenType].nVideoMux1);

      // Ton von extern
      ReadBT(GPIO_DATA, &nValue);
      WriteBT(GPIO_DATA, (nValue&~sKarte[nKartenType].nTonMask)|sKarte[nKartenType].nTonMux1);
   
      // Fertig
      return;
   }

   // Eingang Tuner
   ReadBT(IFORM, &nValue);
   WriteBT(IFORM, (nValue&~sKarte[nKartenType].nVideoMask)|sKarte[nKartenType].nVideoMux0);

   // Ton vorbergehend aus
   ReadBT(GPIO_DATA, &nValue);
   WriteBT(GPIO_DATA, (nValue&~sKarte[nKartenType].nTonMask)|sKarte[nKartenType].nTonMute);

   // Empfangsband auswhlen
   int nBandSwitch;

   if(nFrequenz <= sTuner[nTunerType].nThreshold1)
   {
      nBandSwitch = sTuner[nTunerType].nVHFLow;
   }
   else
   {
      if(nFrequenz <= sTuner[nTunerType].nThreshold2)
         nBandSwitch = sTuner[nTunerType].nVHFHigh;
      else
         nBandSwitch = sTuner[nTunerType].nUHF;
   }

   // Die Zwischenfrequenz addieren
   nFrequenz += 3890;
   nFrequenz *= 16;
   nFrequenz /= 100;

   // Frequenz an den Tuner schicken
   WriteI2C(sTuner[nTunerType].nAdresse, nFrequenz);

   // BandSwitch an den Tuner schicken
   WriteI2C(sTuner[nTunerType].nAdresse, nBandSwitch);

   // Ton vom Tuner
   ReadBT(GPIO_DATA, &nValue);
   WriteBT(GPIO_DATA, nValue&~(sKarte[nKartenType].nTonMask)|sKarte[nKartenType].nTonMux0);
}


void CBT848::SetzeTuner
(
   int nTuner
)
{
   m_nTuner = nTuner;
}


void CBT848::SetzeTVKarte
(
   int nTVCard
)
{
   m_nTVCard = nTVCard;
}


bool CBT848::ReadBT
(
   int    nRegister,
   DWORD *nValue
)
{
   if(BTGefunden() == false)
      return false;

   *nValue = m_fnMemoryRead(nRegister);

#ifdef _DEBUG
   DebugString("ReadBT Register=%x Data=%x", nRegister, *nValue);
#endif

   return true;
}


bool CBT848::WriteBT
(
   int   nRegister,
   DWORD nValue
)
{
   if(BTGefunden() == false)
      return false;

#ifdef _DEBUG
   DebugString("WriteBT Register=%x Data=%x", nRegister, nValue);
#endif

   m_fnMemoryWrite(nRegister, nValue);

   return true;
}


bool CBT848::ReadI2C
(  
   DWORD  nAdresse,
   DWORD *nData
)
{
   DWORD  nValue;

   // Statusbit lschen
   WriteBT(INT_STAT, INT_STAT_I2CDONE);

   // Readflag setzen
   nAdresse |= 0x01;

   // Adresse senden
   nValue = (nAdresse&0xFF)<<24 | I2C_MODE | I2C_SDA | I2C_SCL;
   WriteBT(I2C, nValue);

   // 50 ms warten bis die Daten bertragen wurden
   Sleep(50);

   // RACK lesen
   ReadBT(INT_STAT, &nValue);
   if((nValue&INT_STAT_RACK) == 0)
      return false;

   // Byte lesen
   ReadBT(I2C, nData);
   *nData = (*nData>>8)&0xFF;

   return true;
}


bool CBT848::WriteI2C
(  
   DWORD nAdresse, 
   DWORD nData
)
{
   // Statusbit lschen
   WriteBT(INT_STAT, INT_STAT_I2CDONE);

   // Drei Byte senden
   DWORD nValue = (nAdresse&0xFF)<<24 | (nData&0xFFFF)<<8 | I2C_MODE | I2C_W3B | I2C_SDA | I2C_SCL;
   WriteBT(I2C, nValue);

   // 50 ms warten bis die Daten bertragen wurden
   Sleep(50);

   return true;
}
