//-----------------------------//
// Decrypt.c   by:   KennyWood //
//-----------------------------//

#include "Decoder.h"
#include "Tables.h"

//-----------------------------------------------------------------------------

#define MAXBLACKLEVEL   64
#define MAXBLACKPERCENT  5

//-----------------------------------------------------------------------------

SHORT     YUVfield [ 288 * 384 ] ;
PSHORT rowYUVField [ 288 ] ;

//-----------------------------------------------------------------------------
//
// Decoder_IsABlackLine:
//
//  Test if an horizontal line is rather black or not.
//
BOOL Decoder_IsABlackLine ( int y , int src_w , int percent )
{
	PUSHORT YUV = rowYUVField[ P[y] ] ;
	int     x , cnt = 0 ;	

	for ( x = 0 ; x < src_w ; x ++ )
	{
		if ( *(PBYTE)YUV ++ > MAXBLACKLEVEL )
			cnt ++ ;
		if ( cnt > percent )
			return FALSE ;
	}

	return TRUE ;
}

//-----------------------------------------------------------------------------
//
// Decoder_AutoFindCinemascope:
//
//  Automatically finds the upper and lower cinemascope's black bands.
//
void Decoder_AutoFindCinemascope ( int src_w , int src_h )
{
	int percent = (src_w * MAXBLACKPERCENT) / 100 ;

	for ( cinemascopeUp = 0 ; cinemascopeUp < MAXCSCOPEUP ; cinemascopeUp ++ )
		if ( !Decoder_IsABlackLine ( cinemascopeUp , src_w , percent ) )
			break ;	
	for ( cinemascopeDown = 0 ; cinemascopeDown < MAXCSCOPEDOWN ; cinemascopeDown ++ )
		if ( !Decoder_IsABlackLine ( HEIGHT-1-cinemascopeDown , src_w , percent ) )
			break ;
}

//-----------------------------------------------------------------------------
//
// Decoder_GetPhase:
//
//  Finds PAL phase.
//  The algorithm implemented here is rather wrong, but it still works.
//
_inline int Decoder_GetPhase ( int y )
{
	int d , p = P[y] ;

	if ( p <= 32 )
		return 4 ;

	d = y - p ;

	if ( (d > 0) || (d <= -32) )
		return 4;

	return (d & 0x03) ;
}

//-----------------------------------------------------------------------------
//
// Decoder_ComposeDataFromField:
//
//  Composes the original frame from the crypted field and its permutation.
//
void Decoder_ComposeDataFromField ( PSHORT pSurf , int src_w , int src_h , int dst_w , int dst_h )
{
	int     x , y , py , phase , lastPhaseOk , decr ;
	PUSHORT surf , lastSurf , YUV , lastYUVOk = NULL ;
	int     Y1 , Y2 , U , V , tU , tV , offset ;
	PINT	Y , UV ;

	// INITIALIZE POINTERS
	if ( dst_h == HEIGHT )
	{
		// BOTTOM to TOP
		surf = pSurf + ((HEIGHT-cinemascopeUp-1) * src_w << interlace) ;		
		decr = (src_w << 1) ;
		if ( interlace )
		{
			decr += src_w ;
			if ( evenFrame )	// For interlaced mode
				surf += src_w ;
		}
	}
	else
	{
		// TOP to BOTTOM
		surf = pSurf + (cinemascopeUp * src_w << interlace) ;
		decr = 0 ;
		if ( interlace )
		{
			decr -= src_w ;
			if ( evenFrame )	// For interlaced mode
				surf += src_w ;
		}
	}

	lastSurf    = surf ;
	lastPhaseOk = 0 ;
	lastYUVOk   = rowYUVField[ P[0] ] ;

	// REPOSITION LINE BY LINE
	for ( y = cinemascopeUp ; y <= HEIGHT-cinemascopeDown-1 ; y ++ )
	{
		// FIX VFW BUG (first interlaced frame loses its first line, so we replicate last one)
		if ( P[y] == 32 )
		{
			memcpy( surf , lastSurf , src_w << 1 ) ;	
			surf += src_w ;
		}
		else
		{
			lastSurf = surf ;
			YUV = rowYUVField[ P[y] ] ;
			if ( color )
			{
				// FULL COLOR
				if ( decrypt && (highQuality || (palSecam == USESECAM)) )
				{
					Y  = (PINT)YUV ;
					UV = (PINT)YUV ;
					x  = src_w >> 1 ;
					if ( !decrypt || 
						 (palSecam == USESECAM) )					
					{
						phase = 312 << 4 ;
					}
					else
					if ( palSecam == USEPAL )
					{
						py = P[y] ;
						if ( py < 32 )
							py -= 0x1A - (internalTopAdjust >> 1) ;
						 offset = ((y & 0x01) << 1) + (py & 0x01) ;
#if DEBUGFILE
	fprintf( debugFile , "Wo:%d " , offset ) ;
#endif
						if ( phaseCorrector == evenFrame )
						{
							offset ^= 0x03 ;
						}
						phase = (((312 + py - y) << 2) + offset) << 2 ;
					}
#if DEBUGFILE
	fprintf( debugFile , "Wi:%d\n" , offset ) ;
#endif
					while ( x -- )
					{
						tU = ((*((PINT)UV   )) >> 10) & 0x3F ;
						tV = ((*((PINT)UV ++)) >> 26) & 0x3F ;
						Y1 = ((*((PINT)Y    )) >>  2) & 0x3F ;
						Y2 = ((*((PINT)Y  ++)) >> 18) & 0x3F ;

						U = (((int)palDemod[ phase + 0 ][tU] +
							  (int)palDemod[ phase + 1 ][tV]) & 0x00000FC0) ;
						V = (((int)palDemod[ phase + 2 ][tU] +
							  (int)palDemod[ phase + 3 ][tV]) & 0x00000FC0) << 6 ;

						*(surf ++) = YUV2RGB[ Y1 | U | V ] ;
						*(surf ++) = YUV2RGB[ Y2 | U | V ] ;						
					}						
				}
				else
				{
					phase = decrypt ? Decoder_GetPhase( y ) : 0 ;
					// DEPENDING ON THE PHASE, WE REPLICATE LAST CHROMINANCE, OR USE A NEW ONE
					if ( (phase == 0) || (phase == 2) )
					{
						lastYUVOk   = YUV ;
						lastPhaseOk = phase ;
					}
					else
						phase = lastPhaseOk ;
					// DO PHASE DEMODULATION
					Y  = (PINT)YUV ;
					UV = (PINT)lastYUVOk ;
					x  = src_w >> 1 ;
					switch ( phase )
					{
					case 0:
						while ( x -- )
						{
							U  = ((*((PINT)UV   )) >>  4) & 0x00000FC0 ;
							V  = ((*((PINT)UV ++)) >> 14) & 0x0003F000 ;
							Y1 = ((*((PINT)Y    )) >>  2) & 0x0000003F ;
							Y2 = ((*((PINT)Y  ++)) >> 18) & 0x0000003F ;

							*(surf ++) = YUV2RGB[ Y1 | U | V ] ;
							*(surf ++) = YUV2RGB[ Y2 | U | V ] ;
						}
						break ;

					case 2:
						while ( x -- )
						{
							U  = (((*((PINT)UV   )) >>  4) & 0x00000FC0) ^ 0x00000FC0 ;
							V  = (((*((PINT)UV ++)) >> 14) & 0x0003F000) ^ 0x0003F000 ;
							Y1 =  ((*((PINT)Y    )) >>  2) & 0x0000003F ;
							Y2 =  ((*((PINT)Y  ++)) >> 18) & 0x0000003F ;

							*(surf ++) = YUV2RGB[ Y1 | U | V ] ;
							*(surf ++) = YUV2RGB[ Y2 | U | V ] ;
						}
						break ;

					default:
						surf += src_w ;
						break ;
					}
				}
			}
			else
			{
				// BLACK / WHITE
				for ( x = src_w ; x > 0 ; x -= 2 )
				{
					*surf ++ = Y2RGB[ *(PBYTE)YUV ++ ] ;
					*surf ++ = Y2RGB[ *(PBYTE)YUV ++ ] ;
				}
			}
		}
		surf -= decr ;
	}
}
