//--------------------------------//
// DirectDraw.c   by:   KennyWood //
//--------------------------------//

#include <ddraw.h>
#include "Decoder.h"
#include "DirectDraw.h"

//-----------------------------------------------------------------------------

LPDIRECTDRAW        lpDD        = NULL ;
LPDIRECTDRAWSURFACE lpDDPrimary = NULL ;
LPDIRECTDRAWSURFACE lpDDFrame   = NULL ;

//-----------------------------------------------------------------------------

BOOL DX_on = FALSE ;
int  W , H ;

//-----------------------------------------------------------------------------
//
// DirectDraw_On:
//
//  Tells us if we are switched or not
//
BOOL DirectDraw_On ()
{
	return DX_on ;
}

//-----------------------------------------------------------------------------
//
// DirectDraw_EraseFrameSurface:
//
//  Erases the frame surface :)
//
BOOL DirectDraw_EraseFrameSurface ( HWND hWnd )
{
	RECT          sRect ;
	DDBLTFX       sDDBltFX ;
	DDSURFACEDESC ddsd ;

	memset( &ddsd , 0 , sizeof(ddsd) ) ;
	ddsd.dwSize = sizeof(ddsd) ;
	if ( IDirectDrawSurface_GetSurfaceDesc( lpDDFrame , &ddsd ) != DD_OK )
	{
		MessageBox( hWnd , "Can't get Frame surface descriptor" , appName , MB_ICONEXCLAMATION ) ;
		return FALSE ;
	}

	SetRect( &sRect , 0 , 0 , ddsd.dwWidth , ddsd.dwHeight ) ;

	memset( &sDDBltFX , 0 , sizeof(sDDBltFX) ) ;
	sDDBltFX.dwSize      = sizeof(sDDBltFX) ;
	sDDBltFX.dwFillColor = 0x00000000 ;
	if ( IDirectDrawSurface_Blt( lpDDFrame , &sRect , NULL , NULL , 
			DDBLT_COLORFILL | DDBLT_WAIT , &sDDBltFX ) != DD_OK )
	{
		MessageBox( hWnd , "Can't set blt rect" , appName , MB_ICONEXCLAMATION ) ;
		return FALSE ;
	}

	return TRUE ;
}

//-----------------------------------------------------------------------------
//
// DirectDraw_Open:
//
//  Open a DirectDraw session and set the desired mode
//
BOOL DirectDraw_Open ( HWND hWnd , int width , int height , int bpp , int src_w , int src_h )
{
	DDSURFACEDESC ddsd ;
	char str [ 256 ] ;

	if ( DirectDraw_On() )
		return TRUE ;

	if ( DirectDrawCreate( NULL , &lpDD , NULL ) != DD_OK )
	{
		MessageBox( hWnd , "Can't init DirectDraw" , appName , MB_ICONEXCLAMATION ) ;
		return FALSE ;
	}

	if ( IDirectDraw_SetCooperativeLevel( lpDD , hWnd , 
			DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN | DDSCL_ALLOWMODEX ) != DD_OK )
	{
		MessageBox( hWnd , "Can't set cooperative level" , appName , MB_ICONEXCLAMATION ) ;
		return FALSE ;
	}

	if ( IDirectDraw_SetDisplayMode( lpDD , width , height , bpp ) != DD_OK )
	{
		sprintf( str , "Can't change to %dx%dx%d display mode" , width , height , bpp ) ;
		MessageBox( hWnd , str , appName , MB_ICONEXCLAMATION ) ;
		return FALSE ;
	}

	memset( &ddsd , 0 , sizeof(ddsd) ) ;
	ddsd.dwSize         = sizeof(ddsd) ;
	ddsd.dwFlags		= DDSD_CAPS ;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_SYSTEMMEMORY ;
	if ( IDirectDraw_CreateSurface( lpDD , &ddsd , &lpDDPrimary , NULL ) != DD_OK )
	{
		MessageBox( hWnd , "Can't create primary surface" , appName , MB_ICONEXCLAMATION ) ;
		return FALSE ;
	}

	memset( &ddsd , 0 , sizeof(ddsd) ) ;
	ddsd.dwSize         = sizeof(ddsd) ;
	ddsd.dwFlags		= DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH ;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY ;
    ddsd.dwWidth        = W = src_w ;
    ddsd.dwHeight       = H = BIFIELDHEIGHT + 1 ;
	if ( IDirectDraw_CreateSurface( lpDD , &ddsd , &lpDDFrame , NULL ) != DD_OK )
	{
		MessageBox( hWnd , "Can't create frame surface" , appName , MB_ICONEXCLAMATION ) ;
		return FALSE ;
	}

	ShowCursor( FALSE ) ;
	
	DX_on = TRUE ;
	return TRUE ;
}

//-----------------------------------------------------------------------------
//
// DirectDraw_ShutDown:
//
//  Close a DirectDraw session and return to GDI mode
//
void DirectDraw_Close ( void )
{
	if ( !DirectDraw_On() )
		return ;

	DX_on = FALSE ;

	if ( lpDDPrimary )
	{
		IDirectDrawSurface_Release( lpDDPrimary ) ;
		lpDDPrimary = NULL ;
	}

	if ( lpDDFrame )
	{
		IDirectDrawSurface_Release( lpDDFrame ) ;
		lpDDFrame = NULL ;
	}

	if ( lpDD )
	{
		IDirectDraw_Release( lpDD ) ;
		lpDD = NULL ;
	}

	ShowCursor( TRUE ) ;
}

//-----------------------------------------------------------------------------
//
// DirectDraw_Lock/Unlock:
//
//  Lock/Unlock the Draw Surface so we can use it
//
BOOL DirectDraw_Lock ( PBYTE* surf )
{
	DDSURFACEDESC ddsd ;

	if ( !DirectDraw_On() )
		return FALSE ;

	memset( &ddsd , 0 , sizeof(ddsd) ) ;
	ddsd.dwSize = sizeof(ddsd) ;
	if ( IDirectDrawSurface_Lock( lpDDFrame , NULL , &ddsd , 
				DDLOCK_WAIT , NULL ) == DD_OK )
	{
		*surf = ddsd.lpSurface ;
		return TRUE ;
	}
	else
		return FALSE ;
}

void DirectDraw_Unlock ()
{
	RECT dst ;
	RECT src = { 0 , 0 , W , interlace ? H : FIELDHEIGHT } ;
	
	GetWindowRect( hMainWnd , &dst ) ;

    IDirectDrawSurface_Unlock( lpDDFrame , NULL ) ;

//	IDirectDraw_WaitForVerticalBlank( lpDD , DDWAITVB_BLOCKBEGIN , NULL ) ;

	IDirectDrawSurface_Blt( lpDDPrimary , &dst , lpDDFrame , &src , 0 , NULL ) ;	
}

//-----------------------------------------------------------------------------
//
// DirectDraw_GetDC/ReleaseDC:
//
//  Get/Release the DC of the Draw Surface so we can 'GDI'-paint on it
//
BOOL DirectDraw_GetDC ( HDC* phDC )
{
	return (IDirectDrawSurface_GetDC( lpDDPrimary , phDC ) == DD_OK) ;
}

void DirectDraw_ReleaseDC ( HDC hDC )
{
	IDirectDrawSurface_ReleaseDC( lpDDPrimary , hDC ) ;
}
