#pragma warning(disable: 4201 4514)

#include <windows.h>
#include <stdio.h>

#include "DirectDraw.h"
#include "Debug.h"
#include "resource.h"


const char pError[] = "Die Funktion %s() ist fehlgeschlagen. Fehlercode %d";


CDirectDraw::CDirectDraw
(
   HINSTANCE hInstance
)
{
   HRESULT hError;

   m_nOverlayWidth      = 0;
   m_nOverlayHeight     = 0;
   m_bOverlay           = false;
   m_bFensterModus      = false;
   m_pDirectDraw        = NULL;
   m_pDirectDrawSurface = NULL;
   m_pDirectDrawOverlay = NULL;
   m_hInstance          = hInstance;

   // DirectDraw Object ffnen
   hError = DirectDrawCreate(NULL, &m_pDirectDraw, NULL);
   if(ShowError(hError, "DirectDrawCreate", true))
      return;

   DebugString("Caps %x", GetCaps());
   DebugString("ColorKeyCaps %x", GetColorKeyCaps());
   DebugString("RGB15 %d", IsRGB15());
}


CDirectDraw::~CDirectDraw()
{
   CloseDirectX();
}


void CDirectDraw::CloseDirectX()
{
   if(m_pDirectDraw != NULL)
   {
      if( m_bOverlay &&
          m_pDirectDrawOverlay != NULL
        )
      {
         UpdateOverlay(NULL, DDOVER_HIDE, false);
         m_pDirectDrawOverlay->Release();
         m_pDirectDrawOverlay = NULL;
      }

      if(m_pDirectDrawSurface != NULL)
      {
         m_pDirectDrawSurface->Release();
         m_pDirectDrawSurface = NULL;
      }

      m_pDirectDraw->Release();
      m_pDirectDraw = NULL;
   }

   // Cursor einschalten
   if(m_bFensterModus == false)
      ShowCursor(true);
}


HRESULT CALLBACK CDirectDraw::EnumDisplayModesCallback
(
   LPDDSURFACEDESC  pDirectDrawSurfaceDesc,
   LPVOID          *pContext
)
{
   DisplayMode *pDisplayMode = (DisplayMode*)pContext;

   if( pDisplayMode->nWidth == pDirectDrawSurfaceDesc->dwWidth &&
       pDisplayMode->nHeight == pDirectDrawSurfaceDesc->dwHeight &&
       pDisplayMode->nBitCount == pDirectDrawSurfaceDesc->ddpfPixelFormat.dwRGBBitCount
     )
   {
      pDisplayMode->bIsDisplayMode = true;
      return DDENUMRET_CANCEL;
   }

   pDisplayMode->bIsDisplayMode = false;
   return DDENUMRET_OK;
}


bool CDirectDraw::EraseOverlaySurface()
{
   DWORD          nPixel;
   DWORD          nZeile;
   HRESULT        hError;
   DDSURFACEDESC  sDirectDrawSurfaceDesc;

   if(m_pDirectDrawOverlay == NULL)
      return false;

   // Overlay locken
   ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
   sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
   hError = m_pDirectDrawOverlay->Lock(NULL, &sDirectDrawSurfaceDesc, DDLOCK_WAIT, 0);
   if(ShowError(hError, "Lock", true))
      return false;

   // Overlay lschen
   for(nZeile=0; nZeile<sDirectDrawSurfaceDesc.dwHeight; nZeile++)
      for(nPixel=0; nPixel<sDirectDrawSurfaceDesc.dwWidth*OVERLAY_BITCOUNT/8; nPixel+=4)
         *((int*)sDirectDrawSurfaceDesc.lpSurface+(nZeile*sDirectDrawSurfaceDesc.lPitch+nPixel)/4) = OVERLAY_BACKGROUND;

   // Overlay freigeben
   hError = m_pDirectDrawOverlay->Unlock(NULL);
   if(ShowError(hError, "Unlock", true))
      return false;

   return true;
}


bool CDirectDraw::ErasePrimarySurface()
{
   RECT           sRect;
   HRESULT        hError;
   DDBLTFX        sDirectDrawBltFX;
   DDSURFACEDESC  sDirectDrawSurfaceDesc;

   if(m_pDirectDrawSurface == NULL)
      return false;

   ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
   sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
   hError = m_pDirectDrawSurface->GetSurfaceDesc(&sDirectDrawSurfaceDesc);
   if(ShowError(hError, "GetSurfaceDesc", true))
      return false;

   SetRect(&sRect, 0, 0, sDirectDrawSurfaceDesc.dwWidth, sDirectDrawSurfaceDesc.dwHeight);

   ZeroMemory(&sDirectDrawBltFX, sizeof(sDirectDrawBltFX));
   sDirectDrawBltFX.dwSize = sizeof(sDirectDrawBltFX);
   sDirectDrawBltFX.dwFillColor = SURFACE_BACKGROUND;

   hError = m_pDirectDrawSurface->Blt(&sRect, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &sDirectDrawBltFX);
   if(ShowError(hError, "Blt", true))
      return false;

   return true;
}


bool CDirectDraw::EraseSurface()
{
   if(m_bOverlay)
      return EraseOverlaySurface();
   else
      return ErasePrimarySurface();
}


bool CDirectDraw::GetDC
(
   HDC *hDC
)
{
   HRESULT hError;

   if(m_pDirectDrawSurface == NULL)
      return false;

   hError = m_pDirectDrawSurface->GetDC(hDC);
   if(ShowError(hError, "GetDC", false))
      return false;

   return true;
}


DWORD CDirectDraw::GetCaps()
{
   HRESULT hError;
   DDCAPS  sDirectDrawCaps;

   if(m_pDirectDraw == NULL)
      return 0;

   ZeroMemory(&sDirectDrawCaps, sizeof(sDirectDrawCaps));
   sDirectDrawCaps.dwSize = sizeof(sDirectDrawCaps);
   hError = m_pDirectDraw->GetCaps(&sDirectDrawCaps, NULL);
   if(ShowError(hError, "GetCaps", true))
      return 0;

   return sDirectDrawCaps.dwCaps;
}


DWORD CDirectDraw::GetColorKeyCaps()
{
   HRESULT hError;
   DDCAPS  sDirectDrawCaps;

   if(m_pDirectDraw == NULL)
      return 0;

   ZeroMemory(&sDirectDrawCaps, sizeof(sDirectDrawCaps));
   sDirectDrawCaps.dwSize = sizeof(sDirectDrawCaps);
   hError = m_pDirectDraw->GetCaps(&sDirectDrawCaps, NULL);
   if(ShowError(hError, "GetCaps", true))
      return 0;

   return sDirectDrawCaps.dwCKeyCaps;
}


bool CDirectDraw::IsDisplayMode
(
   int nWidth,
   int nHeight,
   int nBitCount
)
{
   HRESULT     hError;
   DisplayMode sDisplayMode;

   sDisplayMode.nWidth = nWidth;
   sDisplayMode.nHeight = nHeight;
   sDisplayMode.nBitCount = nBitCount;
   sDisplayMode.bIsDisplayMode = false;

   hError = m_pDirectDraw->EnumDisplayModes(0, NULL, (LPVOID)&sDisplayMode, (LPDDENUMMODESCALLBACK)EnumDisplayModesCallback);
   if(ShowError(hError, "EnumDisplayModes", true))
      return false;

   return sDisplayMode.bIsDisplayMode;
}


bool CDirectDraw::IsRGB15()
{
   HRESULT        hError;
   DDSURFACEDESC  sDirectDrawSurfaceDesc;

   if(m_pDirectDraw == NULL)
      return false;

   // Herausfinden ob es sich um eine RGB15 oder RGB16 Grafikkarte handelt
   ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
   sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
   hError = m_pDirectDraw->GetDisplayMode(&sDirectDrawSurfaceDesc);
   if(ShowError(hError, "GetDisplayMode", true))
      return false;

   if(sDirectDrawSurfaceDesc.ddpfPixelFormat.dwGBitMask == 0x03e0)
      return true;

   return false;
}


bool CDirectDraw::LockSurface
(
   void **pSurface,
   int   *nStartOfNextLine,
   bool   bAustastung
)
{
   HRESULT       hError;
   DDSURFACEDESC sDirectDrawSurfaceDesc;

   // Surface erst in der vertikalen Austastlcke locken
   if(bAustastung == true)
   {
      if(m_pDirectDraw == NULL)
         return false;

      hError = m_pDirectDraw->WaitForVerticalBlank(DDWAITVB_BLOCKBEGIN, NULL);
      if(ShowError(hError, "WaitForVerticalBlank", true))
         return false;
   }

   // Surface locken
   if(m_bOverlay == true)
   {
      if(m_pDirectDrawOverlay == NULL)
         return false;

      ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
      sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
      hError = m_pDirectDrawOverlay->Lock(NULL, &sDirectDrawSurfaceDesc, DDLOCK_WAIT, 0);
      if(ShowError(hError, "Lock", false))
         return false;

      *pSurface = sDirectDrawSurfaceDesc.lpSurface;
      *nStartOfNextLine = sDirectDrawSurfaceDesc.lPitch;
   }
   else
   {
      if(m_pDirectDrawSurface == NULL)
         return false;

      ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
      sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
      hError = m_pDirectDrawSurface->Lock(NULL, &sDirectDrawSurfaceDesc, DDLOCK_WAIT, 0);
      if(ShowError(hError, "Lock", false))
         return false;

      *pSurface = sDirectDrawSurfaceDesc.lpSurface;
      *nStartOfNextLine = sDirectDrawSurfaceDesc.lPitch;
   }

   return true;
}


bool CDirectDraw::OpenDirectDraw
(
   HWND  hWnd,
   bool  bOverlay,
   int   nScreenWidth,
   int   nScreenHeight,
   int   nOverlayWidth,
   int   nOverlayHeight,
   bool  bFensterModus,
   DWORD dwCooperativeLevel
)
{
   HRESULT        hError;
   DDSURFACEDESC  sDirectDrawSurfaceDesc;

   m_nOverlayWidth  = nOverlayWidth;
   m_nOverlayHeight = nOverlayHeight;
   m_bOverlay       = bOverlay;
   m_bFensterModus  = bFensterModus;

	if(IsWindow(hWnd) == FALSE)
	{
		DebugString("HWND ungltig");
      return false;
	}

   // Im Vollbildmodus den ganzen Bildschirm exklusiv besorgen 
   hError = m_pDirectDraw->SetCooperativeLevel(hWnd, dwCooperativeLevel);
   if(ShowError(hError, "SetCooperativeLevel", true))
      return false;

   if(m_bFensterModus == false)
   {
      // Auflsung whlen
      hError = m_pDirectDraw->SetDisplayMode(nScreenWidth, nScreenHeight, SURFACE_BITCOUNT);
      if(ShowError(hError, "SetDisplayMode", true))
         return false;
   }

   // Surface erzeugen
   ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
   sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
   sDirectDrawSurfaceDesc.dwFlags = DDSD_CAPS;
   sDirectDrawSurfaceDesc.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
   hError = m_pDirectDraw->CreateSurface(&sDirectDrawSurfaceDesc, &m_pDirectDrawSurface, NULL);
   if(ShowError(hError, "CreateSurface", true))
      return false;

   // Overlay erzeugen
   if(m_bOverlay)
   {
      // Pixelformat fr Overlay
      DDPIXELFORMAT sDirectDrawPixelFormat;

      ZeroMemory(&sDirectDrawPixelFormat, sizeof(sDirectDrawPixelFormat));
      sDirectDrawPixelFormat.dwSize = sizeof(DDPIXELFORMAT);
      sDirectDrawPixelFormat.dwFlags = DDPF_FOURCC;
      sDirectDrawPixelFormat.dwFourCC = OVERLAY_FORMAT;
      sDirectDrawPixelFormat.dwYUVBitCount = OVERLAY_BITCOUNT;

      // Overlay erzeugen
      ZeroMemory(&sDirectDrawSurfaceDesc, sizeof(sDirectDrawSurfaceDesc));
      sDirectDrawSurfaceDesc.dwSize = sizeof(sDirectDrawSurfaceDesc);
      sDirectDrawSurfaceDesc.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_PIXELFORMAT;
      sDirectDrawSurfaceDesc.ddsCaps.dwCaps = DDSCAPS_OVERLAY | DDSCAPS_VIDEOMEMORY | DDSCAPS_LIVEVIDEO;
      sDirectDrawSurfaceDesc.dwWidth = m_nOverlayWidth;
      sDirectDrawSurfaceDesc.dwHeight = m_nOverlayHeight;
      sDirectDrawSurfaceDesc.ddpfPixelFormat = sDirectDrawPixelFormat;
      hError = m_pDirectDraw->CreateSurface(&sDirectDrawSurfaceDesc, &m_pDirectDrawOverlay, NULL);
      if(ShowError(hError, "CreateSurface", true))
         return false;

      // Overlay lschen
      EraseOverlaySurface();
   }

   // Cursor ausschalten
   if(m_bFensterModus == false)
      ShowCursor(false);

   return true;
}


bool CDirectDraw::ReleaseDC
(
   HDC hDC
)
{
   HRESULT  hError;

   if(m_pDirectDrawSurface == NULL)
      return false;

   hError = m_pDirectDrawSurface->ReleaseDC(hDC);
   if(ShowError(hError, "ReleaseDC", true))
      return false;

   return true;
}


bool CDirectDraw::ShowError
(
   HRESULT  hError,
   char    *strFunktion,
   bool     bShowError
)
{
   if(hError == DD_OK)
      return false;

   // Cursor einschalten
   if(m_bFensterModus == false)
      ShowCursor(true);

   // Fehlermeldung ausgeben
   if(bShowError == true)
      MessageBox(pError, strFunktion, hError&0xFFFF);
   else
      DebugString(pError, strFunktion, hError&0xFFFF);

   // Cursor ausschalten
   if(m_bFensterModus == false)
      ShowCursor(false);

   return true;
}


bool CDirectDraw::UnlockSurface()
{
   HRESULT hError;

   if(m_bOverlay == true)
   {
      if(m_pDirectDrawOverlay == NULL)
         return false;

      hError = m_pDirectDrawOverlay->Unlock(NULL);
      if(ShowError(hError, "Unlock", true))
         return false;
   }
   else
   {
      if(m_pDirectDrawSurface == NULL)
         return false;

      hError = m_pDirectDrawSurface->Unlock(NULL);
      if(ShowError(hError, "Unlock", true))
         return false;
   }

   return true;
}


bool CDirectDraw::UpdateOverlay
(
   LPRECT pDestinationRect,
   DWORD  dwFlags,
   bool   bColorKey
)
{
   RECT    sSourceRect;
   HRESULT hError;

   if(m_pDirectDrawOverlay == NULL)
      return false;

   sSourceRect.left   = 0;
   sSourceRect.top    = 2;
   sSourceRect.right  = m_nOverlayWidth;
   sSourceRect.bottom = m_nOverlayHeight-2;

   // Das Overlay ist nur auf schwarzen Hintergrund sichtbar
   DDOVERLAYFX sDirectDrawOverlayFX;

   ZeroMemory(&sDirectDrawOverlayFX, sizeof(sDirectDrawOverlayFX));
   sDirectDrawOverlayFX.dwSize = sizeof(sDirectDrawOverlayFX);

   // Abfragen ob Funktion untersttzt wird
   if( bColorKey == true &&
       GetColorKeyCaps() & DDCKEYCAPS_DESTOVERLAY
     )
   {
      dwFlags |= DDOVER_DDFX | DDOVER_KEYDESTOVERRIDE;
      sDirectDrawOverlayFX.dckDestColorkey.dwColorSpaceLowValue = SURFACE_BACKGROUND;
      sDirectDrawOverlayFX.dckDestColorkey.dwColorSpaceHighValue = SURFACE_BACKGROUND;
   }

   hError = m_pDirectDrawOverlay->UpdateOverlay(&sSourceRect, m_pDirectDrawSurface, pDestinationRect, dwFlags, &sDirectDrawOverlayFX);
   if(ShowError(hError, "UpdateOverlay", false))
      return false;

   return true;
}
