#ifndef __DIRECTDRAW__
#define __DIRECTDRAW__


#include <ddraw.h>


const int   SURFACE_BITCOUNT   = 16;
const int   SURFACE_BACKGROUND = 0x00000000;
const int   OVERLAY_BITCOUNT   = 16;
const int   OVERLAY_BACKGROUND = 0x80008000;
const DWORD OVERLAY_FORMAT     = MAKEFOURCC('Y','U','Y','2');


class CDirectDraw
{
   public:
      CDirectDraw
      (
         HINSTANCE hInstance
      );

      virtual ~CDirectDraw();

      bool  ErasePrimarySurface();
      bool  EraseOverlaySurface();
      bool  EraseSurface();
      DWORD GetCaps();
      DWORD GetColorKeyCaps();

      bool GetDC
      (
         HDC *hDC
      );

      bool IsDisplayMode
      (
         int nWidth,
         int nHeight,
         int nBitCount
      );

      bool IsRGB15();

      bool LockSurface
      (
         void **pSurface,
         int   *nStartOfNextLine,
         bool   bAustastung
      );

      bool OpenDirectDraw
      (
         HWND  hWnd,
         bool  bOverlay,
         int   nScreenWidth,
         int   nScreenHeight,
         int   nOverlayWidth,
         int   nOverlayHeight,
         bool  bFensterModus,
         DWORD dwCooperativeLevel
      );

      bool ReleaseDC
      (
         HDC hDC
      );

      bool UpdateOverlay
      (
         LPRECT pDestinationRect,
         DWORD  dwFlags,
         bool   bColorKey
      );

      bool UnlockSurface();

   private:
      typedef struct
      {
         DWORD nWidth;
         DWORD nHeight;
         DWORD nBitCount;
         bool  bIsDisplayMode;
      }DisplayMode;

      void CloseDirectX();

      static HRESULT CALLBACK EnumDisplayModesCallback
      (
         LPDDSURFACEDESC  pDirectDrawSurfaceDesc,
         LPVOID          *pContext
      );

      bool ShowError
      (  
         HRESULT  hError,
         char    *strFunktion,
         bool     bShowError
      );

      bool                 m_bOverlay;
      bool                 m_bFensterModus;
      int                  m_nOverlayWidth;
      int                  m_nOverlayHeight;
      HINSTANCE            m_hInstance;
      LPDIRECTDRAW         m_pDirectDraw;
      LPDIRECTDRAWSURFACE  m_pDirectDrawSurface;
      LPDIRECTDRAWSURFACE  m_pDirectDrawOverlay;
};

#endif
