//-----------------------------//
// FindKey.c   by:   KennyWood //
//-----------------------------//

#include "Decoder.h"
#include "Tables.h"

//-----------------------------------------------------------------------------

int matrix [ (288 << 9) + 288 ] ;			// Correlation matrix

struct
{
	int pos ;
	int val ;
} bestCorrelations [ MAXCHECKLEVELS ] ;		// 'Top ten' of correlations

BYTE testKey [ 32768 ] ;					// May each key be tested or not ?

//-----------------------------------------------------------------------------
//
// Decoder_KeySearch:
//
//  We use the correlation matrix to find the (hopefully) best key.
//
void Decoder_KeySearch ()
{
	int     i , j , key , sum , pos ;
	PINT    p ;
	PUSHORT resource ;

	minimum = 0x7FFFFFFF ;

	kcnt = 0 ;
	memset( testKey , 0x00 , sizeof(testKey) ) ;	
	for ( j = 0 ; j < checkLevels ; j ++ )
	{
		pos = bestCorrelations[j].pos ;
		resource = keys_resource + keys_directory[ pos ].first ;
		for ( i = keys_directory[ pos ].keys - 1 ; i >= 0 ; i -- )
			testKey[ *resource ++ ] = 0xFF ;
	}
	
	for ( key = 0 ; key < 32768 ; key ++ )
		if ( testKey[key] )
		{
			kcnt ++ ;

			p   = checkPair[ key ] ;
			sum = 0 ;

			for ( i = 0 ; i < checkPairs ; i ++ )
			{
				sum += matrix[ *p ++ ] ;
				if ( sum >= minimum )
					break ;
			}
			if ( i == checkPairs )
			{
				minimum = sum ;
				bestKey = key ;
			}
		}
}

//-----------------------------------------------------------------------------
//
// Decoder_InsertCorrelation:
//
//  We try to insert a correlation between the list of best correlations.
//
void Decoder_InsertCorrelation ( int c , int p )
{
	int i , m ;

	if ( c < bestCorrelations[checkLevels-1].val )
	{
		i = checkLevels-1 ;
		while ( (i > 0) && (c < bestCorrelations[i-1].val) )
		{
			i -- ;
		}

		for ( m = checkLevels-1 ; m > i ; m -- )
			bestCorrelations[m] = bestCorrelations[m-1] ;
		
		bestCorrelations[i].val = c ;
		bestCorrelations[i].pos = p ;
	}
}

//-----------------------------------------------------------------------------
//
// Decoder_ComputeCorrelations:
//
//  We compute all entries for our correlation matrix.
//
void Decoder_ComputeCorrelations ()
{
	int   i , x , d , index ;
	int   sum ;
	PBYTE a1 , a2 ;

	for ( i = 0 ; i < MAXCHECKLEVELS ; i ++ )
	{
		bestCorrelations[i].pos = -1 ;
		bestCorrelations[i].val = 0x7FFFFFFF ;
	}
	
	for ( i = 0 ; i < combination_level[checkPairs-1] ; i ++ )
	{
		index = combination[i] ;
		a1    = testPixel[ index >> 0x0009 ] ;
		a2    = testPixel[ index &  0x01FF ] ;
		sum   = 0 ;
		x     = checkWidth ;
		while ( x -- )
		{
			d = (int)*a1 ++ - (int)*a2 ++ ;
			if ( d < 0 )
				sum -= d ;
			else
				sum += d ;
		}
		matrix[index] = sum ;
		Decoder_InsertCorrelation( sum , i ) ;
	}
}

//-----------------------------------------------------------------------------
//
// Decoder_FindBestKey:
//
//  We try ALL the possible (2^15 = 32768) keys and calculate
// a penalty function for each of them.  We remember at any
// point which key yielded the best results and the minimum
// penalty (for that key) so far.
//  The penalty function is computed from correlation between
// points placed in consecutive rows.
//
void Decoder_FindBestKey ()
{
	int    i , x , cx_inc ;
	PSHORT row ;

	cx_inc = ((captureWidth - 1 - CX_MARGIN*2) / (checkWidth - 1)) ;
	for ( i = 0 ; i < 288 ; i ++ )
	{
		row = rowYUVField[i] + CX_MARGIN ;
		for ( x = 0 ; x < checkWidth ; x ++ )
		{
			testPixel[i][x] = *(PBYTE)row ;
			row += cx_inc ;
		}
	}

	Decoder_ComputeCorrelations() ;
	Decoder_KeySearch() ;

	if ( showCheckArea )
		for ( i = 0 ; i < checkPairs ; i ++ )
		{
			row = rowYUVField[ checkPair[bestKey][i] &  0x01FF ] + CX_MARGIN ;
			for ( x = 0 ; x < checkWidth ; x ++ )   *row = (SHORT)0xFFFF , row += cx_inc ;
			row = rowYUVField[ checkPair[bestKey][i] >> 0x0009 ] + CX_MARGIN ;
			for ( x = 0 ; x < checkWidth ; x ++ )   *row = (SHORT)0xFFFF , row += cx_inc ;
		}
}
