//----------------------------//
// GenTab.c   by:   KennyWood //
//----------------------------//

#include <math.h>
#include "Tables.h"

//-----------------------------------------------------------------------------

int    usedCombination [ (288 << 9) + 288 ] ;
USHORT apparitions     [ (288 << 9) + 288 ] ;
int    key_combination [ 32768 ][ 288 - CY_MARGIN*2 ] ;

//-----------------------------------------------------------------------------
//
// Decoder_GeneratePermInv:
//
//  This is the MAIN Nagravision encryption algorithm.
//  Given a key, we fill 'P' with the permutation function y' = P(y)
//
void Decoder_GeneratePermInv ( int key )
{
	SHORT  B [ 32 ] ;
	int    i , key_inc ;
	BYTE   b ;
	PSHORT perm = P ;

	key_inc = ((key & 127) << 1) + 1 ;
	key     = key >> 7 ;

	for ( i = 0 ; i < 32 ; i ++ )
		B[i] = i ;

	for ( i = 0 ; i < 255 ; i ++ )
	{
		b = S[ key ] ;
		*perm ++ = B[b] ;
		B[b] = i + 32 ;
		key += key_inc ;
		key &= 0xFF ;
	}

	for ( i = 0 ; i < 32 ; i ++ )
		*perm ++ = B[i] ;
}

//-----------------------------------------------------------------------------
//
// GenTab_InitYUV2RGB:
//
//  Self explained.
//
void GenTab_InitYUV2RGB ( void )
{
	int    Y , U , V , idx ;
	double R , G , B ;

	printf( "[                                ]\r[" ) ;
	for ( Y = 0 ; Y < 256 ; Y += 4 )
	{
		if ( !(Y & 0x07) )
			printf( "." ) ;
		for( U = 0 ; U < 256 ; U += 4 )
		{
			for ( V = 0 ; V < 256 ; V += 4 )
			{
				/*
				 * Y = 0.299R + 0.587G + 0.114B
				 * U = 0.493(B-Y)Sin(phase)
				 * V = 0.877(R-Y)Cos(phase)
				 *
				 * We know Y,U,V from the captured frame, so:
				 *
				 * R =  Y + (V / 0.877)
				 * B =  Y + (U / 0.493)
				 * G = (Y - 0.299R - 0.114B) / 0.587
				 */
				
				R = (double)Y + ((double)U - 128.0) / (0.877 * 0.707) ;
				B = (double)Y + ((double)V - 128.0) / (0.493 * 0.707) ;
				G = ((double)Y - 0.299*R - 0.114*B) / 0.587 ;

				if ( R > 255.0 )   R = 255.0 ;
				if ( G > 255.0 )   G = 255.0 ;
				if ( B > 255.0 )   B = 255.0 ;
				if ( R <   0.0 )   R =   0.0 ;
				if ( G <   0.0 )   G =   0.0 ;
				if ( B <   0.0 )   B =   0.0 ;

				idx = ((V>>2)<<12) | ((U>>2)<<6) | (Y>>2) ;

				YUV2RGBbpp[0][idx] = RGB15(R,G,B) ;
				YUV2RGBbpp[1][idx] = RGB16(R,G,B) ;
			}
		}
	}
}

//-----------------------------------------------------------------------------
//
// GenTab_InitPalDemod:
//
//  Self explained.
//
void GenTab_InitPalDemod ( void )
{
	int    i , uv , mult , d ;
	double alpha , c , s ;

#define PIFACTOR 1.5607109571204510500
#define PHASE    0.1047197551196597746
#define RATIO    0.7058823529411764706

	printf( "[                                ]\r[" ) ;
	for ( i = 0 ; i < PALPERIOD ; i ++ )
	{
		if ( !(i % ((PALPERIOD-1)/31)) )
			printf( "." ) ;
		alpha = PIFACTOR * ( PHASE + (PALPERIOD-1) - i ) ;

		c = cos(alpha) * 32.0 ;
		s = sin(alpha) * 32.0 ;
		d = (0x80 << 3) ;

		for ( uv = 0 ; uv < 64 ; uv ++ )
		{
			mult = uv - (0x80 >> 2) ;

			palDemod[(i<<4)+ 0][uv] = (SHORT) ( c         * mult + d) ;
			palDemod[(i<<4)+ 1][uv] = (SHORT) ( s         * mult + d) ;
			palDemod[(i<<4)+ 2][uv] = (SHORT) (-s * RATIO * mult + d) ;
			palDemod[(i<<4)+ 3][uv] = (SHORT) ( c * RATIO * mult + d) ;

			palDemod[(i<<4)+ 4][uv] = (SHORT) ( c         * mult + d) ;
			palDemod[(i<<4)+ 5][uv] = (SHORT) (-s         * mult + d) ;
			palDemod[(i<<4)+ 6][uv] = (SHORT) (-s * RATIO * mult + d) ;
			palDemod[(i<<4)+ 7][uv] = (SHORT) (-c * RATIO * mult + d) ;

			palDemod[(i<<4)+ 8][uv] = (SHORT) ( c         * mult + d) ;
			palDemod[(i<<4)+ 9][uv] = (SHORT) ( s         * mult + d) ;
			palDemod[(i<<4)+10][uv] = (SHORT) ( s * RATIO * mult + d) ;
			palDemod[(i<<4)+11][uv] = (SHORT) (-c * RATIO * mult + d) ;

			palDemod[(i<<4)+12][uv] = (SHORT) ( c         * mult + d) ;
			palDemod[(i<<4)+13][uv] = (SHORT) (-s         * mult + d) ;
			palDemod[(i<<4)+14][uv] = (SHORT) ( s * RATIO * mult + d) ;
			palDemod[(i<<4)+15][uv] = (SHORT) ( c * RATIO * mult + d) ;
		}
	}
}

//-----------------------------------------------------------------------------
//
// GenTab_SearchCombinationID:
//
//  Auxiliar function for 'InitCheckPairsOfLines'.
//
int GenTab_SearchCombinationID ( int comb )
{
	int s ;

	for ( s = 0 ; s < num_combinations ; s ++ )
		if ( comb == combination[s] )
			return s ;

	return -1 ;
}

//-----------------------------------------------------------------------------
//
// GenTab_InitCheckPairsOfLines:
//
//  We precalculate the correlation-check pairs and the key sets for them.
//
void GenTab_InitCheckPairsOfLines ( void )
{	
	int i , c , y , bestY , key , maximum ;
	int index , invIndex , M , maxApps ;

	memset(     apparitions , 0x00 , sizeof(    apparitions) ) ;
	memset( usedCombination , 0xFF , sizeof(usedCombination) ) ;
	num_combinations = 0 ;

	printf( "[                                ] Step 1 of 3\r[" ) ;
	for ( key = 0 ; key < 32768 ; key ++ )
	{
		if ( !(key & 0x03FF) )
			printf( "." ) ;
		Decoder_GeneratePermInv( key ) ;
		for ( y = 0 ; y < 288 - CY_MARGIN*2 ; y ++ )
		{
			index = ((int)P[CY_MARGIN+y  ] & 0x01FF) |
				   (((int)P[CY_MARGIN+y+1] & 0x01FF) << 9);
			invIndex = ((index & 0x01FF) << 9) | (index >> 9) ;
			key_combination[key][y] = index ;
			apparitions[   index] ++ ;
			apparitions[invIndex] ++ ;
		}
	}

	printf( "\n[                                ] Step 2 of 3\r[" ) ;
	for ( i = 0 ; i < MAXCHECKPAIRS ; i ++ )
	{
		if ( !(i & 0x0001) )
			printf( "." ) ;
		for ( key = 0 ; key < 32768 ; key ++ )
		{
			maximum = 0 ;
			for ( y = 0 ; y < 288 - CY_MARGIN*2 ; y ++ )
			{
				index = key_combination[key][y] ;				
				if ( index == -1 )
					continue ;
				invIndex = ((index & 0x01FF) << 9) | (index >> 9) ;
				maxApps = max( apparitions[index] , apparitions[invIndex] ) ;
				if ( maxApps > maximum )
				{
					M = min( index , invIndex ) ;
					maximum = maxApps ;
					bestY   = y ;
				}
			}			
			if ( usedCombination[M] == -1 )
			{
				usedCombination[M] = num_combinations ;
				combination[ num_combinations ++ ] = M ;
			}			
			checkPair[key][i] = M ;
			key_combination[key][bestY] = -1 ;
		}
		combination_level[i] = num_combinations ;
	}

	printf( "\n[                                ] Step 3 of 3\r[" ) ;
	keys_resource_cnt = 0 ;
	for ( c = 0 ; c < num_combinations ; c ++ )
	{
		if ( !(c % (((num_combinations) >> 5)+1)) )
			printf( "." ) ;
		index = combination[c] ;
		keys_directory[c].first = keys_resource_cnt ;
		keys_directory[c].keys  = 0 ;
		for ( key = 0 ; key < 32768 ; key ++ )
			for ( i = 0 ; i < MAXCHECKPAIRS ; i ++ )
				if ( checkPair[key][i] == index )
				{
					keys_resource[ keys_resource_cnt ++ ] = key ;
					keys_directory[c].keys ++ ;
					break ;
				}
	}
}

//-----------------------------------------------------------------------------
//
// main:
//
//  Wops.  MAIN program.
//
void main ()
{
	int   i , bind ;
	FILE* stream ;
	
	printf( "      TABLES PRECALCULATOR FOR:\n" ) ;
	printf( "%s\n\n" , appName ) ;
	
	printf( "Reading KEY.TXT...\n\n" , tablesFile ) ;
	stream = fopen( "Key.txt" , "rb" ) ;
	if ( !stream )
	{
		printf( "ERROR !  Could not find KEY.TXT...\n\n" ) ;
		printf( "This file is *NECESSARY* for both GenTab.exe and NagraView.exe.\n" ) ;
		printf( "Read How-To.doc for info on what a keyfile is.\n\n" , web ) ;
		printf( "You can find the file KEY.TXT at:     %s\n\n" , web ) ;
		printf( "The key file for Canal+ France, at:   http://eurosat.com/salp/zorglub\n\n" ) ;
		exit( -1 ) ;
	}
	for ( i = 0 ; i < 256 ; i ++ )
	{
		fscanf( stream , "%d" , &bind ) ;
		S[i] = bind ;
	}
	fclose( stream ) ;

	printf( "Creating destination file (%s)...\n" , tablesFile ) ;
	stream = fopen( tablesFile , "wb" ) ;
	printf( "Done !\n\n" ) ;

	printf( "Saving signature...\n\n" ) ;	
	fwrite( appName , 1 , strlen(appName)+1 , stream ) ;	// NULL TERMINATED

	printf( "Generating YUV -> RGB15/16 conversion table...\n" ) ;	
	GenTab_InitYUV2RGB() ;
	printf( "\nSaving (size: %dKB)...\n" , sizeof(YUV2RGBbpp) >> 10 ) ;
	fwrite( YUV2RGBbpp , 1 , sizeof(YUV2RGBbpp) , stream ) ;
	printf( "Done !\n\n" ) ;

	printf( "Generating PAL demodulation table...\n" ) ;	
	GenTab_InitPalDemod() ;
	printf( "\nSaving (size: %dKB)...\n" , sizeof(palDemod) >> 10 ) ;
	fwrite( palDemod , 1 , sizeof(palDemod) , stream ) ;
	printf( "Done !\n\n" ) ;

	printf( "Configured to check up to %d pairs of lines each frame...\n" , MAXCHECKPAIRS ) ;
	printf( "Generating minimized set of check pairs...\n" ) ;
	printf( "THIS WILL TAKE A *LONG* WHILE (up to 20 minutes in my Pentium 100MHz)...\n" ) ;
	GenTab_InitCheckPairsOfLines() ;
	printf( "\nThe minimized set has %d pairs...\n" , num_combinations ) ;
	printf( "Saving (size: %dKB)...\n" ,
		(sizeof(num_combinations     ) +
		 sizeof(    combination      ) +
		 sizeof(    combination_level) +
		 sizeof(    checkPair        ))  >> 10 ) ;
	fwrite( &num_combinations      , 1 , sizeof(num_combinations     ) , stream ) ;
	fwrite(      combination       , 1 , sizeof(    combination      ) , stream ) ;
	fwrite(      combination_level , 1 , sizeof(    combination_level) , stream ) ;
	fwrite(      checkPair         , 1 , sizeof(    checkPair        ) , stream ) ;
	printf( "The keys directory has %d entries...\n" , keys_resource_cnt ) ;
	printf( "Saving (size: %dKB)...\n" ,
		(sizeof(keys_resource ) +
		 sizeof(keys_directory))  >> 10 ) ;
	fwrite( keys_resource  , 1 , sizeof(keys_resource ) , stream ) ;
	fwrite( keys_directory , 1 , sizeof(keys_directory) , stream ) ;
	printf( "Done !\n\n" ) ;

	printf( "Ensure '%s' is in the SAME directory that Nagraview.exe...\n\n" , tablesFile ) ;

	printf( "Enjoy !\n\n" ) ;

	fclose( stream ) ;
}
