//-------------------------------//
// NagraView.c   by:   KennyWood //
//-------------------------------//

#include <time.h>
#include "BTChip.h"
#include "Resource.h"
#include "Capture.h"
#include "Decoder.h"
#include "DirectDraw.h"

//-----------------------------------------------------------------------------
//
// WINDOWS VARIABLES
//
HINSTANCE   hInstance ;     // Application instance
HWND        hMainWnd ;      // The main window
HWND        hCapWnd ;       // The capture window associated with the main window
BITMAPINFO* DIB ;           // A bitmap header with the characteristics of the cap. frame
HFONT       hFont ;
HACCEL      hAccel ;

const long dwGDIWindowStyle = WS_CAPTION | WS_CLIPCHILDREN | WS_OVERLAPPED | 
							  WS_SYSMENU | WS_MINIMIZEBOX | WS_POPUP | WS_VISIBLE ;
const long dwDDrWindowStyle = WS_VISIBLE | WS_SYSMENU | WS_POPUP ;

//-----------------------------------------------------------------------------
//
// GLOBAL STATE CONFIGURABLE VARIABLES
//
BOOL fullScreen ;
BOOL color ;
BOOL decrypt ;
BOOL statistics ;
int  cinemascopeUp   = 0 ;
int  cinemascopeDown = 0 ;
int  checkPairs ;
int  checkWidth ;
int  checkLevels ;
int  channel ;
int  card ;
int  priority ;
int  screenSize ;
BOOL showCheckArea ;
BOOL useSound ;
BOOL rgb565 ;
int  whichDriver ;
BOOL useBT848 ;
BOOL highQuality ;
int  interlace ;
BOOL phaseCorrector = TRUE ;
int  palSecam ;
int  alwaysOnTop ;

int topAdjust = 0 ;

volatile BOOL autoFindCScope = FALSE ;

//-----------------------------------------------------------------------------
//
// GLOBAL STATE INTERNAL VARIABLES
//
int  displayWidth ;
int  displayHeight ;
int  displayBPP ;
int  captureWidth ;
BOOL started ;
BOOL canSound ;
int  minimum ;
int  bestKey ;
int  kcnt ;
BOOL evenFrame = TRUE ;
int  keyID ;
int  bindID ;

#if DEBUGFILE	
	FILE* debugFile ;
#endif

//-----------------------------------------------------------------------------
//
// LOCAL STATE VARIABLES
//
BOOL clearBkg = TRUE ;

/*volatile */BOOL descramblingNow = FALSE ;
/*volatile */BOOL switchOffAFCS   = FALSE ;

//-----------------------------------------------------------------------------

#define MSGSECONDS 5	// # of seconds that a message remains on screen

char    lastMsg [ 256 ] ;
clock_t msgDate ;
BOOL    putMsg ;

#define PutMessage() \
	msgDate = clock()

#define PutMessageValue( label , var ) \
	{ sprintf( lastMsg , label ": %d" , var ) ; \
	PutMessage() ; }

#define PutMessageBanner( banner ) \
	{ sprintf( lastMsg , banner ) ; \
	PutMessage() ; }

#define PutMessageCondBanner( cond , banner1 , banner2 ) \
	{ if ( cond ) \
		sprintf( lastMsg , banner1 ) ; \
	else \
		sprintf( lastMsg , banner2 ) ; \
	PutMessage() ; }

//-----------------------------------------------------------------------------
//
// PrintStatistics:
//
//  Print some important execution info over the current frame.
//
void PrintStatistics ( HDC hDC )
{	
	static float   value     = 0.0 ;
	static int     frame_cnt = 0 ;
	static clock_t tiempo ;

	char str [ 256 ] ;

	putMsg = ((clock() - msgDate) <= CLOCKS_PER_SEC*MSGSECONDS) ;

	if ( statistics || putMsg )
	{
		SetBkMode( hDC , OPAQUE ) ;
		SetBkColor( hDC , RGB(0x00,0x00,0x00) ) ;
		SetTextColor( hDC , RGB(0xFF,0xFF,0x00) ) ;
	}

	if ( putMsg )
		TextOut( hDC , 0 , displayHeight-16  , lastMsg , strlen(lastMsg) ) ;
	
	if ( !statistics )
		return ;
	
	if ( !frame_cnt )
		tiempo = clock() ;

	frame_cnt ++ ;

	if ( frame_cnt == 10 )
	{
		value = ((float)(frame_cnt) * (float)CLOCKS_PER_SEC) / (float)(clock() - tiempo) ;
		frame_cnt = 0 ;
	}

	sprintf( str , "FPS:%1.1f CS:(%d,%d) C:%05d T:%05d" , value , cinemascopeUp , cinemascopeDown , minimum , kcnt ) ;
	TextOut( hDC , 0 , 0 , str , strlen(str) ) ;
}

//-----------------------------------------------------------------------------
//
// VideoStreamCallback:
//
//  This is the MAIN capture procedure.  It is called automatically when
// our video capturer has a frame ready to be processed.
//
LRESULT CALLBACK VideoStreamCallback ( HWND hWnd , LPVIDEOHDR pVideoHeader )
{
	HDC   hDC ;
	PBYTE surf ;

/*	CAPSTATUS status ;
	capGetStatus( hCapWnd , &status , sizeof(status) ) ;
	evenFrame = status.dwCurrentVideoFrame & 1 ;*/
	evenFrame = !evenFrame ;

	if ( DirectDraw_On() )
	{
		if ( clearBkg )
		{
			DirectDraw_EraseFrameSurface ( hMainWnd ) ;
			clearBkg = FALSE ;
		}
		if ( DirectDraw_Lock( &surf ) )
		{
			descramblingNow = TRUE ;
			Decoder_Descramble( (PSHORT)pVideoHeader->lpData , 
								(PSHORT)surf , 
								captureWidth , HEIGHT , 
								displayWidth , displayHeight ,
								displayBPP , autoFindCScope ) ;
			descramblingNow = FALSE ;
			DirectDraw_Unlock() ;
		}
		if ( DirectDraw_GetDC( &hDC ) )
		{
			PrintStatistics( hDC ) ;
			DirectDraw_ReleaseDC( hDC ) ;
		}
	}
	else
	{
		if ( clearBkg )
		{
			memset( YUVfield , 0 , captureWidth * HEIGHT * sizeof(SHORT) ) ;
			clearBkg = FALSE ;
		}		
		descramblingNow = TRUE ;
		Decoder_Descramble( (PSHORT)pVideoHeader->lpData , 
							(PSHORT)YUVfield , 
							captureWidth , HEIGHT , 
							displayWidth , HEIGHT , 
							displayBPP , autoFindCScope ) ;
		descramblingNow = FALSE ;

		hDC = GetDC( hMainWnd ) ;

		/*if ( (displayWidth  != DIB -> bmiHeader.biWidth ) ||
			 (displayHeight != DIB -> bmiHeader.biHeight) )
		{*/
			StretchDIBits( hDC , 0 , 0 , displayWidth , displayHeight , 0 , 0 ,
				DIB -> bmiHeader.biWidth , interlace ? DIB -> bmiHeader.biHeight : HEIGHT ,
							YUVfield , DIB , DIB_RGB_COLORS , SRCCOPY ) ;
		/*}
		else
		{
			SetDIBitsToDevice( hDC , 0 , 0 ,
								DIB -> bmiHeader.biWidth , DIB -> bmiHeader.biHeight ,
								0 , 0 , 0 , DIB -> bmiHeader.biHeight ,
								YUVfield , DIB , DIB_RGB_COLORS ) ;
		}*/

		PrintStatistics( hDC ) ;

		ReleaseDC( hCapWnd , hDC ) ;	
	}

	if ( autoFindCScope )
	{
		if ( switchOffAFCS )
			autoFindCScope = FALSE ;
		switchOffAFCS  = TRUE ;
	}

	return (LRESULT)TRUE ;
}

//-----------------------------------------------------------------------------
//
// Layout:
//
//  Give the application's window the correct size and position.
//
void Layout ( HWND hWnd )
{
	int   nXBorder ;
	int   nYBorder ;
	int   nYMenuAndCaption ;
	RECT  clientRect ;
	RECT  windowRect ;

	SetWindowLong( hWnd , GWL_STYLE , dwGDIWindowStyle ) ;
		
	nYBorder = GetSystemMetrics(SM_CYBORDER) ;
	nXBorder = GetSystemMetrics(SM_CXBORDER) ;

	GetWindowRect( hWnd , &windowRect ) ;
	GetClientRect( hWnd , &clientRect ) ;

	ClientToScreen( hWnd , (LPPOINT)&clientRect ) ;
	nYMenuAndCaption = clientRect.top - windowRect.top + 1 ;

	SetWindowPos( hWnd , alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST , 
				  windowRect.left ,
				  windowRect.top ,
				  displayWidth  + nXBorder*2 ,
				  displayHeight + nYBorder*2 + nYMenuAndCaption , 0 ) ;
}

//-----------------------------------------------------------------------------
//
// ChangeVideoMode:
//
//  Opens/Closes the fullscreen mode depending on the value of 'fullScreen'.
//
void ChangeVideoMode ()
{
	if ( !VideoCapture_Open( captureWidth , 576 , 16 ) )
	{
		DestroyWindow( hMainWnd ) ;
		return ;
	}
	if ( !fullScreen )
	{		
		DirectDraw_Close() ;		
		Layout( hMainWnd ) ;
	}
	else
	{
		SetWindowLong( hMainWnd , GWL_STYLE , dwDDrWindowStyle ) ;
		if ( !DirectDraw_Open( hMainWnd , displayWidth , displayHeight , 
								displayBPP , captureWidth , HEIGHT ) )
		{
			DestroyWindow( hMainWnd ) ;
			return ;
		}
	}	
}

//-----------------------------------------------------------------------------
//
// MainWnd:
//
//  Nice Windows message handling for the main application window.
//
LRESULT CALLBACK MainWnd ( HWND hWnd , UINT uMsg , WPARAM wParam , LPARAM lParam )
{
	switch ( uMsg )
	{
		case WM_KEYDOWN:
			switch ( (int)wParam )
			{
			case VK_UP:
				topAdjust ++ ;
				PutMessageValue( "Top Adjust" , topAdjust ) ;
				break ;

			case VK_DOWN:
				topAdjust -- ;
				PutMessageValue( "Top Adjust" , topAdjust ) ;
				break ;
			}
			break ;

		case WM_COMMAND:
			if ( (LOWORD(wParam) >= ID_VIDEOMODE0       ) &&
				 (LOWORD(wParam) <  ID_VIDEOMODE0+VMODES) )
			{				
				screenSize = LOWORD(wParam) - ID_VIDEOMODE0 ;
				Setup_MakeSetup(1,0,1) ;
				break ;
			}

			switch ( (UINT)LOWORD(wParam) )
			{
			case ID_PAL:		palSecam = 0 ;	Setup_UpdateMenus() ;	break ;
			case ID_SECAM:		palSecam = 1 ;	Setup_UpdateMenus() ;	break ;

			case ID_IDLE:		priority = 0 ;	Setup_MakeSetup(0,1,0) ;	break ;
			case ID_NORMAL:		priority = 1 ;	Setup_MakeSetup(0,1,0) ;	break ;
			case ID_HIGH:		priority = 2 ;	Setup_MakeSetup(0,1,0) ;	break ;
			case ID_REALTIME:	priority = 3 ;	Setup_MakeSetup(0,1,0) ;	break ;

			case ID_RGB555:		rgb565 = FALSE ;	Setup_UpdateMenus() ;	break ;
			case ID_RGB565:		rgb565 =  TRUE ;	Setup_UpdateMenus() ;	break ;

			case ID_SHOWCHECK:	showCheckArea = !showCheckArea ;	Setup_UpdateMenus() ;	break ;

			case ID_ALWAYSONTOP:	alwaysOnTop = !alwaysOnTop ;	Setup_UpdateMenus() ;	Layout( hWnd ) ;	break ;

			case ID_USEBT:
				if ( haveBT848 )
				{
					useBT848 = !useBT848 ;
					Setup_MakeSetup(0,0,1) ;
					PutMessageCondBanner( useBT848 , "BTChip phase bugfix" , "BTChip phase without bugfix" ) ;
				}
				break ;

			case ID_AUTOFIND:
				autoFindCScope = TRUE ;
				clearBkg = TRUE ;
				if ( descramblingNow )
					switchOffAFCS = FALSE ;
				PutMessageBanner( "Cinemascope found" ) ;
				break ;

			case ID_LOADCONFIG:
				Setup_GetINI() ;
				Setup_MakeSetup(1,1,1) ;
				PutMessageBanner( "Configuration loaded" ) ;
				break ;

			case ID_SAVECONFIG:
				Setup_SetINI() ;
				PutMessageBanner( "Configuration saved" ) ;
				break ;

			case ID_SPEED:
				Setup_DoSpeedSetup() ;
				break ;
			
			case ID_DECRYPTVIDEO:		
				decrypt     = !decrypt ;		Setup_UpdateMenus() ;	
				PutMessageCondBanner( decrypt , "Decrypting video" , "TV-mode" ) ;	break ;
			case ID_DECRYPTCOLOR:		
				color	    = !color ;			Setup_UpdateMenus() ;	
				PutMessageCondBanner( color , "Color" , "Black/White" ) ;	break ;
			case ID_HQ:					
				highQuality = !highQuality ;	Setup_MakeSetup(0,0,1) ;
				PutMessageCondBanner( highQuality , "High Quality color" , "Low Quality color" ) ;
				break ;			
			case ID_INTERLACE:			
				interlace   = 1 - interlace ;	Setup_UpdateMenus() ;
				PutMessageCondBanner( interlace , "Interlaced" , "Non-interlaced" ) ;	break ;
			case ID_SHOWSTATISTICS:		
				statistics  = !statistics ;		Setup_UpdateMenus() ;	break ;

			case ID_CORRECTION:		phaseCorrector = !phaseCorrector ;
				PutMessageBanner( "Changing PAL phase" ) ;	break ;

			case ID_HELPBOX:
				Setup_DoHelp() ;				
				break ;

			case ID_EXITNOW:
				DestroyWindow( hWnd ) ;
				break ;

			case ID_DECRYPTSOUND:
				if ( canSound )
				{
					useSound = !useSound ;
					if ( useSound )
						SoundCapture_Play() ;
					else
						SoundCapture_Stop() ;
					Setup_UpdateMenus() ;
					PutMessageCondBanner( useSound , "Sound ON" , "Sound OFF" ) ;	break ;
				}
				break ;

			case ID_START:
				SetWindowText( hMainWnd , "Initializing Nagravision descrambler..." ) ;
				if ( !Decoder_InitDescrambler() )
					exit( -1 ) ;
				SetWindowText( hMainWnd, "Initializing video hardware..." ) ;
				hCapWnd = capCreateCaptureWindow( appName , WS_CHILD , 0 , 0 , 1 , 1 , hMainWnd , 1 ) ; // Size does not matter
				if( !hCapWnd )
				{
					MessageBox( hMainWnd , "Can't create capture window" , appName , MB_ICONEXCLAMATION ) ;
					exit( -1 ) ;
				}
				if( !VideoCapture_Init( whichDriver ) )
				{
					MessageBox( hMainWnd , "Can't initialize video capturer" , appName , MB_ICONEXCLAMATION ) ;
					exit( -1 ) ;
				}				
				SetWindowText( hMainWnd, "Initializing sound capture..." ) ;
				SoundCapture_Open() ;
				if ( useSound )
					SoundCapture_Play() ;
				started = TRUE ;
				SetWindowText( hMainWnd , appName ) ;
				Setup_MakeSetup(1,1,1) ;
				sprintf( lastMsg , "[F1] -> Help / [R] -> Color Correction / UP/DOWN -> Top Adjust" ) ;
				PutMessage() ;
				break ;
			}
			break ;

		case WM_DESTROY:
			SoundCapture_Close() ;
			VideoCapture_Close() ;
			Sleep( 500 ) ;
			DirectDraw_Close() ;
			PostQuitMessage( 0 ) ;
			break ;

		default:
			return DefWindowProc( hWnd , uMsg , wParam , lParam ) ;
	}

	return 0 ;
}

//-----------------------------------------------------------------------------
//
// WinMain:
//
//  And last, but not least...  The MAIN PROGRAM.
//
int WINAPI WinMain
(
   HINSTANCE   hCurrInstance ,
   HINSTANCE   hPrevInstance ,
   LPSTR       lpszCmdLine ,
   int         nCmdShow
)
{
	WNDCLASS sWinClass ;
	MSG msg ;

#if DEBUGFILE
	debugFile = fopen( "Debug.txt" , "wt" ) ;
#endif
		
	hInstance = hCurrInstance ;

	hFont = CreateFont( 8 , 0 , 0 , 0 , FW_NORMAL , 
						FALSE , FALSE , FALSE , 
						ANSI_CHARSET , OUT_DEFAULT_PRECIS , 
						CLIP_DEFAULT_PRECIS , NONANTIALIASED_QUALITY , 
						VARIABLE_PITCH , "Arial" ) ;

	VideoCapture_FindDrivers() ;

	hAccel = LoadAccelerators( hInstance , MAKEINTRESOURCE(IDR_ACCELERATOR) ) ;

	sWinClass.lpszClassName = appName ;
	sWinClass.hInstance     = hInstance ;
	sWinClass.lpfnWndProc   = MainWnd ;
	sWinClass.hCursor       = NULL ;
	sWinClass.hIcon         = LoadIcon( hInstance , MAKEINTRESOURCE(IDI_APPICON ) ) ;
	sWinClass.lpszMenuName  = MAKEINTRESOURCE(IDR_MAINMENU) ;
	sWinClass.hbrBackground = GetStockObject(BLACK_BRUSH) ;
	sWinClass.style         = CS_HREDRAW | CS_VREDRAW ;
	sWinClass.cbClsExtra    = 0 ;
	sWinClass.cbWndExtra    = 0 ;

	if( !RegisterClass( &sWinClass ) )
	{
		MessageBox( HWND_DESKTOP , "Can't register window class" , appName , MB_ICONEXCLAMATION ) ;
		return FALSE ;
	}        

	hMainWnd = CreateWindowEx(	0 ,
								appName ,
								appName ,
								dwGDIWindowStyle,
								CW_USEDEFAULT ,
								CW_USEDEFAULT ,
								384 ,
								HEIGHT ,
								HWND_DESKTOP ,
								NULL ,
								hInstance ,
								NULL ) ;

	if( !hMainWnd )
	{
		MessageBox( hMainWnd , "Can't create window" , appName , MB_ICONEXCLAMATION ) ;
		return FALSE ;
	}
	
	Setup_GetINI() ;
	Setup_PrepareMenus() ;
	Setup_UpdateMenus() ;

	if ( !Setup_DoInitialSetup() )
		return TRUE ;

	Setup_MakeSetup(0,0,0) ;

	Layout( hMainWnd ) ;
	
	SendMessage( hMainWnd , WM_COMMAND , MAKEWPARAM(ID_START,0) , 0 ) ;

	while( GetMessage( &msg , NULL , 0 , 0 ) ) 
	{		 
		if ( !TranslateAccelerator( hMainWnd , hAccel , &msg ) )
		{
			TranslateMessage( &msg ) ;
			DispatchMessage( &msg ) ;
		}
	}

#if DEBUGFILE
	fclose( debugFile ) ;
#endif

	return msg.wParam ;
}
