#pragma warning(disable: 4056 4201 4514)

#include <windows.h>
#include <math.h>

#include "RGBSysterHQ.h"
#include "Debug.h"
#include "resource.h"


const int SIZE_FARBKORREKTUR = 345;


CRGBSysterHQ::CRGBSysterHQ
(
   int   nBreiteQuelle,
   int   nHoeheZiel,
   int   nTestzeilen,
   bool  bRGB15,
   bool  bPalPlus,
   bool  bSchwarzeZeile,
   bool  bSecam
) : CSyster(nTestzeilen, bSchwarzeZeile)
{
   m_nBreiteQuelle  = nBreiteQuelle;
   m_bPALplus       = bPalPlus;

   m_nHoeheZiel     = nHoeheZiel;
   m_bRGB15         = bRGB15;
   m_bSecam         = bSecam;
	m_pCromaPuffer   = NULL;
   m_pLinePuffer    = NULL;
	m_pFarbkorrektur = NULL;

   // Speicher fr die Farbe einer Zeile
   m_pCromaPuffer = new char[m_nBreiteQuelle*2];
   if(m_pCromaPuffer == NULL)
   {
      MessageBox(IDS_ERR_MEMORY);
      m_bError = true;
      return;
   }

   // Speicher fr eine Zeile
   m_pLinePuffer = new DWORD[m_nBreiteQuelle/2];
   if(m_pLinePuffer == NULL)
   {
      MessageBox(IDS_ERR_MEMORY);
      m_bError = true;
      return;
   }

   // Speicher fr die Farbkorrekturtabelle
   m_pFarbkorrektur = new short[SIZE_FARBKORREKTUR*4*4];
   if(m_pFarbkorrektur == NULL)
   {
      MessageBox(IDS_ERR_MEMORY);
      m_bError = true;
      return;
   }

   BaueFarbkorrekturTabelle();
}


CRGBSysterHQ::~CRGBSysterHQ()
{
   if(m_pCromaPuffer != NULL)
      delete[] m_pCromaPuffer;

   if(m_pLinePuffer != NULL)
      delete[] m_pLinePuffer;

   if(m_pFarbkorrektur != NULL)
      delete[] m_pFarbkorrektur;
}


void CRGBSysterHQ::BaueFarbkorrekturTabelle()
{
   int      nZeilenAbstand;
   double   dPhase;
   short   *pTabelle;

   // Das sind die -90 Grad Phasendrehung pro Zeile plus eine 
   // Schwingung Offset pro Vollbild.
   const double dPIHalbe  = -1.56074323;
   const double dFarbton  =  0.321699087;
   const double dUVFaktor =  0.708661417;
 
   pTabelle = m_pFarbkorrektur;

   for(nZeilenAbstand=0; nZeilenAbstand<SIZE_FARBKORREKTUR; nZeilenAbstand++)
   {
      dPhase = (double)(nZeilenAbstand-(SIZE_FARBKORREKTUR-1))*dPIHalbe + dFarbton;

      *pTabelle++ = (short)( cos(dPhase) * 8192.0);
      *pTabelle++ = (short)( sin(dPhase) * 8192.0);
      *pTabelle++ = (short)(-sin(dPhase) * 8192.0 * dUVFaktor);
      *pTabelle++ = (short)( cos(dPhase) * 8192.0 * dUVFaktor);

      // nQuelleInvers
      *pTabelle++ = (short)( cos(dPhase) * 8192.0);
      *pTabelle++ = (short)(-sin(dPhase) * 8192.0);
      *pTabelle++ = (short)(-sin(dPhase) * 8192.0 * dUVFaktor);
      *pTabelle++ = (short)(-cos(dPhase) * 8192.0 * dUVFaktor);

      // nZielInvers
      *pTabelle++ = (short)( cos(dPhase) * 8192.0);
      *pTabelle++ = (short)( sin(dPhase) * 8192.0);
      *pTabelle++ = (short)( sin(dPhase) * 8192.0 * dUVFaktor);
      *pTabelle++ = (short)(-cos(dPhase) * 8192.0 * dUVFaktor);

      // nQuelleInvers + nZielInvers
      *pTabelle++ = (short)( cos(dPhase) * 8192.0);
      *pTabelle++ = (short)(-sin(dPhase) * 8192.0);
      *pTabelle++ = (short)( sin(dPhase) * 8192.0 * dUVFaktor);
      *pTabelle++ = (short)( cos(dPhase) * 8192.0 * dUVFaktor);
   }
}


void CRGBSysterHQ::DecodeColor
(
   int **pLinesAddress,
   int  *pSortLines,
   int  *pDestination,
   int   nBreiteZiel,
   int   nErsteSichtbareZeile,
   int   nLetzteSichtbareZeile,
   bool  bBild,
   bool  bDecode,
   bool  bPhase
)
{
   int    nZielInvers;
   int    nQuelleInvers;
   int    nZeileZlr;
   int    nZeileZiel;
   int    nZeileQuelle;
   int    nScreenLine;
   int    nAbstandLetztesHalbbild;
   DWORD *pQuelle;
   DWORD *pZiel;
   short *pColorFaktor;

   // Linebuffer lschen
   ZeroMemory(m_pCromaPuffer, m_nBreiteQuelle*2);

   // Zeilen sortieren
   nScreenLine = nErsteSichtbareZeile*m_nHoeheZiel/ZEILENZAHL;
   for(nZeileZlr=nErsteSichtbareZeile; nZeileZlr<nLetzteSichtbareZeile; nZeileZlr++)
   {
      if(bDecode == false)
      {
         // Korrekturfaktor bestimmen
         pColorFaktor = m_pFarbkorrektur+312*4*4;

         // Adresse der Zeile holen
         pQuelle = (DWORD*)pLinesAddress[nZeileZlr];
      }
      else
      {
         //
         // Secam Decodierung von Xetra
         //
         if(m_bSecam)
         {
            // Korrekturfaktor bestimmen
            pColorFaktor = m_pFarbkorrektur+312*4*4;

            // Adresse der Zeile holen
            pQuelle = (DWORD*)pLinesAddress[pSortLines[nZeileZlr]];
         }
         else
         {
            // Abhngig ob es sich ums erste oder zweite Halbild handelt
            if(m_bPALplus)
               nAbstandLetztesHalbbild = 25;
            else
               nAbstandLetztesHalbbild = 26;

            // Hier ist die Zeile bei der Ausgabe
            nZeileZiel = nZeileZlr;

            // Hier ist die Zeile im Decodierten Bild
            nZeileQuelle = pSortLines[nZeileZiel];

            // Adresse der Zeile holen
            pQuelle = (DWORD*)pLinesAddress[nZeileQuelle];

            // Kommt die Zeile aus dem letzten Halbbild
            if(nZeileQuelle < 32)
               nZeileQuelle -= nAbstandLetztesHalbbild;

            // Ist die Zielzeile invers
            nZielInvers = nZeileZiel & 1;

            // Ist die Quellzeile invers
            nQuelleInvers = nZeileQuelle & 1;

            // Da waren ja noch die 90 Phasendrehung des Farbtrgers nach
            // jedem Bild. Nach vier Bildern fngt alles wieder von vorne an.
            if(bBild == bPhase)
            {
               nQuelleInvers ^= 1;
               nZielInvers   ^= 1;
            }

            // Abstand der Zeilen
            nZeileQuelle -= nZeileZiel;

            // Korrekturfaktor bestimmen
            pColorFaktor = m_pFarbkorrektur+((nZeileQuelle+312)*4+nQuelleInvers+nZielInvers*2)*4;
         }
      }

      // Die Bild auf Bildschirmhhe strecken
      bool bErsterAufruf = true;

      while(nScreenLine < nZeileZlr*m_nHoeheZiel/ZEILENZAHL)
      {
         pZiel = (DWORD*)(pDestination+nScreenLine*nBreiteZiel/4);
         nScreenLine++;

         // Wurde die Zeile bereits berechnet ?
         if(bErsterAufruf == true)
         {
            bErsterAufruf = false;

            // Nein, wird die Zeile noch einmal bentigt ?
            if(nScreenLine < nZeileZlr*m_nHoeheZiel/ZEILENZAHL)
            {
               // Ja, dann in den LinePuffer schreiben
               if(m_bRGB15 == true)
                  ZeileAusgebenRGB15(pQuelle, m_pLinePuffer, pColorFaktor);
               else
                  ZeileAusgebenRGB16(pQuelle, m_pLinePuffer, pColorFaktor);

               // Anschlieend auf den Bildschirm ausgeben
               CopyMemory(pZiel, m_pLinePuffer, m_nBreiteQuelle*2);
            }
            else
            {
               // Nein, dann sofort ausgeben
               if(m_bRGB15 == true)
                  ZeileAusgebenRGB15(pQuelle, pZiel, pColorFaktor);
               else
                  ZeileAusgebenRGB16(pQuelle, pZiel, pColorFaktor);
            }
         }
         else
         {
            // Ja, aus dem LinePuffer ausgeben
            CopyMemory(pZiel, m_pLinePuffer, m_nBreiteQuelle*2);
         }
      }
   }
}


void inline CRGBSysterHQ::ZeileAusgebenRGB16
(
   DWORD *pQuelle,
   DWORD *pZiel,
   short *pColorFaktor
)
{
   int   nBreiteQuelle = m_nBreiteQuelle;
   char *pCromaPuffer  = m_pCromaPuffer;

   const __int64  nMaske    = 0x00ff00ff00ff00ff;
   const __int64  nMinus128 = 0x8000800080008000;
   const __int64  nFaktoren = 0xf2fe20491989f9bf;

   _asm
   {
      push esi                         // Register retten
      push edi

      mov edi,dword ptr [pColorFaktor]
      mov edx,pQuelle
      mov ecx,nBreiteQuelle
      movq mm7,qword ptr [edi]         // mm7 = *pColorFaktor (64Bit)
      mov edi,dword ptr [pCromaPuffer]
      movq mm4,qword ptr[nMaske]       // mm4 = 0x00ff00ff00ff00ff
      shr ecx,02                       // nBreiteQuelle /= 4
      mov ebx,pZiel
      movq mm0,qword ptr [edx]         // mm0 = *pQuelle 
      add edx,8                        // pQuelle += 8
      jmp StartPixel                   // Ab hier gehts in die Schleife fr alle Pixel

   align 16

   NaechsterPixel:
      movq mm0,qword ptr[edx]          // mm0 = *pQuelle 
      add edx,8                        // pQuelle += 8
      add ebx,8                        // pZiel += 8
         
   StartPixel:
      movq mm2,mm0
      psubb mm0,qword ptr[nMinus128]   // U-128 V-128
      psraw mm0,0x08
      pand mm2,mm4                     // UV ausmaskieren
      movq mm1,mm0
      punpckldq mm0,mm0
      psrlw mm2,01
      pmaddwd mm0,mm7                  // mm0 += *pColorFaktor
      movq mm6,qword ptr[edi+8*ecx-8]  // mm6 = pLinebuffer+nBreiteQuelle*8
      punpckhdq mm1,mm1
      movq mm5,mm2
      pmaddwd mm1,mm7
      punpcklbw mm2,mm2
      psrad mm0,11
      psrad mm1,11
      packssdw mm0,mm1
      movd eax,mm0
      packssdw mm1,mm1
      movd esi,mm1
      paddsw mm0,mm6
      punpckhbw mm5,mm5
      movq mm6,mm0
      movq mm3,qword ptr[nFaktoren]
      punpckldq mm6,mm6
      pmulhw mm6,mm3
      punpckhdq mm0,mm0
      pmulhw mm0,mm3
      movq mm1,mm2
      psllq mm2,0x08
      movq mm3,mm6
      psrlq mm6,0x30
      por mm2,mm1
      paddsw mm6,mm3
      movq mm3,mm5
      psllq mm5,0x08
      packsswb mm6,mm6
      movq mm1,mm0
      psrlq mm0,0x30
      por mm5,mm3
      paddsw mm0,mm1
      pxor mm1,mm1
      packsswb mm0,mm0
      pxor mm3,mm3
      paddsb mm5,mm0
      paddsb mm2,mm6
      pcmpgtb mm3,mm5
      pandn mm3,mm5
      pcmpgtb mm1,mm2
      movq mm0,mm3
      pand mm3,mm4
      psrld mm0,0x0A
      pandn mm1,mm2
      pslld mm0,0x1B
      movq mm5,mm3
      psrlw mm3,2
      movq mm2,mm1
      psrld mm5,1
      pand mm1,mm4
      pslld mm5,0x15
      por mm5,mm0
      movq mm0,mm1
      psrlw mm1,2
      por mm5,mm3
      psrld mm0,1
      mov dword ptr [edi+8*ecx-8],eax
      pslld mm0,0x15
      mov dword ptr [edi+8*ecx-4],esi
      psrad mm5,0x10
      por mm1,mm0
      psrld mm2,0x0A
      pslld mm2,0x1B
      por mm2,mm1
      psrad mm2,0x10
      dec ecx                          // nBreiteQuelle--
      packssdw mm2,mm5
      movq qword ptr[ebx],mm2          // Pixel ausgeben
      jne near NaechsterPixel          // Sind noch weitere Pixel in dieser Zeile

      emms                             // MMX zurcksetzen

      pop edi                          // Register zurckspeichern
      pop esi
   }
}


void inline CRGBSysterHQ::ZeileAusgebenRGB15
(
   DWORD *pQuelle,
   DWORD *pZiel,
   short *pColorFaktor
)
{
   int   nBreiteQuelle = m_nBreiteQuelle;
   char *pCromaPuffer  = m_pCromaPuffer;

   const __int64  nMaske    = 0x00ff00ff00ff00ff;
   const __int64  nMinus128 = 0x8000800080008000;
   const __int64  nFaktoren = 0xf2fe20491989f9bf;

   // Die Daten kommen im YUY2 Format:
   //
   //  0xFF000000 0x00FF0000 0x000FF00 0x0000FF
   //  31--V0--24 23--Y1--16 15--U0--8 7--Y0--0
   //  MSB                                  LSB
   //
   // U0 und V0 gelten jeweils fr Y0 und Y1
   //
   _asm
   {
      push esi                         // Register retten
      push edi

      mov edi,dword ptr [pColorFaktor]
      mov edx,pQuelle
      mov ecx,nBreiteQuelle
      movq mm7,qword ptr [edi]         // mm7 = *pColorFaktor (64Bit)
      mov edi,dword ptr [pCromaPuffer]
      movq mm4,qword ptr[nMaske]       // mm4 = 0x00ff00ff00ff00ff
      shr ecx,02                       // nBreiteQuelle /= 4
      mov ebx,pZiel
      movq mm0,qword ptr [edx]         // mm0 = *pQuelle 
      add edx,0x00000008               // pQuelle += 8
      jmp StartPixel                   // Ab hier gehts in die Schleife fr alle Pixel

   align 16

   NaechsterPixel:
      movq mm0,qword ptr[edx]          // mm0 = *pQuelle 
      add edx,8                        // pQuelle += 8
      add ebx,8                        // pZiel += 8
         
   StartPixel:
      movq mm2,mm0
      psubb mm0,qword ptr[nMinus128]   // U-128 V-128
      psraw mm0,8
      pand mm2,mm4
      movq mm1,mm0
      punpckldq mm0,mm0
      psrlw mm2,01
      pmaddwd mm0,mm7
      movq mm6,qword ptr[edi+8*ecx-8]
      punpckhdq mm1,mm1
      movq mm5,mm2
      pmaddwd mm1,mm7
      punpcklbw mm2,mm2
      psrad mm0,11
      psrad mm1,11
      packssdw mm0,mm1
      movd eax,mm0
      packssdw mm1,mm1
      movd esi,mm1
      paddsw mm0,mm6
      punpckhbw mm5,mm5
      movq mm6,mm0
      movq mm3,qword ptr[nFaktoren]
      punpckldq mm6,mm6
      pmulhw mm6,mm3
      punpckhdq mm0,mm0
      pmulhw mm0,mm3
      movq mm1,mm2
      psllq mm2,0x08
      movq mm3,mm6
      psrlq mm6,0x30
      por mm2,mm1
      paddsw mm6,mm3
      movq mm3,mm5
      psllq mm5,0x08
      packsswb mm6,mm6
      movq mm1,mm0
      psrlq mm0,0x30
      por mm5,mm3
      paddsw mm0,mm1
      pxor mm1,mm1
      packsswb mm0,mm0
      pxor mm3,mm3
      paddsb mm5,mm0
      paddsb mm2,mm6
      pcmpgtb mm3,mm5
      pandn mm3,mm5
      pcmpgtb mm1,mm2
      movq mm0,mm3
      pand mm3,mm4
      psrld mm0,0x0A
      pandn mm1,mm2
      pslld mm0,0x1A
      movq mm5,mm3
      psrlw mm3,02
      movq mm2,mm1
      psrld mm5,02
      pand mm1,mm4
      pslld mm5,0x15
      por mm5,mm0
      movq mm0,mm1
      psrlw mm1,2
      por mm5,mm3
      psrld mm0,2
      mov dword ptr [edi+8*ecx-8],eax
      pslld mm0,0x15
      mov dword ptr [edi+8*ecx-4],esi
      psrad mm5,0x10
      por mm1,mm0
      psrld mm2,0x0A
      pslld mm2,0x1A
      por mm2,mm1
      psrad mm2,0x10
      dec ecx
      packssdw mm2,mm5
      movq qword ptr[ebx],mm2          // Pixel ausgeben
      jne near NaechsterPixel          // Sind noch weitere Pixel in dieser Zeile

      emms                             // MMX zurcksetzen

      pop edi                          // Register zurckspeichern
      pop esi
   }
}