//---------------------------------//
// SetupDialog.c   by:   KennyWood //
//---------------------------------//

#include "resource.h"
#include "NagraView.h"
#include "DirectDraw.h"
#include "Capture.h"
#include "BTChip.h"

//-----------------------------------------------------------------------------

const char ini_name [] = "NagraView.ini" ;

//-----------------------------------------------------------------------------

char priorities [4][16] = { "IDLE" , "NORMAL" , "HIGH" , "REALTIME" } ;

struct
{
	int  displayWidth  ;
	int  displayHeight ;
	int  captureWidth  ;
	int  displayBPP    ;
	BOOL fullScreen    ;
} videomode [VMODES] = {	{ 192 , 144 , 192 , 16 , FALSE } ,
							{ 384 , 288 , 384 , 16 , FALSE } ,
							{ 512 , 384 , 512 , 16 , FALSE } ,
							{ 640 , 480 , 640 , 16 , FALSE } ,
							{ 768 , 576 , 768 , 16 , FALSE } ,

							{ 352 , 288 , 352 , 16 , FALSE } ,
	
							{ 320 , 200 , 192 , 16 , TRUE } ,
							{ 320 , 240 , 384 , 16 , TRUE } ,
							{ 512 , 384 , 512 , 16 , TRUE } ,
							{ 640 , 480 , 640 , 16 , TRUE } ,
							{ 800 , 600 , 768 , 16 , TRUE } } ;

//-----------------------------------------------------------------------------

struct
{
	char name [ 32 ] ;
	int  key ;
} channelType [CHANNELS] = {	{ "Use Key.txt"      , 0 } ,
								{ "Canal + Spain"    , 1 } ,
								{ "Canal + France"   , 2 } ,
								{ "Paris Premiere"   , 1 } ,
								{ "Canal J"          , 1 } ,
								{ "Canal Jimmy"      , 1 } ,
								{ "MCM"              , 1 } ,
								{ "Planete"          , 1 } ,
								{ "Eurosport"        , 1 } ,
								{ "Cine Cinema 1"    , 1 } ,
								{ "Cine Cinefil"     , 1 } ,
								{ "Canal + Horizons" , 1 } ,
								{ "Canal + Pologne"  , 1 } ,
								{ "Premiere"         , 1 } ,
								{ "Playboy TV"       , 1 } ,
								{ "Maxi TV"          , 1 } ,
								{ "SuperSport"       , 1 } } ;

//-----------------------------------------------------------------------------

struct
{
	char name [ 32 ] ;
	int  bind ;
} cardType [CARDTYPES] = {	{ "Use Bind.txt" , 0 } ,
							{ "miroPCTV"     , 1 } ,
							{ "Test card #2" , 2 } ,
							{ "Test card #3" , 3 } ,
							{ "Test card #4" , 4 } ,
							{ "Test card #5" , 5 } ,
							{ "Test card #6" , 6 } ,
							{ "Aver Media"   , 7 } } ;

//-----------------------------------------------------------------------------
//
// Setup_CheckParams:
//
//  Clip some values for sanity.
//
void Setup_CheckParams ()
{
	if ( checkPairs      > MAXCHECKPAIRS  )   checkPairs      = MAXCHECKPAIRS  ;
	if ( checkPairs      < 1              )   checkPairs      = 1              ;
	if ( checkWidth      > MAXCHECKWIDTH  )   checkWidth      = MAXCHECKWIDTH  ;
	if ( checkWidth      < 1              )   checkWidth      = 1              ;
	if ( checkLevels     > MAXCHECKLEVELS )   checkLevels     = MAXCHECKLEVELS ;
	if ( checkLevels     < 1              )   checkLevels     = 1              ;
	if ( whichDriver     > numDrivers     )   whichDriver     = 0              ;
	if ( whichDriver     < 0              )   whichDriver     = 0              ;
	if ( card            > CARDTYPES      )   card            = CARDTYPES-1    ;
	if ( card            < 0              )   card            = 0              ;
	if ( channel         > CHANNELS       )   channel         = CHANNELS-1     ;
	if ( channel         < 0              )   channel         = 0              ;
}

//-----------------------------------------------------------------------------
//
// Setup_PrepareMenus:
//
//  Here we prepare the menus when the application starts.
//
void Setup_PrepareMenus ()
{
	int   i ;
	HMENU hMainMenu ;
	HMENU hDisplayMenu ;
	HMENU hVideoModeMenu ;
	char  text [ 256 ] ;

	hMainMenu      = GetMenu( hMainWnd ) ;
	hDisplayMenu   = GetSubMenu( hMainMenu , 2 ) ;
	hVideoModeMenu = GetSubMenu( hDisplayMenu , 8 ) ;
	for ( i = 0 ; i < VMODES ; i ++ )
	{
		sprintf( text , "%s %dx%dx%d" , videomode[i].fullScreen?"FullScreen\t":"Window\t" , videomode[i].displayWidth , videomode[i].displayHeight , videomode[i].displayBPP ) ;
		AppendMenu( hVideoModeMenu , MF_STRING , ID_VIDEOMODE0+i , text ) ;
	}
}

//-----------------------------------------------------------------------------
//
// Setup_UpdateMenus:
//
//  Here we update the menus, checkign or greying options, etc...
//
void Setup_UpdateMenus ()
{
	int   i ;
	HMENU hMainMenu ;
	HMENU hSystemMenu ;
	HMENU hPriorityMenu ;
	HMENU hCardMenu ;
	HMENU hDisplayMenu ;
	HMENU hVideoModeMenu ;	
	HMENU hSoundMenu ;
	HMENU hAlgorithmMenu ;
	HMENU hPalSecamMenu ;

	hMainMenu     = GetMenu( hMainWnd ) ;
	hSystemMenu   = GetSubMenu( hMainMenu , 0 ) ;
	CheckMenuItem( hSystemMenu , ID_ALWAYSONTOP , alwaysOnTop ? MF_CHECKED : MF_UNCHECKED ) ;
	hPriorityMenu = GetSubMenu( hSystemMenu , 0 ) ;
	CheckMenuItem( hPriorityMenu , ID_IDLE     , priority == 0 ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hPriorityMenu , ID_NORMAL   , priority == 1 ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hPriorityMenu , ID_HIGH     , priority == 2 ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hPriorityMenu , ID_REALTIME , priority == 3 ? MF_CHECKED : MF_UNCHECKED ) ;	
	hCardMenu      = GetSubMenu( hMainMenu , 1 ) ;
	EnableMenuItem( hCardMenu , ID_USEBT , haveBT848?MF_ENABLED:MF_GRAYED ) ;
	CheckMenuItem( hCardMenu , ID_USEBT , useBT848 ? MF_CHECKED : MF_UNCHECKED ) ;
	hDisplayMenu   = GetSubMenu( hMainMenu , 2 ) ;
	hVideoModeMenu = GetSubMenu( hDisplayMenu , 8 ) ;
	for ( i = 0 ; i < VMODES ; i ++ )
		CheckMenuItem( hVideoModeMenu , ID_VIDEOMODE0+i , screenSize == i ? MF_CHECKED : MF_UNCHECKED ) ;
	hPalSecamMenu = GetSubMenu( hDisplayMenu , 10 ) ;
	CheckMenuItem( hPalSecamMenu , ID_PAL   , palSecam == USEPAL   ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hPalSecamMenu , ID_SECAM , palSecam == USESECAM ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hDisplayMenu , ID_DECRYPTVIDEO   , decrypt       ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hDisplayMenu , ID_DECRYPTCOLOR   , color         ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hDisplayMenu , ID_HQ             , highQuality   ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hDisplayMenu , ID_SHOWSTATISTICS , statistics    ? MF_CHECKED : MF_UNCHECKED ) ;	
	CheckMenuItem( hDisplayMenu , ID_RGB555         , !rgb565       ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hDisplayMenu , ID_RGB565         ,  rgb565       ? MF_CHECKED : MF_UNCHECKED ) ;
	CheckMenuItem( hDisplayMenu , ID_INTERLACE      , interlace     ? MF_CHECKED : MF_UNCHECKED ) ;
	hSoundMenu = GetSubMenu( hMainMenu , 3 ) ;
	CheckMenuItem( hSoundMenu , ID_DECRYPTSOUND , useSound ? MF_CHECKED : MF_UNCHECKED ) ;	
	hAlgorithmMenu = GetSubMenu( hMainMenu , 4 ) ;
	CheckMenuItem( hAlgorithmMenu , ID_SHOWCHECK      , showCheckArea ? MF_CHECKED : MF_UNCHECKED ) ;	
}

//-----------------------------------------------------------------------------
//
// Setup_SetINI:
//
//  Save the customized profile.
//
void Setup_SetINI ()
{
	char path_name     [ MAX_PATH ] ;
	char ini_full_name [ MAX_PATH ] ;
	char text [ 256 ] ;

	GetCurrentDirectory( sizeof(path_name) , path_name ) ;
	sprintf( ini_full_name , "%s\\%s" , path_name , ini_name ) ;

	Setup_CheckParams() ;

#define WProfile(v) \
	sprintf( text , "%d" , v ) ; \
	WritePrivateProfileString( appName , #v , text , ini_full_name )

	WProfile( decrypt         ) ;
	WProfile( color           ) ;
	WProfile( statistics      ) ;
	WProfile( useSound        ) ;
	WProfile( showCheckArea   ) ;
	WProfile( rgb565          ) ;
	WProfile( checkPairs      ) ;
	WProfile( checkWidth      ) ;
	WProfile( checkLevels     ) ;
	WProfile( priority        ) ;
	WProfile( screenSize      ) ;
	WProfile( whichDriver     ) ;
	WProfile( useBT848        ) ;
	WProfile( highQuality     ) ;
	WProfile( interlace       ) ;
	WProfile( palSecam        ) ;
	WProfile( alwaysOnTop     ) ;
	WProfile( channel         ) ;
	WProfile( card            ) ;
}

//-----------------------------------------------------------------------------
//
// Setup_MakeSetup:
//
//  Performs some aspects of the current configuration.
//
void Setup_MakeSetup ( BOOL changeVideo , BOOL changePriority , BOOL changeBT )
{
	int  Priority_ID [ 4 ] = {
								IDLE_PRIORITY_CLASS   ,
								NORMAL_PRIORITY_CLASS ,
								HIGH_PRIORITY_CLASS   ,
								REALTIME_PRIORITY_CLASS 
							} ;

	if ( changeVideo )
	{
		VideoCapture_Stop() ;
	}

	fullScreen    = videomode[screenSize].fullScreen ;
	displayWidth  = videomode[screenSize].displayWidth  ;
	displayHeight = videomode[screenSize].displayHeight ;
	captureWidth  = videomode[screenSize].captureWidth  ;
	displayBPP    = videomode[screenSize].displayBPP    ;

	if ( changeVideo )
	{
		ChangeVideoMode() ;
		VideoCapture_Play() ;
	}

	Setup_UpdateMenus() ;

	if ( changePriority )
	{
		SetPriorityClass( GetCurrentProcess() , Priority_ID[priority] ) ;
	}

	if ( changeBT )
	{
		if ( useBT848 )
			if ( highQuality )
				BTChip_BugFix_HighQuality() ;
			else
				BTChip_BugFix_LowQuality() ;
		else
			BTChip_Restore() ;
	}
}

//-----------------------------------------------------------------------------
//
// Setup_GetINI:
//
//  Retrieve the customized profile.
//
void Setup_GetINI ()
{
	char path_name     [ MAX_PATH ] ;
	char ini_full_name [ MAX_PATH ] ;
	char text [ 256 ] ;

	GetCurrentDirectory( sizeof(path_name) , path_name ) ;
	sprintf( ini_full_name , "%s\\%s" , path_name , ini_name ) ;

#define RProfile(v,d) \
	GetPrivateProfileString( appName , #v , d , text , sizeof(text) , ini_full_name ) ; \
	v = atoi(text) ; \
	if ( v < 0 ) \
		v = 0

	RProfile( decrypt         ,  "1" ) ;
	RProfile( color           ,  "1" ) ;
	RProfile( statistics      ,  "1" ) ;
	RProfile( useSound        ,  "1" ) ;
	RProfile( showCheckArea   ,  "0" ) ;
	RProfile( rgb565          ,  "1" ) ;
	RProfile( checkPairs      , "12" ) ;
	RProfile( checkWidth      , "24" ) ;
	RProfile( checkLevels     , "24" ) ;
	RProfile( priority        ,  "1" ) ;
	RProfile( screenSize      ,  "1" ) ;
	RProfile( whichDriver     ,  "0" ) ;
	RProfile( useBT848        ,  "1" ) ;
	RProfile( highQuality     ,  "1" ) ;
	RProfile( interlace       ,  "0" ) ;
	RProfile( palSecam        ,  "0" ) ;
	RProfile( alwaysOnTop     ,  "0" ) ;
	RProfile( channel         ,  "0" ) ;
	RProfile( card            ,  "0" ) ;

	Setup_CheckParams() ;
}

//-----------------------------------------------------------------------------
//
// Setup_InitialSetupProc:
//
//  Nice windoze message handling for our nice dialog box.
//
BOOL CALLBACK Setup_InitialSetupProc ( HWND hWnd , INT uMsg , WPARAM wParam , LPARAM lParam )
{
	int  i ;
	char text [ 256 ] ;

	switch( uMsg )
	{
		case WM_INITDIALOG:
			for ( i = 0 ; i < numDrivers ; i ++ )
			{
				VideoCapture_GetDriverName( i , text ) ;
				SendDlgItemMessage( hWnd , IDC_DRIVER , CB_ADDSTRING , 0 , (LPARAM)text );
			}
			SendDlgItemMessage( hWnd , IDC_DRIVER , CB_SETCURSEL , whichDriver , 0 ) ;		
			for ( i = 0 ; i < VMODES ; i ++ )
			{
				sprintf( text , "%s %dx%dx%d" , videomode[i].fullScreen?"FullScreen ":"Window " , videomode[i].displayWidth , videomode[i].displayHeight , videomode[i].displayBPP ) ;
				SendDlgItemMessage( hWnd , IDC_STARTVIDEOMODE , CB_ADDSTRING , 0 , (LPARAM)text );
			}
			SendDlgItemMessage( hWnd , IDC_STARTVIDEOMODE , CB_SETCURSEL , screenSize , 0 ) ;
			for ( i = 0 ; i < CHANNELS ; i ++ )
			{
				SendDlgItemMessage( hWnd , IDC_CHANNEL , CB_ADDSTRING , 0 , (LPARAM)channelType[i].name );
			}
			SendDlgItemMessage( hWnd , IDC_CHANNEL , CB_SETCURSEL , channel , 0 ) ;
			for ( i = 0 ; i < CARDTYPES ; i ++ )
			{
				SendDlgItemMessage( hWnd , IDC_CARD , CB_ADDSTRING , 0 , (LPARAM)cardType[i].name );
			}
			SendDlgItemMessage( hWnd , IDC_CARD , CB_SETCURSEL , card , 0 ) ;
			return TRUE ;
 			break ;

		case WM_COMMAND:
			switch( LOWORD(wParam) )
			{
			case IDOK:
				whichDriver = SendDlgItemMessage( hWnd , IDC_DRIVER         , CB_GETCURSEL , 0 , 0 ) ;
				screenSize  = SendDlgItemMessage( hWnd , IDC_STARTVIDEOMODE , CB_GETCURSEL , 0 , 0 ) ;
				channel     = SendDlgItemMessage( hWnd , IDC_CHANNEL        , CB_GETCURSEL , 0 , 0 ) ;
				keyID       = channelType[channel].key ;
				card        = SendDlgItemMessage( hWnd , IDC_CARD           , CB_GETCURSEL , 0 , 0 ) ;
				bindID      = cardType[card].bind ;
			case IDCANCEL:
				EndDialog( hWnd , LOWORD(wParam) ) ;
				return TRUE ;
				break ;
			}
			break ;
	}

	return FALSE ;
}

//-----------------------------------------------------------------------------
//
// Setup_DoInitialSetup:
//
//  Configuration of files and driver.
//
BOOL Setup_DoInitialSetup ()
{
	return (DialogBox( hInstance , 
					   MAKEINTRESOURCE(IDD_SETUP) , 
					   hMainWnd , 
					   (DLGPROC)Setup_InitialSetupProc ) == IDOK) ;
}

//-----------------------------------------------------------------------------
//
// Setup_SpeedSetupProc:
//
//  Nice windoze message handling for our nice dialog box.
//
BOOL CALLBACK Setup_SpeedSetupProc ( HWND hWnd , INT uMsg , WPARAM wParam , LPARAM lParam )
{
	char text [ 256 ] ;

	switch( uMsg )
	{
		case WM_INITDIALOG:
			sprintf( text , "%d" , checkPairs ) ;
			SetWindowText( GetDlgItem(hWnd,IDC_CHECKPAIRS ) , text ) ;
			sprintf( text , "%d" , checkWidth ) ;
			SetWindowText( GetDlgItem(hWnd,IDC_CHECKWIDTH ) , text ) ;
			sprintf( text , "%d" , checkLevels ) ;
			SetWindowText( GetDlgItem(hWnd,IDC_CHECKLEVELS) , text ) ;
			return TRUE ;
 			break ;

		case WM_COMMAND:
			switch( LOWORD(wParam) )
			{
			case IDOK:
				GetWindowText( GetDlgItem(hWnd,IDC_CHECKPAIRS ) , text , sizeof(text) ) ;
				checkPairs  = atoi(text) ;
				GetWindowText( GetDlgItem(hWnd,IDC_CHECKWIDTH ) , text , sizeof(text) ) ;
				checkWidth  = atoi(text) ;	
				GetWindowText( GetDlgItem(hWnd,IDC_CHECKLEVELS) , text , sizeof(text) ) ;
				checkLevels = atoi(text) ;	
			case IDCANCEL:
				EndDialog( hWnd , LOWORD(wParam) ) ;
				return TRUE ;
				break ;
			}
			break ;
	}

	return FALSE ;
}

//-----------------------------------------------------------------------------
//
// Setup_DoSpeedSetup:
//
//  Configuration of algorithm speed/quality.
//
BOOL Setup_DoSpeedSetup ()
{
	return (DialogBox( hInstance , 
					   MAKEINTRESOURCE(IDD_SPEED) , 
					   hMainWnd , 
					   (DLGPROC)Setup_SpeedSetupProc ) == IDOK) ;
}

//-----------------------------------------------------------------------------
//
// Setup_HelpProc:
//
//  Nice windoze message handling for our nice dialog box.
//
BOOL CALLBACK Setup_HelpProc ( HWND hWnd , INT uMsg , WPARAM wParam , LPARAM lParam )
{
	switch( uMsg )
	{
		case WM_INITDIALOG:
			return TRUE ;
 			break ;

		case WM_COMMAND:
			switch( LOWORD(wParam) )
			{
			case IDOK:
				EndDialog( hWnd , LOWORD(wParam) ) ;
				return TRUE ;
				break ;
			}
			break ;
	}

	return FALSE ;
}

//-----------------------------------------------------------------------------
//
// Setup_DoHelp:
//
//  Just the help.
//
void Setup_DoHelp ()
{
	DialogBox( hInstance , 
			   MAKEINTRESOURCE(IDD_HELP) , 
			   hMainWnd , 
			   (DLGPROC)Setup_HelpProc ) ;
}
