//----------------------------------//
// SoundCapture.c   by:   KennyWood //
//----------------------------------//

#include <math.h>
#include "Capture.h"

//-----------------------------------------------------------------------------

#define BUFFERS    25			// Splitted buffer chunks.  
								// Latency is approx. 1/BUFFERS of second
#define MODFREQ    12800		// Frequency of audio signal modulation
#define SOUNDFREQ  44100/*(MODFREQ * 2)*/
#define BPSAMPLE   2
#define BUFLEN    ((SOUNDFREQ * BPSAMPLE) / BUFFERS)

//-----------------------------------------------------------------------------

SHORT cos_tab [ SOUNDFREQ ] ;

#define FIX_SHIFT 15

//-----------------------------------------------------------------------------

WAVEHDR  waveIHeader [ BUFFERS ] ;
WAVEHDR  waveOHeader [ BUFFERS ] ;
BYTE     bufferI     [ BUFFERS ][ BUFLEN ] ;
BYTE     bufferO     [ BUFFERS ][ BUFLEN ] ;
HWAVEOUT hWaveO ;
HWAVEIN  hWaveI ;

//-----------------------------------------------------------------------------
//
// WaveStreamCallback:
//
//  Here we decode a chunk of sound
//
void CALLBACK WaveStreamCallback ( HWAVEIN hwi , UINT uMsg , 
								   DWORD dwInstance , DWORD dwParam1 , DWORD dwParam2 )
{
	LPWAVEHDR wI , wO ;
	PSHORT    src , dst ;
	int       i ;

	if ( !useSound )
		return ;

	if ( uMsg == WIM_DATA )
	{
		wI = (LPWAVEHDR) dwParam1 ;

		if ( wI -> dwBytesRecorded == wI -> dwBufferLength )
		{
			wO = (LPWAVEHDR) wI -> dwUser ;

			src = (PSHORT) wI -> lpData ;
			dst = (PSHORT) wO -> lpData ;

/*			// THIS WORKED WHEN SOUNDFREQ = 25600
			// AND src & dst WERE DEFINED AS PULONG
			for ( i = 0 ; i < BUFLEN / 4 ; i ++ )
				*dst ++ = *src ++ ^ 0x0000FFFF ;*/

			for ( i = 0 ; i < BUFLEN / 2 ; i ++ )
				*dst ++ = ((int)(*src ++) * cos_tab[i]) >> FIX_SHIFT ;
				
			   waveOutWrite( hWaveO , wO , sizeof(WAVEHDR) ) ;
			waveInAddBuffer( hWaveI , wI , sizeof(WAVEHDR) ) ;
		}
	}
}

//-----------------------------------------------------------------------------
//
// SoundCapture_Open:
//
//  Here we make the setup for the characteristics of the captured sound.
//
void SoundCapture_Open ()
{
	WAVEFORMATEX waveFormat ;
	int    i ;
	double freq ;

	// Set up audio capture format...
	waveFormat.wFormatTag      = WAVE_FORMAT_PCM ;
	waveFormat.nChannels       = 1 ;
    waveFormat.nSamplesPerSec  = SOUNDFREQ ;
	waveFormat.wBitsPerSample  = BPSAMPLE * 8 ;
	waveFormat.nBlockAlign     = waveFormat.nChannels * BPSAMPLE ;
    waveFormat.nAvgBytesPerSec = ((long)waveFormat.nSamplesPerSec * waveFormat.nBlockAlign) ;

	canSound = (waveOutOpen( &hWaveO , WAVE_MAPPER , &waveFormat , 
							 0 , 0 , CALLBACK_NULL ) == MMSYSERR_NOERROR) &&
			   (waveInOpen(  &hWaveI , WAVE_MAPPER , &waveFormat , 
							 (ULONG) WaveStreamCallback , 
							 0 , CALLBACK_FUNCTION ) == MMSYSERR_NOERROR) ;

	if ( canSound )
	{
		for ( i = 0 ; i < BUFFERS ; i ++ )
		{
			waveIHeader[i].dwUser		  = (ULONG) &waveOHeader[i] ;
			waveIHeader[i].lpData         = bufferI[i] ;
			waveIHeader[i].dwBufferLength = BUFLEN ;
			waveIHeader[i].dwLoops        = 0 ;
			waveIHeader[i].dwFlags        = 0 ;

			waveOHeader[i].lpData         = bufferO[i] ;
			waveOHeader[i].dwBufferLength = BUFLEN ;
			waveOHeader[i].dwLoops        = 0 ;
			waveOHeader[i].dwFlags        = 0 ;

			canSound = canSound && 
						(waveInPrepareHeader(  hWaveI , &waveIHeader[i] , sizeof(WAVEHDR) ) 
							== MMSYSERR_NOERROR) &&
						(waveInAddBuffer(      hWaveI , &waveIHeader[i] , sizeof(WAVEHDR) ) 
							== MMSYSERR_NOERROR) &&
						(waveOutPrepareHeader( hWaveO , &waveOHeader[i] , sizeof(WAVEHDR) )
							== MMSYSERR_NOERROR) ;

			if ( !canSound )
				break ;

			waveOHeader[i].dwFlags |= WHDR_DONE ;
		}
	}

	if ( canSound )
	{
		freq = ((double)MODFREQ / (double)SOUNDFREQ * M_2PI) ;
		for ( i = 0 ; i < SOUNDFREQ ; i ++ )
			cos_tab[i] = (SHORT) (cos( freq * i ) * (double)(1 << FIX_SHIFT)) ;
	}
	else
	{
		MessageBox( hMainWnd , "Can't open wave device... Won't be able to decode audio" , appName , MB_ICONEXCLAMATION ) ;
		SoundCapture_Close() ;
		useSound = FALSE ;		
	}
}

//-----------------------------------------------------------------------------
//
// SoundCapture_Close:
//
//  Here we shut down the sound devices
//
void SoundCapture_Close ()
{
	int i ;

	if ( !canSound )
		return ;

/*	waveInReset( hWaveI ) ;
	waveOutReset( hWaveO ) ;*/

	SoundCapture_Stop() ;

/*	do
	{
		shit = TRUE ;
		for ( i = 0 ; i < BUFFERS ; i ++ )
			if ( waveOHeader[i].dwFlags == WHDR_INQUEUED )
			{
				shit = FALSE ;
				break ;
			}
	}
	while ( !shit ) ;*/

	for ( i = 0 ; i < BUFFERS ; i ++ )
	{
		 waveInUnprepareHeader( hWaveI , &waveIHeader[i] , sizeof(WAVEHDR) ) ;
		waveOutUnprepareHeader( hWaveO , &waveOHeader[i] , sizeof(WAVEHDR) ) ;
	}

	 waveInClose( hWaveI ) ;
	waveOutClose( hWaveO ) ;

	canSound = FALSE ;	
}

//-----------------------------------------------------------------------------
//
// SoundCapture_Play/Stop:
//
//  Here we make the ball roll/stop
//
void SoundCapture_Play ()
{
	if ( canSound )
		waveInStart( hWaveI ) ;
}

void SoundCapture_Stop ()
{
	if ( canSound )
	{
/*		 waveInReset( hWaveI ) ;
		waveOutReset( hWaveO ) ;*/
		  waveInStop( hWaveI ) ;

		Sleep( 500 ) ;   // For Sanity
	}
}
