#pragma warning(disable: 4100 4201 4514)

#include <windows.h>
#include <stdio.h>
#include <vfw.h>
#include <pbt.h>

#include "Vidcap.h"
#include "DirectDraw.h"
#include "YVUSyster.h"
#include "YVUSysterHQ.h"
#include "RGBSyster.h"
#include "RGBSysterHQ.h"
#include "Konfig.h"
#include "BT848.h"
#include "Debug.h"
#include "resource.h"

// Sound by Satan
#include "Nvsound.h"


HINSTANCE      hInstance         = NULL;
HWND           hWndMain          = NULL;
HWND           hWndCap           = NULL;
HACCEL         hAccel            = NULL;

DWORD          dwFirstLines      = 0;
LPVOID         pFirstLines       = NULL;

CBT848        *pBT848            = NULL;
CDirectDraw   *pDirectDraw       = NULL;
CKonfig       *pKonfig           = NULL;
CSyster       *pSyster           = NULL;

const char     pGerade[]         = "Gerade";
const char     pUngerade[]       = "Ungerade";
const char     pBT[]             = "BT";
const char     pExtension[]      = ".bnd";
const char     pHideDialog[]     = "Quick";
const char     pVFW[]            = "Video fr Windows starten";
const char     pIndex[]          = "Index erstellen";
const char     pError[]          = "Die Funktion %s() ist fehlgeschlagen.";

const char     pHelligkeit[]     = "Helligkeit %d";
const char     pFarbe[]          = "Farbe %d";
const char     pKontrast[]       = "Kontrast %d";
const char     pGespeichert[]    = "Kanal gespeichert";
const char     pStatuszeile[]    = "%02dfps  ";
const char     pDecode[]         = "Dekoder  ";
const char     pPALplus[]        = "PALplus  ";
const char     pFormat_4_3[]     = "4:3  ";
const char     pFormat_14_9[]    = "14:9  ";
const char     pFormat_16_9[]    = "16:9  ";

char           pStatus[20]       = "";
int            nStatusTime       = 0;
const int      STATUS_TIME       = 100;
const COLORREF STATUS_COLOR      = RGB(255, 255, 192);
const COLORREF BACKGROUND_COLOR  = RGB(0, 0, 0);

const int      VIDEO_BITCOUNT    = 16;
const DWORD    VIDEO_FORMAT      = MAKEFOURCC('Y','U','Y','2');

int            nFormat           = 0x00;
const int      FORMAT_MASK       = 0x0F;
const int      FORMAT_4_3        = 0x08;
const int      FORMAT_14_9       = 0x01;
const int      FORMAT_16_9       = 0x0B;

bool           bDecode           = true;
bool           bBild             = true;
bool           bPhase            = false;

int            nWidthMin         = 0;
int            nHeightMin        = 0;

int            nLineOrder[ZEILENZAHL];

// Sound by Satan
bool		   bSoundON			 = false;


bool Start(char*);
void Stop();
bool ReadBND();

LRESULT CALLBACK MainWnd(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK VideoStreamCallback(HWND, LPVIDEOHDR);
LRESULT CALLBACK ErrorCallback(HWND, int, LPSTR);


int WINAPI WinMain
(
   HINSTANCE hCurrInstance,
   HINSTANCE hPrevInstance,
   LPSTR     pCmdLine,
   int       nCmdShow
)
{
   hInstance = hCurrInstance;

   if(Start(pCmdLine) == false)
   {
      Stop();
      return false;
   }

   MSG sMsg;
   while(GetMessage(&sMsg, NULL, 0, 0))
   {
      if(FALSE == TranslateAccelerator(hWndMain, hAccel, &sMsg)) 
      {
         TranslateMessage(&sMsg);
         DispatchMessage(&sMsg);
      }
   }

   Stop();

   return sMsg.wParam;
}


bool Start
(
   char *pCmdLine
)
{
   ZeroMemory(nLineOrder, sizeof(nLineOrder));

   // Klasse zur Fehleranzeige
   DebugInit(hInstance);

   // Zugriff auf den BT
   pBT848 = new CBT848();
   if(pBT848 == NULL)
   {
      MessageBox(IDS_ERR_MEMORY);
      return false;
   }

   // DirectDraw starten
   pDirectDraw = new CDirectDraw(hInstance);
   if(pDirectDraw == NULL)
   {
      MessageBox(IDS_ERR_MEMORY);
      return false;
   }

   // Konfiguration erstellen
   pKonfig = new CKonfig(pDirectDraw);
   if(pKonfig == NULL)
   {
      MessageBox(IDS_ERR_MEMORY);
      return false;
   }

   // Soll der Dialog bersprungen werden ?
   bool bHideDialog = false;
   if(strstr(pCmdLine, pHideDialog) != NULL)
      bHideDialog = true;

   // Konfigurationsdialog ffnen
   if(pKonfig->DoModal(hInstance, pBT848->BTGefunden(), bHideDialog) != IDOK)
      return false;

   // Tunertype bergeben
   pBT848->SetzeTuner(pKonfig->GetTuner());

   // Kartentyp bergeben
   pBT848->SetzeTVKarte(pKonfig->GetTVKarte());

   // Bindfile laden
   if(ReadBND() == false)
      return false;

   // Auf Wunsch hohe Prioritt anmelden
   if(pKonfig->GetPriority() == true)
   {
      SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
      SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);
   }

   // Windowsklasse Registrieren
   WNDCLASS sWinClass;

   sWinClass.lpszClassName = pAppTitle;
   sWinClass.hInstance     = hInstance;
   sWinClass.lpfnWndProc   = MainWnd;
   sWinClass.hCursor       = NULL;
   sWinClass.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_FRED));
   sWinClass.lpszMenuName  = NULL;
   sWinClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
   sWinClass.style         = CS_HREDRAW | CS_VREDRAW;
   sWinClass.cbClsExtra    = 0;
   sWinClass.cbWndExtra    = 0;

   if(FALSE == RegisterClass(&sWinClass)) 
   {
      MessageBox(pError, "RegisterClass");
      return false;
   }

   // Tastaturaccelerator laden
   hAccel = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDA_MORETV));
   if(hAccel == NULL)
   {
      MessageBox(pError, "LoadAccelerators");
      return false;
   }

   // Fenstergre berechnen
   RECT  sRect;
   DWORD dwStyle;

   SetRect( &sRect,
            0,
            0,
            pKonfig->GetScreenWidth(),
            pKonfig->GetScreenHeight()
          );

   if(pKonfig->GetFensterMode() == false)
   {
      dwStyle = WS_VISIBLE | WS_OVERLAPPED | WS_MAXIMIZE;
   }
   else
   {
      if(pKonfig->GetStatistic() == true)
         dwStyle = WS_VISIBLE | WS_OVERLAPPEDWINDOW;
      else
         dwStyle = WS_VISIBLE | WS_POPUP | WS_THICKFRAME;

      AdjustWindowRectEx(&sRect, dwStyle, FALSE, WS_EX_TOPMOST);
   }

   nWidthMin = sRect.right-sRect.left;
   nHeightMin = sRect.bottom-sRect.top;

   // Hauptfenster ffnen
   hWndMain =
   CreateWindowEx( WS_EX_TOPMOST,
                   pAppTitle,
                   pAppTitle,
                   dwStyle,
                   CW_USEDEFAULT,
                   CW_USEDEFAULT,
                   nWidthMin,
                   nHeightMin,
                   HWND_DESKTOP,
                   NULL,
                   hInstance,
                   NULL
                 );

   if(hWndMain == NULL)
   {
      MessageBox(pError, "CreateWindowEx");
      return false;
   }

   // Direct X starten
   if(false == pDirectDraw->OpenDirectDraw( hWndMain,
                                            pKonfig->GetOverlay(),
                                            pKonfig->GetScreenWidth(),
                                            pKonfig->GetScreenHeight(),
                                            pKonfig->GetVideoWidth(),
                                            pKonfig->GetZeilenAusgabe(),
                                            pKonfig->GetFensterMode(),
                                            pKonfig->GetCooperativeLevel()
                                          )
     )
      return false;

   // Etwas Platz fr die Statistik lassen
   int nStatusHeight;

   if( pKonfig->GetStatistic() == true &&
       pKonfig->GetFensterMode() == false
     )
      nStatusHeight = 13;
   else
      nStatusHeight = 0;

   // Status ausgeben
   SetWindowText(hWndMain, pIndex);

   // Hier wird entschieden welches Bildschirmformat benutzt wird
   if(pKonfig->GetOverlay())
   {
      if(pKonfig->GetMMX())
      {
         pSyster = new CYVUSysterHQ( pKonfig->GetVideoWidth(),
                                     pKonfig->GetZeilenAusgabe(),
                                     pKonfig->GetTestZeilen(),
                                     pKonfig->GetPALplus(),
                                     pKonfig->GetSchwarzeZeile(),
                                     pKonfig->GetSecam()
                                   );
      }
      else
      {
         pSyster = new CYVUSyster( pKonfig->GetVideoWidth(),
                                   pKonfig->GetZeilenAusgabe(),
                                   pKonfig->GetTestZeilen(),
                                   pKonfig->GetPALplus(),
                                   pKonfig->GetSchwarzeZeile(),
                                   pKonfig->GetSecam()
                                 );
      }
   }
   else
   {
      if(pKonfig->GetMMX())
      {
         pSyster = new CRGBSysterHQ( pKonfig->GetVideoWidth(),
                                     pKonfig->GetZeilenAusgabe()-nStatusHeight,
                                     pKonfig->GetTestZeilen(),
                                     pDirectDraw->IsRGB15(),
                                     pKonfig->GetPALplus(),
                                     pKonfig->GetSchwarzeZeile(),
                                     pKonfig->GetSecam()
                                   );
      }
      else
      {
         pSyster = new CRGBSyster( pKonfig->GetVideoWidth(),
                                   pKonfig->GetZeilenAusgabe()-nStatusHeight,
                                   pKonfig->GetTestZeilen(),
                                   pDirectDraw->IsRGB15(),
                                   pKonfig->GetPALplus(),
                                   pKonfig->GetSchwarzeZeile(),
                                   pKonfig->GetSecam()
                                 );
      }
   }

   if(pSyster == NULL)
   {
      MessageBox(IDS_ERR_MEMORY);
      return false;
   }

   if(pSyster->FehlerStatus() == true)
      return false;


   // Status ausgeben
   SetWindowText(hWndMain, pVFW);

   // Capture initialisieren
   hWndCap = capCreateCaptureWindow(pAppTitle, WS_CHILD, 0, 0, 80, 80, hWndMain, 100);
   if(hWndCap == NULL)
   {
      MessageBox(pError, "capCreateCaptureWindow");
      return false;
   }

   if(FALSE == capDriverConnect(hWndCap, pKonfig->GetCaptureDevice()))
   {
      MessageBox(IDS_ERR_FIND_HARDWARE);
      return false;
   }

   if(FALSE == capOverlay(hWndCap, false))
   {
      MessageBox(pError, "capOverlay");
      return false;
   }

   if(FALSE == capPreview(hWndCap, false))
   {
      MessageBox(pError, "capPreview");
      return false;
   }

   if(FALSE == capSetCallbackOnVideoStream(hWndCap, VideoStreamCallback))
   {
      MessageBox(pError, "capSetCallbackOnVideoStream");
      return false;
   }

   if(FALSE == capSetCallbackOnError(hWndCap, ErrorCallback))
   {
      MessageBox(pError, "capSetCallbackOnError");
      return false;
   }

   CAPTUREPARMS sCapParms;

   if(FALSE == capCaptureGetSetup(hWndCap, &sCapParms, sizeof(CAPTUREPARMS)))
   {
      MessageBox(pError, "capCaptureGetSetup");
      return false;
   }

   sCapParms.dwRequestMicroSecPerFrame = 40000;
   sCapParms.fMakeUserHitOKToCapture   = FALSE;
   sCapParms.fAbortLeftMouse           = FALSE;
   sCapParms.fAbortRightMouse          = FALSE;
   sCapParms.fYield                    = TRUE;
   sCapParms.fCaptureAudio             = FALSE;

   if(FALSE == capCaptureSetSetup(hWndCap, &sCapParms, sizeof(CAPTUREPARMS)))
   {
      MessageBox(pError, "capCaptureSetSetup");
      return false;
   }

   // Buffer fr die letzten 32 Zeilen festlegen
   if(pKonfig->GetPALplus())
   {
      dwFirstLines = pKonfig->GetVideoWidth() * 32 * VIDEO_BITCOUNT / 8;

      pFirstLines = new char[dwFirstLines];
      if(pFirstLines == NULL)
      {
         MessageBox(IDS_ERR_MEMORY);
         return false;
      }

		ZeroMemory(pFirstLines, dwFirstLines);
   }

   // Eingabeformat festlegen
   BITMAPINFO sBitmapInfo;

   sBitmapInfo.bmiHeader.biSize          = sizeof(BITMAPINFOHEADER);
   sBitmapInfo.bmiHeader.biWidth         = pKonfig->GetVideoWidth();
   sBitmapInfo.bmiHeader.biHeight        = pKonfig->GetVideoHeight();
   sBitmapInfo.bmiHeader.biBitCount      = VIDEO_BITCOUNT;
   sBitmapInfo.bmiHeader.biCompression   = VIDEO_FORMAT;
   sBitmapInfo.bmiHeader.biPlanes        = 1;
   sBitmapInfo.bmiHeader.biSizeImage     = sBitmapInfo.bmiHeader.biWidth    *
                                           sBitmapInfo.bmiHeader.biHeight   *
                                           sBitmapInfo.bmiHeader.biBitCount /
                                           8;
   sBitmapInfo.bmiHeader.biClrUsed       = 0;
   sBitmapInfo.bmiHeader.biXPelsPerMeter = 0;
   sBitmapInfo.bmiHeader.biYPelsPerMeter = 0;
   sBitmapInfo.bmiHeader.biClrImportant  = 0;

   if(FALSE == capSetVideoFormat(hWndCap, &sBitmapInfo, sizeof(sBitmapInfo)))
   {
      MessageBox(pError, "capSetVideoFormat");
      return false;
   }

   // Fenster positionieren
   if(pKonfig->GetFensterMode() == true)
   {
      SendMessage(hWndMain, WM_MOVE, 0, 0);
   }
   else
   {
      if(pKonfig->GetOverlay())
      {
         // Overlay aktivieren und positionieren
         RECT sRect;

         sRect.left   = 0;
         sRect.top    = 0;
         sRect.right  = pKonfig->GetScreenWidth()-1;
         sRect.bottom = pKonfig->GetScreenHeight()-1-nStatusHeight;
         if(pDirectDraw->UpdateOverlay(&sRect, DDOVER_SHOW, false) == false)
            return false;
      }
   }

   // Videostream Starten
   SendMessage(hWndMain, WM_COMMAND, MAKEWPARAM(IDM_START, 0), 0);

   // Sound by Satan
   if(pKonfig->GetSound()) {
     if (startAudioDecode(0,pKonfig->GetSoundFilter(),100,pKonfig->GetSoundMMX()) == 0)
		 bSoundON = true;
	 else {
		MessageBox(IDS_ERR_SOUND_INIT);
		return false;
	 }
   }

   return true;
}


void Stop()
{
 	// Sound by Satan
	if(bSoundON)
		stopAudioDecode();

	if(hWndCap != NULL)
   {
      capSetCallbackOnVideoStream(hWndCap, NULL);
      capSetCallbackOnError(hWndCap, NULL);

      // Disconnect the current capture driver
      capDriverDisconnect(hWndCap);
   }

   if(pFirstLines != NULL)
   {
      delete[] pFirstLines;
      pFirstLines = NULL;
   }

   if(pSyster != NULL)
      delete pSyster;

   if(pBT848 != NULL)
      delete pBT848;

   if(pKonfig != NULL)
      delete pKonfig;

   if(pDirectDraw != NULL)
      delete pDirectDraw;

   DebugEnd();
}


LRESULT CALLBACK VideoStreamCallback
(
   HWND        hWnd,
   LPVIDEOHDR  pVideoHeader
)
{
   bBild = !bBild;
   
   int  nIndex;
   int  nPALplusData   = 0;
   bool bKeyGefunden   = false;
   bool bDecodePicture = bDecode;

   // Array mit Sortierindex
   int pSortLines[ZEILENZAHL];

   // Array mit den Anfangsadressen der Zeilen. Es wird nur ein Halbbild verarbeitet
   int *pLinesAddress[ZEILENZAHL];

   if(bDecode == true)
   {
      if(pKonfig->GetPALplus())
      {
         // Adressen der Zeilen im 1. Halbbild bestimmen
         for(nIndex=0; nIndex<32; nIndex++)
            pLinesAddress[nIndex] = (int*)pFirstLines+nIndex*pKonfig->GetVideoWidth()/2;

         for(nIndex=32; nIndex<ZEILENZAHL; nIndex++)
            pLinesAddress[nIndex] = (int*)pVideoHeader->lpData+nLineOrder[nIndex]*pKonfig->GetVideoWidth()/2;
      }
      else
      {
         // Adressen der Zeilen im 2. Halbbild bestimmen
         for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
            pLinesAddress[nIndex] = (int*)pVideoHeader->lpData+nLineOrder[nIndex]*pKonfig->GetVideoWidth()/2;
      }

      // Sortierung finden
      bKeyGefunden = pSyster->FindeZeilenFolge( pLinesAddress,
                                                pSortLines,
                                                &nPALplusData
                                              );

      // Wenn die PALplusZeile verschwindet, Dekodierung ausschalten
      if( pKonfig->GetPALplus() &&
          bKeyGefunden == false
        )
      {
         bDecodePicture = false;
      }
   }

   // Adressen der Zeilen fr den uncodierten Modus bestimmen
   if(bDecodePicture == false)
   {
      // Adressen der Zeilen bestimmen
      for(nIndex=0; nIndex<ZEILENZAHL; nIndex++)
         pLinesAddress[nIndex] = ((int*)pVideoHeader->lpData+nIndex*pKonfig->GetVideoWidth());
   }

   // Sobald sich etwas ndert, neues Format whlen
   if(nFormat != (nPALplusData & FORMAT_MASK))
   {
      nFormat = (nPALplusData & FORMAT_MASK);

      switch(nFormat)
      {
         case FORMAT_16_9:
            pKonfig->SetErsteZeile(36);
            pKonfig->SetLetzteZeile(ZEILENZAHL-36);
         break;

         case FORMAT_14_9:
            pKonfig->SetErsteZeile(20);
            pKonfig->SetLetzteZeile(ZEILENZAHL-20);
         break;

         default:
            pKonfig->SetErsteZeile(2);
            pKonfig->SetLetzteZeile(ZEILENZAHL-2);
      }

      pDirectDraw->EraseSurface();
   }

   // Surface Locken
   void *pSurface;
   int   nStartOfNextLine;

   if(pDirectDraw->LockSurface(&pSurface, &nStartOfNextLine, pKonfig->GetAustastung()) == true)
   {
      // Bild ausgeben
      pSyster->DecodeColor( pLinesAddress,
                            pSortLines,
                            (int*)pSurface,
                            nStartOfNextLine,
                            pKonfig->GetErsteZeile(),
                            pKonfig->GetLetzteZeile(),
                            bBild,
                            bDecodePicture,
                            bPhase
                          );

      // Surface wieder freigeben
      pDirectDraw->UnlockSurface();
   }

   if(pKonfig->GetPALplus())
   {
      // Die ersten 32 Zeilen fr das nchste Bild retten
      for(nIndex=0; nIndex<32; nIndex++)
         CopyMemory((short*)pFirstLines+nIndex*pKonfig->GetVideoWidth(), (int*)pVideoHeader->lpData+nLineOrder[nIndex]*pKonfig->GetVideoWidth()/2, pKonfig->GetVideoWidth()*2);
   }

   // Statusinformationen ausgeben
   if(pKonfig->GetStatistic() == true)
   {
      HDC         hDC;
      char        pText[40];
      int         nTickCount;
      static int  nFPS         = 0;
      static int  nFrame       = 0;
      static int  nLastTime    = 0;
      static char pOldText[40] = "";

      // FPS berechnen
      nFrame++;
      nTickCount = GetTickCount();

      if(nLastTime+1000 <= nTickCount)
      {
         nFPS      = nFrame;
         nFrame    = 0;
         nLastTime = nTickCount;
      }

      // Nach Ablauf der Zeit den normalen Status anzeigen
      if(nStatusTime > 0)
      {
         nStatusTime--;
         lstrcpy(pText, pStatus);
      }
      else
      {
         sprintf(pText, pStatuszeile, nFPS);

         if(bDecodePicture == true)
            strcat(pText, pDecode);

         // PALplus und Format ausgeben
         if( pKonfig->GetPALplus() &&
             bKeyGefunden == true
           )
         {
            lstrcat(pText, pPALplus);

            switch(nFormat)
            {
               case FORMAT_4_3:
                  lstrcat(pText, pFormat_4_3);
               break;

               case FORMAT_14_9:
                  lstrcat(pText, pFormat_14_9);
               break;

               case FORMAT_16_9:
                  lstrcat(pText, pFormat_16_9);
               break;
            }
         }
      }
      
      // Text nur ausgeben, wenn er sich gendert hat
      if(lstrcmp(pOldText, pText) != 0)
      {
         lstrcpy(pOldText, pText);

         if(pKonfig->GetFensterMode() == true)
         {
            SetWindowText(hWndMain, pText);
         }
         else
         {
            if(pDirectDraw->GetDC(&hDC) == true)
            {
               RECT sRect;

               // Hintergrund mit schwarz fllen
               SetRect(&sRect, 0, pKonfig->GetScreenHeight()-16, pKonfig->GetScreenWidth(), pKonfig->GetScreenHeight());
               FillRect(hDC, &sRect, (HBRUSH)GetStockObject(BLACK_BRUSH));

               // Status ausgeben
               SetBkColor(hDC, BACKGROUND_COLOR);
               SetTextColor(hDC, STATUS_COLOR);
               TextOut(hDC, 0, pKonfig->GetScreenHeight()-16, pText, strlen(pText));

               pDirectDraw->ReleaseDC(hDC);
            }
         }
      }
   }

   return 0;
}


LRESULT CALLBACK ErrorCallback
(
   HWND  hWnd, 
   int   nErrorID, 
   LPSTR pErrorText
)
{
   if(nErrorID == 0)
      return false;
   
   // Meldung unterdrcken:"Dieses Format kann nicht dargestellt werden"
   if(nErrorID != 439)
      MessageBox(pErrorText);

   return true;
}


// Reihenfolge der Zeilen aus Datei einlesen
bool ReadBND()
{
   int            nZlr;
   int            nInputValue;
   char           pBndDatei[80];
   FILE          *sFile;
   OSVERSIONINFO  sOSVersion;
   
   sOSVersion.dwOSVersionInfoSize = sizeof(sOSVersion);
   GetVersionEx(&sOSVersion);

   // Bei NT mssen die geraden mit den ungeraden getauscht werden
   if(sOSVersion.dwPlatformId == VER_PLATFORM_WIN32_NT)
   {
      if(pKonfig->GetPALplus())
         lstrcpy(pBndDatei, pUngerade);
      else
         lstrcpy(pBndDatei, pGerade);
   }
   else
   {
      if(pKonfig->GetPALplus())
         lstrcpy(pBndDatei, pGerade);
      else
         lstrcpy(pBndDatei, pUngerade);
   }

   if(pKonfig->GetOptimierungBT())
      lstrcat(pBndDatei, pBT);

   lstrcat(pBndDatei, pExtension);

   sFile = fopen(pBndDatei, "r");
   if(sFile == NULL)
   {
      MessageBox(IDS_ERR_READ_BND);
      return false;
   }

   for(nZlr=0; nZlr<ZEILENZAHL; nZlr++)
   {
      if(fscanf(sFile, "%d\n", &nInputValue) == EOF)
      {
			fclose(sFile);
         MessageBox(IDS_ERR_READ_BND);
         return false;
      }

      if(nInputValue < 0 || nInputValue >= pKonfig->GetVideoHeight())
      {
			fclose(sFile);
         MessageBox(IDS_ERR_READ_BND);
         return false;
      }

      nLineOrder[nZlr] = nInputValue;
   }

   fclose(sFile);
   return true;
}


LRESULT CALLBACK MainWnd
(
   HWND     hWnd,
   UINT     uMessage,
   WPARAM   wParam,
   LPARAM   lParam
)
{
   switch(uMessage) 
   {
      case WM_COMMAND:
         switch(LOWORD(wParam))
         {
            case IDM_EXIT:
               DestroyWindow(hWnd);
            break;

            case IDM_START:
					// Status ausgeben
					SetWindowText(hWndMain, pAppTitle);

               // VideoStreaming ohne zu speichern
               capCaptureSequenceNoFile(hWndCap);

               // Den BT etwas programmieren
               if(pKonfig->GetOptimierungBT())
               {
                  // Kanal einstellen
                  nStatusTime = STATUS_TIME;
                  pBT848->SetzeKanal(pKonfig->KanalLesen(0), pStatus);

                  // Einstellungen holen
                  pBT848->SetzeFarbe(pKonfig->FarbeLesen());
                  pBT848->SetzeHelligkeit(pKonfig->HelligkeitLesen());
                  pBT848->SetzeKontrast(pKonfig->KontrastLesen());

                  Sleep(500);

                  if(pKonfig->GetMMX())
                     pBT848->InitBTxxHQ();
                  else
                     pBT848->InitBTxx();
               }
            break;

            case IDM_BILD_KLEINER:
               pKonfig->SetErsteZeile(pKonfig->GetErsteZeile()+1);
               pKonfig->SetLetzteZeile(pKonfig->GetLetzteZeile()-1);
               pDirectDraw->EraseSurface();
            break;

            case IDM_BILD_GROESSER:
               pKonfig->SetErsteZeile(pKonfig->GetErsteZeile()-1);
               pKonfig->SetLetzteZeile(pKonfig->GetLetzteZeile()+1);
            break;

            case IDM_FORMAT_20_9:
               pKonfig->SetErsteZeile(60);
               pKonfig->SetLetzteZeile(ZEILENZAHL-60);
               pDirectDraw->EraseSurface();
            break;

            case IDM_FORMAT_16_9:
               pKonfig->SetErsteZeile(40);
               pKonfig->SetLetzteZeile(ZEILENZAHL-40);
               pDirectDraw->EraseSurface();
            break;

            case IDM_FORMAT_4_3:
               pKonfig->SetErsteZeile(2);
               pKonfig->SetLetzteZeile(ZEILENZAHL-2);
               pDirectDraw->EraseSurface();
            break;

            case IDM_BILD_SPEICHERN:
               pKonfig->FormatSpeichern();
            break;

            case IDM_DEKODER:
               bDecode = !bDecode;
            break;

            case IDM_PHASE:
               bPhase = !bPhase;
            break;

            case IDM_HELLIGKEIT_PLUS:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						sprintf(pStatus, pHelligkeit, pBT848->SetzeHelligkeit(pBT848->HoleHelligkeit()+1));
					}
            break;

            case IDM_HELLIGKEIT_MINUS:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						sprintf(pStatus, pHelligkeit, pBT848->SetzeHelligkeit(pBT848->HoleHelligkeit()-1));
					}
            break;

            case IDM_FARBE_PLUS:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						sprintf(pStatus, pFarbe, pBT848->SetzeFarbe(pBT848->HoleFarbe()+1));
					}
            break;

            case IDM_FARBE_MINUS:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						sprintf(pStatus, pFarbe, pBT848->SetzeFarbe(pBT848->HoleFarbe()-1));
					}
            break;

            case IDM_KONTRAST_PLUS:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						sprintf(pStatus, pKontrast, pBT848->SetzeKontrast(pBT848->HoleKontrast()+1));
					}
            break;

            case IDM_KONTRAST_MINUS:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						sprintf(pStatus, pKontrast, pBT848->SetzeKontrast(pBT848->HoleKontrast()-1));
					}
            break;

            case IDM_KANAL_PLUS:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						pBT848->SetzeKanal(pBT848->HoleKanal()+1, pStatus);
					}
            break;

            case IDM_KANAL_MINUS:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						pBT848->SetzeKanal(pBT848->HoleKanal()-1, pStatus);
					}
            break;

            case IDM_LESE_0:
            case IDM_LESE_1:
            case IDM_LESE_2:
            case IDM_LESE_3:
            case IDM_LESE_4:
            case IDM_LESE_5:
            case IDM_LESE_6:
            case IDM_LESE_7:
            case IDM_LESE_8:
            case IDM_LESE_9:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						pBT848->SetzeKanal(pKonfig->KanalLesen(LOWORD(wParam)-IDM_LESE_0), pStatus);
					}
            break;

            case IDM_SPEICHER_0:
            case IDM_SPEICHER_1:
            case IDM_SPEICHER_2:
            case IDM_SPEICHER_3:
            case IDM_SPEICHER_4:
            case IDM_SPEICHER_5:
            case IDM_SPEICHER_6:
            case IDM_SPEICHER_7:
            case IDM_SPEICHER_8:
            case IDM_SPEICHER_9:
					if(pKonfig->GetOptimierungBT())
					{
						nStatusTime = STATUS_TIME;
						lstrcpy(pStatus, pGespeichert);
						pKonfig->KanalSpeichern(LOWORD(wParam)-IDM_SPEICHER_0, pBT848->HoleKanal());
					}
            break;
         }
      break;

      // Bei einem Mouseclick ins Fenster, so tun als ob die CAPTION getroffen wurde
      case WM_NCHITTEST:
      {
         LRESULT hResult = DefWindowProc(hWnd, uMessage, wParam, lParam);

         if(hResult == HTCLIENT)
            return HTCAPTION;
         else
            return hResult;
      }
      break;

		// Im Vollbildmodus Rnder schwarz zeichnen
		case WM_NCPAINT:
		{
			HDC  hDC;
         RECT sRect;

         if(pKonfig->GetFensterMode() == true)
            return DefWindowProc(hWnd, uMessage, wParam, lParam);

			hDC = GetWindowDC(hWnd);

         // Hintergrund mit schwarz fllen
			GetWindowRect(hWnd, &sRect);
         FillRect(hDC, &sRect, (HBRUSH)GetStockObject(BLACK_BRUSH));

			ReleaseDC(hWnd, hDC);

			return 0;
		}
		break;

      case WM_SYSCOMMAND:
			switch(wParam&0xFFF0)
			{
				case SC_SIZE:
				case SC_MAXIMIZE:
					// Wenn sich das Overlay nicht skalieren lt, size verbieten
					if((pDirectDraw->GetCaps()&DDCAPS_OVERLAYSTRETCH) == 0)
						return 0;
				break;

				// Verhindern das der Bildschirmschoner luf oder der Monitor ausgeschaltet wird
				case SC_SCREENSAVE:
				case SC_MONITORPOWER:
					return 0;
				break;
			}

         return DefWindowProc(hWnd, uMessage, wParam, lParam);
      break;

      // Gre und Position des Fensters berwachen
      case WM_SIZING:
      case WM_MOVING:
      {
         LPRECT pRect  = (LPRECT)lParam;
         int    nXSize = GetSystemMetrics(SM_CXSCREEN);
         int    nYSize = GetSystemMetrics(SM_CYSCREEN);

         if(pRect->left < 0)
            pRect->left = 0;

         if(pRect->top < 0)
            pRect->top = 0;

         if(pRect->right >= nXSize)
            pRect->right = nXSize;

         if(pRect->bottom >= nYSize)
            pRect->bottom = nYSize;

         if(pRect->right - pRect->left <= nWidthMin)
         {
            if(pRect->left + nWidthMin < nXSize)
               pRect->right = pRect->left + nWidthMin;
            else
               pRect->left = pRect->right - nWidthMin;
         }

         if(pRect->bottom - pRect->top <= nHeightMin)
         {
            if(pRect->top + nHeightMin < nYSize)
               pRect->bottom = pRect->top + nHeightMin;
            else
               pRect->top = pRect->bottom - nHeightMin;
         }

         return TRUE;
      }
      break;

      // Overlay neu positionieren
      case WM_SIZE:
      case WM_MOVE:
         if(pKonfig->GetFensterMode() == false)
            return 0;

         if(IsIconic(hWnd))
         {
            pDirectDraw->UpdateOverlay(NULL, DDOVER_HIDE, false);
         }
         else
         {
            RECT sRect;

            GetClientRect(hWnd, &sRect);
            ClientToScreen(hWnd, (POINT*)&sRect.left);
            ClientToScreen(hWnd, (POINT*)&sRect.right);
            pDirectDraw->UpdateOverlay(&sRect, DDOVER_SHOW, true);
         }
      break;

      // Verhindern das das System in den Suspendmode geht
      case WM_POWERBROADCAST:
         if(wParam == PBT_APMQUERYSUSPEND)
            return BROADCAST_QUERY_DENY;
      break;

		// Fenster zum Schlieen vorbereiten
      case WM_DESTROY:
         // Capture beenden falls es noch luft
         capCaptureStop(hWndCap);

         // Einstellungen speichern
			if(pKonfig->GetOptimierungBT())
			{
				pKonfig->FarbeSpeichern(pBT848->HoleFarbe());
				pKonfig->HelligkeitSpeichern(pBT848->HoleHelligkeit());
				pKonfig->KontrastSpeichern(pBT848->HoleKontrast());

				// Alles wieder zurcksetzen
            pBT848->ResetBT();
			}

         // Warten bis die letzten Bilder verarbeitet sind
         Sleep(500);

			// Und Tsch
         PostQuitMessage(0);
      break;

      default:
         return DefWindowProc(hWnd, uMessage, wParam, lParam);
   }

   return 0;
}
