#pragma warning(disable: 4056 4100 4201 4514)

#include <windows.h>

#include "YVUSyster.h"
#include "Debug.h"
#include "resource.h"


CYVUSyster::CYVUSyster
(
   int   nBreiteQuelle,
   int   nHoeheZiel,
   int   nTestzeilen,
   bool  bPalPlus,
   bool  bSchwarzeZeile,
   bool  bSecam
) : CSyster(nTestzeilen, bSchwarzeZeile)
{
   m_nBreiteQuelle = nBreiteQuelle;
   m_bPALplus      = bPalPlus;

   m_nHoeheZiel    = nHoeheZiel;
   m_bSecam        = bSecam;
}


CYVUSyster::~CYVUSyster()
{
}


void CYVUSyster::DecodeColor
(
   int **pLinesAddress,
   int  *pSortLines,
   int  *pDestination,
   int   nBreiteZiel,
   int   nErsteSichtbareZeile,
   int   nLetzteSichtbareZeile,
   bool  bBild,
   bool  bDecode,
   bool  bPhase
)
{
   int    nPhase;
   int    nPixel;
   int    nZeileZiel;
   int    nZeileQuelle;
   int    nLastPhase     = 4;
   int    nLastColorLine = 0;
   DWORD *pZiel;
   DWORD *pQuelle;
   DWORD *pColorQuelle;

   //
   // Die Daten kommen im YUY2 Format:
   //
   //  31--V0--24 23--Y1--16 15--U0--8 7--Y0--0
   //  MSB                                  LSB
   //
   // U0 und V0 gelten jeweils fr Y0 und Y1
   //

   // Zeilen sortieren
   for(nZeileZiel=nErsteSichtbareZeile; nZeileZiel<nLetzteSichtbareZeile; nZeileZiel++)
   {
      pZiel = (DWORD*)&pDestination[nZeileZiel*nBreiteZiel/4];

      if(bDecode == false)
      {
         nPhase       = 0;
         pQuelle      = (DWORD*)pLinesAddress[nZeileZiel];
         pColorQuelle = (DWORD*)pLinesAddress[nLastColorLine];
      }
      else
      {
         nZeileQuelle = pSortLines[nZeileZiel] - 32;
         pQuelle      = (DWORD*)pLinesAddress[pSortLines[nZeileZiel]];
         pColorQuelle = (DWORD*)pLinesAddress[pSortLines[nLastColorLine]];

         //
         // Secam Decodierung von Xetra
         //
         if(m_bSecam)
         {
            DWORD nSumU = 0;
            DWORD nSumV = 0;
            DWORD nThreshold = (DWORD)((0x0f>>1)+1.40)*(m_nBreiteQuelle/2);

            nPhase = 0;

            for(nPixel=0; nPixel<m_nBreiteQuelle; nPixel+=2)
            {  
               nSumU += (*pQuelle>>12 & 0x0f);
               nSumV += (*pQuelle>>28 & 0x0f);
               pQuelle++;
            }

            pQuelle = (DWORD*)pLinesAddress[pSortLines[nZeileZiel]];

            if((nSumU>nThreshold || nSumV>nThreshold) && nLastPhase != 4)
               nPhase =4;
         }
         else
         {
            if(nZeileQuelle > 0)
            {
               nPhase = (nZeileZiel-nZeileQuelle)&0x03;
            }
            else
            {
               // Bei den ersten 32 Zeilen Farbe invertieren. Bei PALplus kommt das 
               // erste Halbbild (+1), sonst das zweite (+2).
               if(m_bPALplus)
                  nPhase = (nZeileZiel+1-nZeileQuelle)&0x03;
               else
                  nPhase = (nZeileZiel+2-nZeileQuelle)&0x03;
            }

            // Wenn der Abstand zu gro wird die Farbe aus der letzten gltigen Zeile holen
            if(nZeileZiel-nZeileQuelle > 40)
               nPhase = 4;
         }
      }

      // Farbe korrigieren
      switch(nPhase)
      {
         case 0:
            nLastPhase     = nPhase;
            nLastColorLine = nZeileZiel;

            // v = v, u = u
            for(nPixel=0; nPixel<m_nBreiteQuelle; nPixel+=2)
            {
               *pZiel = *pQuelle;

               pZiel++;
               pQuelle++;
            }
         break;

         case 2:
            nLastPhase     = nPhase;
            nLastColorLine = nZeileZiel;

            // v = -v, u = -u
            for(nPixel=0; nPixel<m_nBreiteQuelle; nPixel+=2)
            {
               *pZiel = *pQuelle^0xff00ff00;

               pZiel++;
               pQuelle++;
            }
         break;

         default:
            switch(nLastPhase)
            {
               case 0:
                  // v = v, u = u
                  for(nPixel=0; nPixel<m_nBreiteQuelle; nPixel+=2)
                  {
                     *pZiel = (*pQuelle&0x00ff00ff) | (*pColorQuelle&0xff00ff00);

                     pZiel++;
                     pQuelle++;
                     pColorQuelle++;
                  }
               break;

               case 2:
                  // v = -v, u = -u
                  for(nPixel=0; nPixel<m_nBreiteQuelle; nPixel+=2)
                  {
                     *pZiel = (*pQuelle&0x00ff00ff) | ((*pColorQuelle&0xff00ff00)^0xff00ff00);

                     pZiel++;
                     pQuelle++;
                     pColorQuelle++;
                  }
               break;

               default:
                  // Alle andern Zeilen ohne Farbe anzeigen
                  for(nPixel=0; nPixel<m_nBreiteQuelle; nPixel+=2)
                  {
                     *pZiel = (*pQuelle&0x00ff00ff) | 0x80008000;

                     pZiel++;
                     pQuelle++;
                  }
            }
         break;
      }
   }
}