#pragma warning(disable: 4056 4201 4514)

#include <windows.h>
#include <math.h>

#include "YVUSysterHQ.h"
#include "Debug.h"
#include "resource.h"


const int SIZE_FARBKORREKTUR = 345;


CYVUSysterHQ::CYVUSysterHQ
(
   int   nBreiteQuelle,
   int   nHoeheZiel,
   int   nTestzeilen,
   bool  bPalPlus,
   bool  bSchwarzeZeile,
   bool  bInterpolation
) : CSyster(nTestzeilen, bSchwarzeZeile, bInterpolation)
{
   m_nBreiteQuelle  = nBreiteQuelle;
   m_bPALplus       = bPalPlus;

   m_nHoeheZiel     = nHoeheZiel;

   m_pCromaPuffer   = NULL;
   m_pFarbkorrektur = NULL;

   // Speicher fr die Farbe einer Zeile
   m_pCromaPuffer = new char[m_nBreiteQuelle*2];
   if(m_pCromaPuffer == NULL)
   {
      MessageBox(IDS_ERR_MEMORY);
      m_bError = true;
      return;
   }

	// Speicher fr die Farbkorrekturtabelle
   m_pFarbkorrektur = new short[SIZE_FARBKORREKTUR*4*4];
   if(m_pFarbkorrektur == NULL)
   {
      MessageBox(IDS_ERR_MEMORY);
      m_bError = true;
      return;
   }

   BaueFarbkorrekturTabelle();
}


CYVUSysterHQ::~CYVUSysterHQ()
{
   if(m_pCromaPuffer != NULL)
      delete[] m_pCromaPuffer;

   if(m_pFarbkorrektur != NULL)
      delete[] m_pFarbkorrektur;
}


void CYVUSysterHQ::BaueFarbkorrekturTabelle()
{
   int     nZeilenAbstand;
   double  dPhase;
   short  *pTabelle;

   // Das sind die -90 Grad Phasendrehung pro Zeile plus eine 
   // Schwingung Offset pro Vollbild. 
   const double dPIHalbe  = -1.56074323;
   const double dFarbton  =  0.321699087;
   const double dUVFaktor =  0.708661417;

   pTabelle = m_pFarbkorrektur;

   for(nZeilenAbstand=0; nZeilenAbstand<SIZE_FARBKORREKTUR; nZeilenAbstand++)
   {
      dPhase = (double)(nZeilenAbstand-(SIZE_FARBKORREKTUR-1))*dPIHalbe + dFarbton;

      *pTabelle++ = (short)( cos(dPhase) * 8192.0);
      *pTabelle++ = (short)( sin(dPhase) * 8192.0);
      *pTabelle++ = (short)(-sin(dPhase) * 8192.0 * dUVFaktor);
      *pTabelle++ = (short)( cos(dPhase) * 8192.0 * dUVFaktor);

      // nQuelleInvers
      *pTabelle++ = (short)( cos(dPhase) * 8192.0);
      *pTabelle++ = (short)(-sin(dPhase) * 8192.0);
      *pTabelle++ = (short)(-sin(dPhase) * 8192.0 * dUVFaktor);
      *pTabelle++ = (short)(-cos(dPhase) * 8192.0 * dUVFaktor);

      // nZielInvers
      *pTabelle++ = (short)( cos(dPhase) * 8192.0);
      *pTabelle++ = (short)( sin(dPhase) * 8192.0);
      *pTabelle++ = (short)( sin(dPhase) * 8192.0 * dUVFaktor);
      *pTabelle++ = (short)(-cos(dPhase) * 8192.0 * dUVFaktor);

      // nQuelleInvers + nZielInvers
      *pTabelle++ = (short)( cos(dPhase) * 8192.0);
      *pTabelle++ = (short)(-sin(dPhase) * 8192.0);
      *pTabelle++ = (short)( sin(dPhase) * 8192.0 * dUVFaktor);
      *pTabelle++ = (short)( cos(dPhase) * 8192.0 * dUVFaktor);
   }
}


void CYVUSysterHQ::DecodeColor
(
   int **pLinesAddress,
   int  *pSortLines,
   int  *pDestination,
   int   nBreiteZiel,
   int   nErsteSichtbareZeile,
   int   nLetzteSichtbareZeile,
   bool  bBild,
   bool  bDecode,
   bool  bPhase
)
{
   int    nZielInvers;
   int    nQuelleInvers;
   int    nAbstandLetztesHalbbild;
   int    nZeileZlr;
   int    nZeileZiel;
   int    nZeileQuelle;
   DWORD *pZiel;
   DWORD *pQuelle;
   short *pColorFaktor;

   // Linebuffer lschen
   ZeroMemory(m_pCromaPuffer, m_nBreiteQuelle*2);

   // Zeilen sortieren
   for(nZeileZlr=nErsteSichtbareZeile; nZeileZlr<nLetzteSichtbareZeile; nZeileZlr++)
   {
      pZiel = (DWORD*)&pDestination[nZeileZlr*nBreiteZiel/4];

      if(bDecode == false)
      {
         // Korrekturfaktor bestimmen
         pColorFaktor = m_pFarbkorrektur+312*4*4;

         // Adresse der Zeile holen
         pQuelle = (DWORD*)pLinesAddress[nZeileZlr];
      }
      else
      {
         {
            // Abhngig ob es sich ums erste oder zweite Halbild handelt
            if(m_bPALplus)
               nAbstandLetztesHalbbild = 25;
            else
               nAbstandLetztesHalbbild = 26;

            // Hier ist die Zeile bei der Ausgabe
            nZeileZiel = nZeileZlr;

            // Hier ist die Zeile im Decodierten Bild
            nZeileQuelle = pSortLines[nZeileZiel];

            // Adresse der Zeile holen
            pQuelle = (DWORD*)pLinesAddress[nZeileQuelle];

            // Kommt die Zeile aus dem letzten Halbbild
            if(nZeileQuelle < 32)
               nZeileQuelle -= nAbstandLetztesHalbbild;

            // Ist die Zielzeile invers
            nZielInvers = nZeileZiel & 1;

            // Ist die Quellzeile invers
            nQuelleInvers = nZeileQuelle & 1;

            // Da waren ja noch die 90 Phasendrehung des Farbtrgers nach
            // jedem Bild. Nach vier Bildern fngt alles wieder von vorne an.
            if(bBild == bPhase)
            {
               nQuelleInvers ^= 1;
               nZielInvers   ^= 1;
            }

            // Abstand der Zeilen
            nZeileQuelle -= nZeileZiel;

            // Korrekturfaktor bestimmen
            pColorFaktor = m_pFarbkorrektur+((nZeileQuelle+312)*4+nQuelleInvers+nZielInvers*2)*4;
         }
      }

      ZeileAusgeben(pQuelle, pZiel, pColorFaktor);
   }
}


void inline CYVUSysterHQ::ZeileAusgeben
(
   DWORD *pQuelle,
   DWORD *pZiel,
   short *pColorFaktor
)
{
   int   nBreiteQuelle = m_nBreiteQuelle;
   char *pCromaPuffer  = m_pCromaPuffer;

   const __int64 nMaskeY  = 0x00FF00FF00FF00FF;
   const __int64 nMaskeUV = 0xFF00FF00FF00FF00;
   const __int64 nPlus128 = 0x8000800080008000;

   // Die Daten kommen im YUY2 Format:
   //
   //  0xFF000000 0x00FF0000 0x000FF00 0x0000FF
   //  31--V0--24 23--Y1--16 15--U0--8 7--Y0--0
   //  MSB                                  LSB
   //
   // U0 und V0 gelten jeweils fr Y0 und Y1
   //
   _asm
   {
      push edi                         // Register retten

      mov edi,dword ptr[pColorFaktor]
      mov edx,pQuelle
      mov ecx,nBreiteQuelle
      movq mm7,qword ptr[edi]          // mm7 = *pColorFaktor
      mov edi,dword ptr[pCromaPuffer]
      movq mm4,qword ptr[nMaskeY]      // mm4 = 0x00FF00FF00FF00FF
      movq mm5,qword ptr[nMaskeUV]     // mm5 = 0xFF00FF00FF00FF00
      movq mm6,qword ptr[nPlus128]     // mm6 = 0x8000800080008000
      shr ecx,2                        // nBreiteQuelle /= 4
      mov ebx,pZiel

   align 16

   NaechsterPixel:
      movq mm0,qword ptr[edx]          // mm0 = *pQuelle 
      add edx,8                        // pQuelle += 8

      movq mm2,mm0                     // mm2 = V2 Y3 U2 Y2 V0 Y1 U0 Y0  Y nach mm2
      psubb mm0,mm6                    // mm0 = V2 Y3 U2 Y2 V0 Y1 U0 Y0  U-128 V-128
      pand mm2,mm4                     // mm2 = 00 Y3 00 Y2 00 Y1 00 Y0  Y Ausmaskieren
      psraw mm0,8                      // mm0 = V2 V2 U2 U2 V0 V0 U0 U0  U und V auf 16 Bit erweitern
      movq mm1,mm0                     // mm1 = V2 V2 U2 U2 V0 V0 U0 U0  Zweites Pixelpaar nach mm1
      punpckldq mm0,mm0                // mm0 = V0 V0 U0 U0 V0 V0 U0 U0  Untere 32 Bit in die oberen 32 Bit kopieren
      punpckhdq mm1,mm1                // mm1 = V2 V2 U2 U2 V2 V2 U2 U2  Obere 32 Bit in die unteren 32 Bit kopieren
      pmaddwd mm0,mm7                  // mm0 = v0 v0 v0 v0 u0 u0 u0 u0  u0 = U0 * *(pColorFaktor+0) + V0 * *(pColorFaktor+1);
                                       //                                v0 = U0 * *(pColorFaktor+2) + V0 * *(pColorFaktor+3);
      pmaddwd mm1,mm7                  // mm1 = v2 v2 v2 v2 u2 u2 u2 u2  u2 = U2 * *(pColorFaktor+0) + V2 * *(pColorFaktor+1);
                                       //                                v2 = U2 * *(pColorFaktor+2) + V2 * *(pColorFaktor+3);
      psrad mm0,14                     // mm0 = v0 v0 v0 v0 u0 u0 u0 u0  Geteilt durch 16384
      psrad mm1,14                     // mm1 = v2 v2 v2 v2 u2 u2 u2 u2  Geteilt durch 16384
      packssdw mm0,mm1                 // mm0 = v2 v2 u2 u2 v0 v0 u0 u0  Ergebnis auf 16 bit krzen
      movq mm3,qword ptr[edi+8*ecx-8]  // mm3 = v2 v2 u2 u2 v0 v0 u0 u0  Farbwerte der letzten Zeile
      movq qword ptr[edi+8*ecx-8],mm0  // mm0 = v2 v2 u2 u2 v0 v0 u0 u0  Farbe fr nchste Zeile merken
      paddsw mm0,mm3                   // mm0 = v2 v2 u2 u2 v0 v0 u0 u0  Farbwerte der letzten Zeile addieren
      packsswb mm0,mm0                 // mm0 = v2 u2 v0 u0 v2 u2 v0 u0  Auf 8 Bit krzen
      punpcklbw mm0,mm0                // mm0 = v2 v2 u2 u2 v0 v0 u0 u0
      pand mm0,mm5                     // mm0 = v2 00 u2 00 v0 00 u0 00  U und V ausmaskieren
      psubb mm0,mm6                    // mm0 = v2 00 u2 00 v0 00 u0 00  U+128 V+128
      por mm2,mm0                      // mm2 = v2 Y3 u2 Y2 v0 Y1 u0 Y0

      movq qword ptr[ebx],mm2          // *pZiel = mm2
      add ebx,8                        // pZiel += 8

      dec ecx                          // nBreiteQuelle--
      jne near NaechsterPixel          // Sind noch weitere Pixel in dieser Zeile

      emms                             // MMX zurcksetzen

      pop edi                          // Register zurckspeichern
   }
}
