//---------------------------------------------------------------------------
// FileName     : debugout.cpp
// Created      : 20/06/94
// Project      : Some projects
// Author       : Elli
// Contents     : Output debug messages to system debug API
// Note         : This program is free software; you can redistribute it
//                and/or modify it under the terms of the GNU General Public
//                License as published by the Free Software Foundation;
//                either version 2 of the License, or (at your option) any
//                later version.
//
//                This program is distributed in the hope that it will be useful,
//                but WITHOUT ANY WARRANTY; without even the implied warranty of
//                MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//                GNU General Public License for more details.
//
//                You should have received a copy of the GNU General Public
//                License along with this program; if not, write to the Free
//                Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// History      :
//                V 1.00 initial version
//---------------------------------------------------------------------------

#include "debugout.h"
#include <stdio.h>
#include <stdarg.h>
#include <string.h>



void WINAPI debugOutputProc(PCHAR lpszMessage);

#define timeStringLength            10

static int               debugOption        = 0xFFFFFFFFL;
static int               debugIndent        = 0;
static int               deviceNameLength   = 0;
static char              debugDeviceName[9] = {0};
static int               nameOffset         = 2;
static int               dwTimerStart       = 0;
static int               dwTimerTick        = 0;
static PDebugOutHandler  debugHandler       = debugOutputProc;

static int               bWin95             = 0;
static int               bWin98             = 0;

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
ULONG WINAPI
debugGetSystemTime(void)
 {
  #if defined (WIN32)

    return( GetTickCount() );

  #elif defined (_NTKERNEL_)

    LARGE_INTEGER systemTime;

    KeQuerySystemTime(&systemTime);           // time in 1000ns
    systemTime =  RtlExtendedLargeIntegerDivide(systemTime, 10000,NULL);

    return( systemTime.u.LowPart );

  #elif defined (WIN95)
   return getSystemTime();
  #else
    return dwTimerTick++;
  #endif
 }


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void
traceOut(ULONG debugLevel, PCHAR format, ... )
 {
  ULONG        i;
  va_list      argp;
  char         szMessage[160];
  char         szSystemTime[15];
  ULONG        offset;

  if ( ! (debugOption & debugLevel) ) return;

  if ( debugLevel & dbFunctionEnter) debugIndent++;

  sprintf(szSystemTime,"%9lu ",debugGetSystemTime() - dwTimerStart);

  sprintf(szMessage,"%8s: %s                   ",debugDeviceName,szSystemTime);
  offset = nameOffset + timeStringLength + debugIndent;

  i = strlen(szMessage);
  if ( i < offset) offset = i;

  va_start(argp,format);
  vsprintf(&szMessage[offset],format,argp);
  va_end(argp);

  if ( debugLevel & dbFunctionExit)
   {
    debugIndent--;
    if ( debugIndent < 0) debugIndent = 0;
   }

  strcat(szMessage,"\n");

  debugHandler(szMessage);
 }



//----------------------------------------------------------------------------
//    dumphex without sprintf
//----------------------------------------------------------------------------
void WINAPI
dumpHex(PCHAR data, int size)
 {
  int   i,j,k;
  const char hexDigit[17]="0123456789ABCDEF";
  char  szDebug[80];

  i = 0;
  memset(&szDebug[0],' ',8);

  while (i  < size)
   {
    j = 0;
    k = i;
    while ( (j < 16) && ( k < size) )
     {
      szDebug[8+(j*3)]=hexDigit[(data[k]>>4) & 0xF];
      szDebug[9+(j*3)]=hexDigit[data[k] & 0x0f];
      szDebug[10+(j*3)]=' ';
      j++;
      k++;
     }

    while  (j < 16)
     {
      memset( &szDebug[8+(j*3)],' ',3);
      j++;
     }

    szDebug[8+(j*3)]  = ' ';
    szDebug[9+(j*3)]  = '-';
    szDebug[10+(j*3)] = ' ';
    j = 0;
    k = i;
    while ( (j < 16) && ( k < size))
     {
      if ( data[k] > 31)
        szDebug[59+j]=data[k];
       else
        szDebug[59+j]='.';

      j++;
      k++;
     }

    szDebug[59+j++]='\r';
    szDebug[59+j++]='\n';
    szDebug[59+j++]='\0';

    debugHandler( szDebug );

    i += 16;
   }
  }


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void WINAPI
debugOutputProc(PCHAR lpszMessage)
 {
  #if defined (WIN32) || defined (_Windows)       // Windows 32/16 mode

   OutputDebugString (lpszMessage);

  #elif defined (_NTKERNEL_)

   DbgPrint("%s",lpszMessage);

  #elif defined (WIN95)

   Out_Debug_String(lpszMessage);

  #endif
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void WINAPI
debugInitialize(PCHAR devName, ULONG dwDebugLevel)
 {
  debugOption = dwDebugLevel & 0x7FFFFFFF;         // ignore function enter/exit

  strncpy(debugDeviceName,devName,sizeof(debugDeviceName));
  nameOffset      = 10;

  debugResetTimeBase();

  #if defined (WIN32)
   OSVERSIONINFO ov;

   ov.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   if (::GetVersionEx( &ov))
    {
     if (ov.dwPlatformId == VER_PLATFORM_WIN32_NT)
      {
       debugOut(dbTrace,"Windows NT %d.%d  build %d %s",
          ov.dwMajorVersion, ov.dwMinorVersion, ov.dwBuildNumber,ov.szCSDVersion);
      }
     else if (ov.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
      {
       bWin95  = TRUE;
       if ( (ov.dwMinorVersion >= 10) && ( (LOWORD(ov.dwBuildNumber) >= 1998) ))
        {
         bWin98 = TRUE;
         debugOut(dbTrace,"Windows 98 build %d %s",LOWORD(ov.dwBuildNumber),ov.szCSDVersion);
        }
       else
        {
         debugOut(dbTrace,"Windows 95 build %d %s",LOWORD(ov.dwBuildNumber),ov.szCSDVersion);
        }
      }
     else
      {
       debugOut(dbTrace,"Windows 3.1");
      }
    }
  #endif
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void WINAPI
debugResetTimeBase(void)
 {
  dwTimerStart = debugGetSystemTime();
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void WINAPI
debugSetOutputHandler(PDebugOutHandler  newDebugHandler)
 {
  if ( ! newDebugHandler )
    {
     debugHandler = debugOutputProc;
    }
   else
    {
     debugHandler = newDebugHandler;
    }
 }






