//---------------------------------------------------------------------------
// FileName     : hwiodrv.cpp
// Created      : 19/11/98
// Project      : Windows kernel device driver for hardware IO
// Author       : Elli
// Contents     : Output debug messages to system debug API
// Note         : This program is free software; you can redistribute it
//                and/or modify it under the terms of the GNU General Public
//                License as published by the Free Software Foundation;
//                either version 2 of the License, or (at your option) any
//                later version.
//
//                This program is distributed in the hope that it will be useful,
//                but WITHOUT ANY WARRANTY; without even the implied warranty of
//                MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//                GNU General Public License for more details.
//
//                You should have received a copy of the GNU General Public
//                License along with this program; if not, write to the Free
//                Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// History      :
//                V 1.00 initial version
//---------------------------------------------------------------------------

#include <drvclass.h>
#include <hwiodef.h>
#include <debugout.h>

class CHWIODriver : public CKernelDriver
 {
  public:
    CHWIODriver();


   DWORD hwioStartDriver          (void);
   DWORD hwioStopDriver           (void);

   BOOL  isDriverOpen             (void) { return bDriverRunning; }

   BYTE  readPort                 (DWORD dwAddress);
   WORD  readPortW                (DWORD dwAddress);
   DWORD readPortL                (DWORD dwAddress);
   void  writePort                (DWORD dwAddress, DWORD dwValue);
   void  writePortW               (DWORD dwAddress, DWORD dwValue);
   void  writePortL               (DWORD dwAddress, DWORD dwValue);
   DWORD readMappedMemory         (DWORD dwAddress);
   void  writeMappedMemory        (DWORD dwAddress, DWORD dwValue);

   DWORD mapPhysicalToLinear      (DWORD dwAddress, DWORD dwLength);
   void  memoryUnmap              (DWORD dwAddress, DWORD dwLength);

   DWORD allocMemory              (
                                   DWORD  dwLength,
                                   DWORD  dwFlags,
                                   PDWORD pdwUserAddress,
                                   PDWORD pdwSystemAddress
                                  );
   DWORD freeMemory               (DWORD dwUserAddress);

   PVOID mapPhysicalToRing3       (DWORD dwAddress, DWORD dwLength);
   DWORD pciGetHardwareResources  (
                                   DWORD  dwVendorID,
                                   DWORD  dwDeviceID,
                                   PDWORD pdwMemoryAddress,
                                   PDWORD pdwMemoryLength,
                                   PDWORD pdwIrqNumber
                                  );

  protected:
    BOOL    bWindows95;
    BOOL    bDriverRunning;
 };


CHWIODriver hwioDriver;


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
int WINAPI
isDriverOpened (void)
 {
  return hwioDriver.isDriverOpen();
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
BYTE   WINAPI
readPort(WORD address)
 {
  return hwioDriver.readPort( (DWORD) address);
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
WORD   WINAPI
readPortW(WORD address)
 {
  return hwioDriver.readPortW( (DWORD) address);
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD   WINAPI
readPortL(WORD address)
 {
  return hwioDriver.readPortL( (DWORD) address);
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void WINAPI
writePort(WORD address, BYTE bValue)
 {
  hwioDriver.writePort( (DWORD) address, (DWORD) bValue);
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void WINAPI
writePortW(WORD address, WORD uValue)
 {
  hwioDriver.writePortW( (DWORD) address, (DWORD) uValue);
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void WINAPI
writePortL(WORD address, DWORD dwValue)
 {
  hwioDriver.writePortL( (DWORD) address, (DWORD) dwValue);
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD WINAPI
memoryMap(DWORD address, DWORD dwLength)
 {
  return hwioDriver.mapPhysicalToLinear(address, dwLength);
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void WINAPI
memoryUnmap(DWORD address, DWORD dwLength)
 {
  hwioDriver.memoryUnmap(address, dwLength);
 }



//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void WINAPI
memoryWrite(DWORD dwAddress, DWORD dwValue)
 {
  hwioDriver.writeMappedMemory( dwAddress, dwValue);
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD WINAPI
memoryRead(DWORD dwAddress)
 {
  return hwioDriver.readMappedMemory( dwAddress );
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
PVOID WINAPI
memoryMapEx(DWORD address, DWORD dwLength)
 {
  return hwioDriver.mapPhysicalToRing3(address, dwLength);
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD   WINAPI
memoryAlloc(
            DWORD  dwLength,
            DWORD  dwFlags,
            PDWORD pdwUserAddress,
            PDWORD pdwSystemAddress
           )
 {
  return hwioDriver.allocMemory(dwLength,dwFlags, pdwUserAddress, pdwSystemAddress);
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD  WINAPI
memoryFree(DWORD dwUserAddress)
 {
  return hwioDriver.freeMemory(dwUserAddress);
 }



//---------------------------------------------------------------------------
// Get back the PCI hardware resources, we assumes that only mapped memory
// is avail.
//---------------------------------------------------------------------------
DWORD  WINAPI
pciGetHardwareResources(
                        DWORD   dwVendorID,
                        DWORD   dwDeviceID,
                        PDWORD  pdwMemoryAddress,
                        PDWORD  pdwMemoryLength,
                        PDWORD  pdwIrqNumber
                       )
 {
  return hwioDriver.pciGetHardwareResources(
                                            dwVendorID,
                                            dwDeviceID,
                                            pdwMemoryAddress,
                                            pdwMemoryLength,
                                            pdwIrqNumber
                                           );
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
CHWIODriver::CHWIODriver(void) :
 CKernelDriver()
 {
  OSVERSIONINFO ov;

  ov.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx( &ov);
  bWindows95 = (ov.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS);

  bDriverRunning = FALSE;
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
BYTE
CHWIODriver::readPort(DWORD dwAddress)
 {
  THWIOParam  param;
  DWORD       dwReturnedLength;

  param.dwAddress = dwAddress;
  deviceControl(
                ioctlReadBYTE,
                &param,
                sizeof(param.dwAddress),
                &param.dwValue,
                sizeof(param.dwValue),
                &dwReturnedLength
               );

  return (BYTE) param.dwValue;
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
WORD
CHWIODriver::readPortW(DWORD dwAddress)
 {
  THWIOParam  param;
  DWORD       dwReturnedLength;

  param.dwAddress = dwAddress;
  deviceControl(
                ioctlReadWORD,
                &param,
                sizeof(param.dwAddress),
                &param.dwValue,
                sizeof(param.dwValue),
                &dwReturnedLength
               );

  return (WORD) param.dwValue;
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
CHWIODriver::readPortL(DWORD dwAddress)
 {
  THWIOParam  param;
  DWORD       dwReturnedLength;

  param.dwAddress = dwAddress;
  deviceControl(
                ioctlReadDWORD,
                &param,
                sizeof(param.dwAddress),
                &param.dwValue,
                sizeof(param.dwValue),
                &dwReturnedLength
               );

  return param.dwValue;
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void
CHWIODriver::writePort(DWORD dwAddress, DWORD dwValue)
 {
  THWIOParam  param;

  param.dwAddress = dwAddress;
  param.dwValue   = dwValue;

  deviceControl(
                ioctlWriteBYTE,
                &param,
                sizeof(param)
               );
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void
CHWIODriver::writePortW(DWORD dwAddress, DWORD dwValue)
 {
  THWIOParam  param;

  param.dwAddress = dwAddress;
  param.dwValue   = dwValue;

  deviceControl(
                ioctlWriteWORD,
                &param,
                sizeof(param)
               );
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void
CHWIODriver::writePortL(DWORD dwAddress, DWORD dwValue)
 {
  THWIOParam  param;

  param.dwAddress = dwAddress;
  param.dwValue   = dwValue;

  deviceControl(
                ioctlWriteDWORD,
                &param,
                sizeof(param)
               );
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
CHWIODriver::mapPhysicalToLinear(DWORD dwAddress, DWORD dwLength)
 {
  THWIOParam  param;
  DWORD       dwMappedAddress;
  DWORD       dwReturnedLength;

  param.dwAddress = dwAddress;
  param.dwValue   = dwLength;
  dwMappedAddress = 0;

  deviceControl(
                ioctlMapMemory,
                &param,
                sizeof(param),
                &dwMappedAddress,
                sizeof(dwMappedAddress),
                &dwReturnedLength
               );

  return dwMappedAddress;
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void
CHWIODriver::memoryUnmap(DWORD dwAddress, DWORD dwLength)
 {
  THWIOParam  param;

  param.dwAddress = dwAddress;
  param.dwValue   = dwLength;

  deviceControl(
                ioctlUnmapMemory,
                &param,
                sizeof(param)
               );

 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
PVOID
CHWIODriver::mapPhysicalToRing3(DWORD dwAddress, DWORD dwLength)
 {
  THWIOParam  param;
  DWORD       dwMappedAddress;
  DWORD       dwReturnedLength;

  param.dwAddress = dwAddress;
  param.dwValue   = dwLength;
  dwMappedAddress = 0;

  deviceControl(
                ioctlMapMemoryEx,
                &param,
                sizeof(param),
                &dwMappedAddress,
                sizeof(dwMappedAddress),
                &dwReturnedLength
               );

  return (PVOID) dwMappedAddress;
 }




//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void
CHWIODriver::writeMappedMemory(DWORD dwAddress, DWORD dwValue)
 {
  THWIOParam  param;

  param.dwAddress = dwAddress;
  param.dwValue   = dwValue;

  deviceControl(
                ioctlWriteMappedMemory,
                &param,
                sizeof(param)
               );
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
CHWIODriver::readMappedMemory(DWORD dwAddress)
 {
  THWIOParam  param;
  DWORD       dwReturnedLength;
  DWORD       dwValue;

  param.dwAddress = dwAddress;
  deviceControl(
                ioctlReadMappedMemory,
                &param,
                sizeof(param.dwAddress),
                &dwValue,
                sizeof(dwValue),
                &dwReturnedLength
               );

  return dwValue;
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
CHWIODriver:: allocMemory(
                          DWORD  dwLength,
                          DWORD  dwFlags,
                          PDWORD pdwUserAddress,
                          PDWORD pdwSystemAddress
                         )
 {
  THWIOParam  paramIn;
  THWIOParam  paramOut;
  DWORD       dwReturnedLength;
  DWORD       status;

  paramIn.dwAddress = dwLength;
  paramIn.dwValue   = dwFlags;
  status            = deviceControl(
                                   ioctlAllocMemory,
                                   &paramIn,
                                   sizeof(paramIn.dwAddress),
                                   &paramOut,
                                   sizeof(paramOut),
                                   &dwReturnedLength
                                  );

  *pdwUserAddress   = paramOut.dwAddress;
  *pdwSystemAddress = paramOut.dwValue;

  return status;
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
CHWIODriver::freeMemory(DWORD dwUserAddress)
 {
  THWIOParam  param;

  param.dwAddress = dwUserAddress;

  return deviceControl(
                        ioctlFreeMemory,
                        &param.dwAddress,
                        sizeof(param.dwAddress)
                       );

 }
















//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
CHWIODriver::pciGetHardwareResources(
                                     DWORD  dwVendorID,
                                     DWORD  dwDeviceID,
                                     PDWORD pdwMemoryAddress,
                                     PDWORD pdwMemoryLength,
                                     PDWORD pdwIrqNumber
                                    )
 {
  PCI_COMMON_CONFIG  pciConfig;
  THWIOParam         hwParam;
  DWORD              dwStatus;
  DWORD              dwLength;

  if ( ! isDriverOpened() ) return ERROR_ACCESS_DENIED;

  hwParam.dwAddress = dwVendorID;
  hwParam.dwValue   = dwDeviceID;

  dwStatus          = deviceControl(
                                     ioctlGetPCIInfo,
                                     &hwParam,
                                     sizeof(hwParam),
                                     &pciConfig,
                                     sizeof(pciConfig),
                                     &dwLength
                                   );

  if ( dwStatus == ERROR_SUCCESS)
   {
    //
    // Ok, here is the simplification, we use only the first address
    // For our current project this is enough (I hope so)
    //

    *pdwMemoryAddress  = pciConfig.u.type0.BaseAddresses[0] & 0xFFFFFFF0;
    *pdwMemoryLength   = 0x1000;
    *pdwIrqNumber      = pciConfig.u.type0.InterruptLine;
   }
  else
   {
    debugOut(dbTrace,"pciGetHardewareResource for %X %X failed",dwVendorID,dwDeviceID);

    *pdwMemoryAddress  = 0;
    *pdwMemoryLength   = 0;
    *pdwIrqNumber      = 0;
   }

  return dwStatus;
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
CHWIODriver::hwioStartDriver(void)
 {
  LPSTR  pszDriverName;
  DWORD  dwResult;
  LPSTR  pszName;
  char   szDriverPath[MAX_PATH];


  pszDriverName = ( bWindows95 ) ? "HWIO95.VXD" : "nthwio";


  if ( ! GetModuleFileName(
                            NULL,
                            szDriverPath,
                            sizeof(szDriverPath)
                              ) )
   {
    debugOut(dbError, "cannot get module file name");
   }


  pszName  = szDriverPath + strlen(szDriverPath);
  while ( *pszName != '\\' )
   {
    *pszName-- = 0;
   }


  dwResult = start( pszDriverName );
  if ( dwResult != ERROR_SUCCESS)
   {
    //
    // OK the driver service is not installed.
    // Build full driver path name and start the driver again.
    // We assume the driver.sys is located in the same path
    //

     strcat(szDriverPath,pszDriverName);
     strcat(szDriverPath,".sys");

     dwResult = start(pszDriverName, szDriverPath);

     if ( dwResult != ERROR_SUCCESS)
      {
       debugOut(dbError,"install driver %s failed",pszDriverName);
      }
   }


  if ( dwResult == ERROR_SUCCESS)
   {
    //
    // Its time to open the device
    //

    strcpy(szDriverPath,pszDriverName);

    dwResult = open(szDriverPath);
    if (  dwResult )
     {
      debugOut(dbError,"open driver failed");
     }
   }

  bDriverRunning = ( dwResult == ERROR_SUCCESS);

  return dwResult;
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
CHWIODriver::hwioStopDriver(void)
 {
  close();

  bDriverRunning = FALSE;

  return stop();
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
loadDriver(void)
 {
  return hwioDriver.hwioStartDriver(  );
 }


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
DWORD
closeDriver(void)
 {
  return hwioDriver.hwioStopDriver();
 }

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
extern "C" int
APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
 {
  switch (dwReason)
   {
    // DLL is attaching to a process, due to process initialization or a
    // call to LoadLibrary

    case DLL_PROCESS_ATTACH:
       debugInitialize("winhwio",0xFFFFFFFFL);

       debugOut(dbEnter,"DLL_PROCESS_ATTACH winhwio.dll, handle %X %s",hInstance, "1.0");

       loadDriver();

       debugOut(dbExit,"DLL_PROCCES_ATTACH, exit");
       break;

    case DLL_PROCESS_DETACH:
      debugOut(dbEnter,"DLL_PROCESS_DETACH, handle %X",hInstance);

      closeDriver();

      debugOut(dbExit,"DLL_PROCESS_DETACH, exit");
      break;
   }

  return 1;   // ok
 }



