//---------------------------------------------------------------------------
// FileName     : hwiodef.h
// Created      : 19/11/98
// Project      : Windows kernel device driver for hardware IO
// Author       : Elli
// Note         : This program is free software; you can redistribute it
//                and/or modify it under the terms of the GNU General Public
//                License as published by the Free Software Foundation;
//                either version 2 of the License, or (at your option) any
//                later version.
//
//                This program is distributed in the hope that it will be useful,
//                but WITHOUT ANY WARRANTY; without even the implied warranty of
//                MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//                GNU General Public License for more details.
//
//                You should have received a copy of the GNU General Public
//                License along with this program; if not, write to the Free
//                Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// History      :
//                V 1.00 initial version
//---------------------------------------------------------------------------

#if ! defined (__HWIODEF_H)
#define __HWIODEF_H

#include <basetype.h>


#if defined (WIN32)

#include <winioctl.h>

#elif defined(WIN95)



//
// Macro definition for defining IOCTL and FSCTL function control codes.  Note
// that function codes 0-2047 are reserved for Microsoft Corporation, and
// 2048-4095 are reserved for customers.
//

#define CTL_CODE( DeviceType, Function, Method, Access ) (                 \
    ((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) )

#define METHOD_BUFFERED                 0
#define METHOD_IN_DIRECT                1
#define METHOD_OUT_DIRECT               2
#define METHOD_NEITHER                  3

//
// Define the access check value for any access
//
//
// The FILE_READ_ACCESS and FILE_WRITE_ACCESS constants are also defined in
// ntioapi.h as FILE_READ_DATA and FILE_WRITE_DATA. The values for these
// constants *MUST* always be in sync.
//


#define FILE_ANY_ACCESS                 0
#define FILE_READ_ACCESS          ( 0x0001 )    // file & pipe
#define FILE_WRITE_ACCESS         ( 0x0002 )    // file & pipe


#elif defined (_NTKERNEL_)

extern "C" {

#include <devioctl.h>

}

//
// Extract transfer type
//

#define IOCTL_TRANSFER_TYPE( _iocontrol)   (_iocontrol & 0x3)


#endif

#ifdef __cplusplus
extern "C" {
#endif


#define FILE_DEVICE_HWIO   0x8001
#define HWIO_BASE          0x800

#define HWIO_READ_BYTE     (HWIO_BASE + 1)
#define HWIO_READ_WORD     (HWIO_BASE + 2)
#define HWIO_READ_DWORD    (HWIO_BASE + 3)

#define HWIO_WRITE_BYTE    (HWIO_BASE + 4)
#define HWIO_WRITE_WORD    (HWIO_BASE + 5)
#define HWIO_WRITE_DWORD   (HWIO_BASE + 6)

#define HWIO_MAP_MEMORY    (HWIO_BASE + 7)
#define HWIO_READ_MEMORY   (HWIO_BASE + 8)
#define HWIO_WRITE_MEMORY  (HWIO_BASE + 9)

#define HWIO_GET_PCI_INFO  (HWIO_BASE + 10)
#define HWIO_UNMAP_MEMORY  (HWIO_BASE + 11)

#define HWIO_ALLOC_MEMORY  (HWIO_BASE + 12)
#define HWIO_FREE_MEMORY   (HWIO_BASE + 13)

#define HWIO_MAP_MEMORY_EX (HWIO_BASE + 14)


//
// The wrapped control codes as required by the system
//

#define HWIO_CTL_CODE(function,method) CTL_CODE( FILE_DEVICE_HWIO,function,method,FILE_ANY_ACCESS)


#define ioctlReadBYTE          HWIO_CTL_CODE( HWIO_READ_BYTE,     METHOD_OUT_DIRECT)
#define ioctlReadWORD          HWIO_CTL_CODE( HWIO_READ_WORD,     METHOD_OUT_DIRECT)
#define ioctlReadDWORD         HWIO_CTL_CODE( HWIO_READ_DWORD,    METHOD_OUT_DIRECT)

#define ioctlWriteBYTE         HWIO_CTL_CODE( HWIO_WRITE_BYTE,    METHOD_IN_DIRECT)
#define ioctlWriteWORD         HWIO_CTL_CODE( HWIO_WRITE_WORD,    METHOD_IN_DIRECT)
#define ioctlWriteDWORD        HWIO_CTL_CODE( HWIO_WRITE_DWORD,   METHOD_IN_DIRECT)

#define ioctlMapMemory         HWIO_CTL_CODE( HWIO_MAP_MEMORY,    METHOD_BUFFERED)
#define ioctlReadMappedMemory  HWIO_CTL_CODE( HWIO_READ_MEMORY,   METHOD_OUT_DIRECT)
#define ioctlWriteMappedMemory HWIO_CTL_CODE( HWIO_WRITE_MEMORY,  METHOD_IN_DIRECT)
#define ioctlUnmapMemory       HWIO_CTL_CODE( HWIO_UNMAP_MEMORY,  METHOD_BUFFERED)
#define ioctlAllocMemory       HWIO_CTL_CODE( HWIO_ALLOC_MEMORY,  METHOD_OUT_DIRECT)
#define ioctlFreeMemory        HWIO_CTL_CODE( HWIO_FREE_MEMORY,   METHOD_IN_DIRECT)

#define ioctlGetPCIInfo        HWIO_CTL_CODE( HWIO_GET_PCI_INFO,  METHOD_OUT_DIRECT)
#define ioctlMapMemoryEx       HWIO_CTL_CODE( HWIO_MAP_MEMORY_EX, METHOD_BUFFERED)



typedef struct tagHWIOParam
 {
  ULONG   dwAddress;
  ULONG   dwValue;
  ULONG   dwFlags;
 } THWIOParam , * PHWIOParam;




//---------------------------------------------------------------------------
// This structure is taken from NTDDK.H, we use this only in WIN32 user mode
//---------------------------------------------------------------------------
#if defined (WIN32) || defined (WIN95)

typedef struct _PCI_COMMON_CONFIG
 {
    USHORT  VendorID;                   // (ro)
    USHORT  DeviceID;                   // (ro)
    USHORT  Command;                    // Device control
    USHORT  Status;
    UCHAR   RevisionID;                 // (ro)
    UCHAR   ProgIf;                     // (ro)
    UCHAR   SubClass;                   // (ro)
    UCHAR   BaseClass;                  // (ro)
    UCHAR   CacheLineSize;              // (ro+)
    UCHAR   LatencyTimer;               // (ro+)
    UCHAR   HeaderType;                 // (ro)
    UCHAR   BIST;                       // Built in self test

    union {
        struct _PCI_HEADER_TYPE_0 {
            ULONG   BaseAddresses[6];
            ULONG   CIS;
            USHORT  SubVendorID;
            USHORT  SubSystemID;
            ULONG   ROMBaseAddress;
            ULONG   Reserved2[2];

            UCHAR   InterruptLine;      //
            UCHAR   InterruptPin;       // (ro)
            UCHAR   MinimumGrant;       // (ro)
            UCHAR   MaximumLatency;     // (ro)
        } type0;


    } u;

    UCHAR   DeviceSpecific[192];

} PCI_COMMON_CONFIG, *PPCI_COMMON_CONFIG;


#if defined (WIN32)

//---------------------------------------------------------------------------
// The HWIO DLL application interface
//---------------------------------------------------------------------------

int    WINAPI isDriverOpened (void);

BYTE    WINAPI readPort      (WORD address);
WORD    WINAPI readPortW     (WORD address);
DWORD   WINAPI readPortL     (WORD address);
void    WINAPI writePort     (WORD address, BYTE bValue);
void    WINAPI writePortW    (WORD address, WORD uValue);
void    WINAPI writePortL    (WORD address, DWORD dwValue);

DWORD   WINAPI memoryMap     (DWORD address, DWORD dwLengt);
void    WINAPI memoryUnmap   (DWORD address, DWORD dwLengt);
void    WINAPI memoryWrite   (DWORD address, DWORD dwValue);
DWORD   WINAPI memoryRead    (DWORD address);


PVOID   WINAPI memoryMapEx   (DWORD address, DWORD dwLengt);

DWORD   WINAPI memoryAlloc   (
                              DWORD  dwLength,
                              DWORD  dwFlags,
                              PDWORD pdwUserAddress,
                              PDWORD pdwSystemAddress
                             );
DWORD   WINAPI memoryFree    (DWORD dwUserAddress);

DWORD  WINAPI pciGetHardwareResources(
                                      DWORD   dwVendorID,
                                      DWORD   dwDeviceID,
                                      PDWORD  pdwMemoryAddress,
                                      PDWORD  pdwMemoryLength,
                                      PDWORD  pdwIrqNumber
                                     );



typedef int   (WINAPI * PIsDriverOpened)(void);
typedef void  (WINAPI * PMemoryUnmap)   (DWORD dwAddress, DWORD dwLength);
typedef DWORD (WINAPI * PMemoryRead)    (DWORD dwAddress);
typedef DWORD (WINAPI * PMemoryWrite)   (DWORD dwAddress, DWORD dwValue);

typedef DWORD (WINAPI * PPCIGetHardwareResources)(
                                                  DWORD   dwVendorID,
                                                  DWORD   dwDeviceID,
                                                  PDWORD  pdwMemoryAddress,
                                                  PDWORD  pdwMemoryLength,
                                                  PDWORD  pdwIrqNumber
                                                 );


typedef DWORD  (WINAPI * PMemoryMap)     ( DWORD dwAddress, DWORD dwLength );
typedef void * (WINAPI * PMemoryMapEx)   ( DWORD dwAddress, DWORD dwLength );


typedef DWORD  (WINAPI * PMemoryAlloc)(
                                        DWORD  dwLength,
                                        DWORD  dwFlags,
                                        PDWORD pdwUserAddress,
                                        PDWORD pdwSystemAddress
                                       );

typedef DWORD  (WINAPI * PMemoryFree)(DWORD dwUserAddress);



#endif
#endif



#ifdef __cplusplus
 }
#endif

#endif



