//---------------------------------------------------------------------------
// FileName     : ioclass.h
// Created      : 19/11/98
// Project      : Windows kernel device driver for hardware IO
// Author       : Elli
// Note         : This program is free software; you can redistribute it
//                and/or modify it under the terms of the GNU General Public
//                License as published by the Free Software Foundation;
//                either version 2 of the License, or (at your option) any
//                later version.
//
//                This program is distributed in the hope that it will be useful,
//                but WITHOUT ANY WARRANTY; without even the implied warranty of
//                MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//                GNU General Public License for more details.
//
//                You should have received a copy of the GNU General Public
//                License along with this program; if not, write to the Free
//                Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// History      :
//                V 1.00 initial version
//                  1.01 alloc and free memory added
//---------------------------------------------------------------------------

#if ! defined (__IOCLASS_H)
#define __IOCLASS_H

#if ! defined (__HWIODEF_H)
#include <hwiodef.h>
#endif

#if defined (WIN95)


#include <vmmreg.h>
//#include <configmg.h>


#define PCI_ENUM_FUNC_GET_DEVICE_INFO  0


typedef DWORD DEVNODE;
typedef DWORD CONFIGRET;
typedef DWORD ENUMFUNC;
typedef DWORD HKEY, * PHKEY;

extern "C" {

int  __cdecl _inp(unsigned short);
unsigned short __cdecl _inpw(unsigned short);
unsigned long __cdecl _inpd(unsigned short);

int __cdecl _outp(unsigned short, int);
unsigned short __cdecl _outpw(unsigned short, unsigned short);
unsigned long __cdecl _outpd(unsigned short, unsigned long);


DWORD __cdecl _LinPageLock (DWORD HLinPgNum, DWORD nPages, DWORD Flags) ;
int  __cdecl  _LinPageUnLock (DWORD HLinPgNum, DWORD nPages, DWORD Flags) ;
//int  __cdecl  _PageFree (DWORD hMem, DWORD Flags) ;
//BOOL __cdecl _PageFree(HANDLE, PVOID, DWORD);
PVOID __cdecl _PageAllocate(HANDLE, DWORD, DWORD, ULONG, DWORD, DWORD, DWORD, PVOID*, DWORD);
void * __cdecl  _MapPhysToLinear(const void * PhysAddr, DWORD nBytes, DWORD Flags) ;
void * __cdecl _MapLinearToPhys(HANDLE, ULONG);

BOOL _CopyPageTable(HANDLE, ULONG, ULONG, PULONG, ULONG);

LONG __cdecl _RegCloseKey (HKEY hkey);
LONG __cdecl _RegOpenKey (HKEY hkey, PCHAR lpszSubKey, PHKEY phkResult) ;
LONG __cdecl _RegEnumKey (HKEY hkey, DWORD iSubKey, PCHAR lpszName, DWORD cchName) ;
LONG __cdecl _RegCloseKey (HKEY hkey) ;



CONFIGRET __cdecl _CONFIGMG_Locate_DevNode (DEVNODE * pNode, PCHAR szDevId, DWORD flags) ;
CONFIGRET __cdecl _CONFIGMG_Call_Enumerator_Function (DEVNODE dnDevNode, ENUMFUNC efFunc, DWORD RefData, void* pBuffer, DWORD ulBufferSize, DWORD ulFlags) ;

}

#define PIRP   PDIOCPARAMETERS


#pragma intrinsic(_outp)
#pragma intrinsic(_outpw)
#pragma intrinsic(_outpd)
#pragma intrinsic(_inp)
#pragma intrinsic(_inpw)
#pragma intrinsic(_inpd)


#define  osPortReadByte(port,data)       data=_inp(port)
#define  osPortReadWord(port,data)       data=_inpw(port)
#define  osPortReadLong(port,data)       data=_inpw(port)

#define  osPortWriteByte(port,data)      _outp(port,data)
#define  osPortWriteWord(port,data)      _outpw(port,data)
#define  osPortWriteLong(port,data)      _outpd(port,data)

#define  osMemoryRead(port,data)         data = *((PULONG)port)
#define  osMemoryWrite(port,data)        *((PULONG)port) = (ULONG)data

#define PAGENUM(p)      (((ULONG)(p)) >> 12)
#define PAGEOFF(p)      (((ULONG)(p)) & 0xFFF)
#define PAGEBASE(p)     (((ULONG)(p)) & ~0xFFF)
#define _NPAGES_(p, k)  ((PAGENUM((char*)p+(k-1)) - PAGENUM(p)) + 1)


#elif defined (_NTKERNEL_)

#define  osPortReadByte(port,data)    data = READ_PORT_UCHAR ( (PUCHAR) &port )
#define  osPortReadWord(port,data)    data = READ_PORT_USHORT( (PUSHORT) &port)
#define  osPortReadLong(port,data)    data = READ_PORT_ULONG( (PULONG) &port )
#define  osPortWriteByte(port,data)   WRITE_PORT_UCHAR ( (PUCHAR) &port, (UCHAR) data)
#define  osPortWriteWord(port,data)   WRITE_PORT_USHORT( (PUSHORT) &port, (USHORT) data)
#define  osPortWriteLong(port,data)   WRITE_PORT_ULONG ( (PULONG) &port, (ULONG) data)

#define  osMemoryRead(port,data)      data = READ_REGISTER_ULONG( (PULONG) port)
#define  osMemoryWrite(port,data)     WRITE_REGISTER_ULONG( (PULONG) port, (ULONG) data)

#endif


#define MAX_FREE_MEMORY_NODES   32

typedef struct tagMemoryNode
 {
  ULONG ulSystemAddress;
  ULONG ulUserAddress;
  PMDL  mdl;
 } TMemoryNode, * PMemoryNode;


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class CIOAccessDevice
 {
  public:
   CIOAccessDevice();
   ~CIOAccessDevice();

   NTSTATUS   deviceIOControl(IN PIRP   irp);

  protected:
   int         isValidAddress     (IN void *  pvAddress);
   NTSTATUS    deviceControl      (
                                   IN  ULONG       ioControlCode,
                                   IN  PHWIOParam  ioParam,
                                   OUT PULONG      outputBuffer,
                                   OUT PULONG      pulBytesWritten
                                  ) ;

   NTSTATUS    pciFindDevice      (
                                  IN   USHORT  vendorID,
                                  IN   USHORT  deviceID,
                                  OUT  PULONG  pdwBusNumber,
                                  OUT  PULONG  pdwSlotNumber
                                 );

   NTSTATUS    pciGetDeviceConfig(
                                 IN   ULONG              dwBusNumber,
                                 IN   ULONG              dwSlotNumber,
                                 OUT  PPCI_COMMON_CONFIG pciConfiguration
                                );

   ULONG       mapMemory        (ULONG ulBaseAddress, ULONG ulLength);
   ULONG       mapMemoryEx      (ULONG ulBaseAddress, ULONG ulLength);
   void        unmapMemory      (void);
   ULONG       mapToUserMode    (ULONG ulBaseAddress);

   ULONG       mapReadLong      (ULONG ulOffset);
   void        mapWriteLong     (ULONG ulOffset, ULONG ulValue);
   NTSTATUS    allocMemory      (
                                 ULONG    ulLength,
                                 PULONG   pulUserAddress,
                                 PULONG   pulSystemAddress
                                );

   NTSTATUS    freeMemory       (ULONG ulUserAddress);
   void        freeMemory       (PMemoryNode node );

  protected:
   ULONG          ulMemoryBase;            // mapped memory base
   ULONG          ulMappedMemoryLength;
   ULONG          ulBusNumber;
   ULONG          ulSlotNumber;

   ULONG          ulMemoryBaseEx;          // mapped memory base
   ULONG          ulMappedMemoryLengthEx;
   void *         virtualAddress;
   TMemoryNode    memoryList[MAX_FREE_MEMORY_NODES];
 };


#endif



